@@ -8,6 +8,7 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.deser.impl.ObjectIdReader;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.AccessPattern;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
@@ -51,14 +52,14 @@ public abstract class JsonDeserializer<T>
     /* Main deserialization methods
     /**********************************************************
      */
-    
+
     /**
      * Method that can be called to ask implementation to deserialize
      * JSON content into the value type this serializer handles.
      * Returned instance is to be constructed by method itself.
      *<p>
      * Pre-condition for this method is that the parser points to the
-     * first event that is part of value to deserializer (and which 
+     * first event that is part of value to deserializer (and which
      * is never JSON 'null' literal, more on this below): for simple
      * types it may be the only value; and for structured types the
      * Object start marker or a FIELD_NAME.
@@ -94,17 +95,24 @@ public abstract class JsonDeserializer<T>
      * fails, event that was not recognized or usable, which may be
      * the same event as the one it pointed to upon call).
      *<p>
-     * Note that this method is never called for JSON null literal,
-     * and thus deserializers need (and should) not check for it.
+     * <strong>Handling null values (JsonToken.VALUE_NULL)</strong>
+     * <br>
+     * : Note that this method is never called for the JSON {@code null} literal to avoid
+     * every deserializer from having to handle null values. Instead, the
+     * {@link JsonDeserializer#getNullValue(DeserializationContext)} method
+     * is called to produce a null value. To influence null handling,
+     * custom deserializers should override
+     * {@link JsonDeserializer#getNullValue(DeserializationContext)}
+     * and usually also {@link JsonDeserializer#getNullAccessPattern()}.
      *
-     * @param p Parsed used for reading JSON content
+     * @param p Parser used for reading JSON content
      * @param ctxt Context that can be used to access information about
      *   this deserialization activity.
      *
      * @return Deserialized value
      */
     public abstract T deserialize(JsonParser p, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException;
+        throws IOException, JacksonException;
 
     /**
      * Alternate deserialization method (compared to the most commonly
@@ -123,7 +131,7 @@ public abstract class JsonDeserializer<T>
      * update-existing-value operation (esp. immutable types)
      */
     public T deserialize(JsonParser p, DeserializationContext ctxt, T intoValue)
-        throws IOException
+        throws IOException, JacksonException
     {
         ctxt.handleBadMerge(this);
         return deserialize(p, ctxt);
@@ -140,14 +148,14 @@ public abstract class JsonDeserializer<T>
      * should not rely on current default implementation.
      * Implementation is mostly provided to avoid compilation errors with older
      * code.
-     * 
+     *
      * @param typeDeserializer Deserializer to use for handling type information
      */
     public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
             TypeDeserializer typeDeserializer)
-        throws IOException
+        throws IOException, JacksonException
     {
-        // We could try calling 
+        // We could try calling
         return typeDeserializer.deserializeTypedFromAny(p, ctxt);
     }
 
@@ -161,7 +169,7 @@ public abstract class JsonDeserializer<T>
      */
     public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
             TypeDeserializer typeDeserializer, T intoValue)
-        throws IOException
+        throws IOException, JacksonException
     {
         ctxt.handleBadMerge(this);
         return deserializeWithType(p, ctxt, typeDeserializer);
@@ -192,7 +200,7 @@ public abstract class JsonDeserializer<T>
      * delegate anything; or it does not want any changes), should either
      * throw {@link UnsupportedOperationException} (if operation does not
      * make sense or is not allowed); or return this deserializer as is.
-     * 
+     *
      * @since 2.1
      */
     public JsonDeserializer<?> replaceDelegatee(JsonDeserializer<?> delegatee) {
@@ -207,7 +215,7 @@ public abstract class JsonDeserializer<T>
      */
 
     /**
-     * Method for accessing type of values this deserializer produces.
+     * Method for accessing concrete physical type of values this deserializer produces.
      * Note that this information is not guaranteed to be exact -- it
      * may be a more generic (super-type) -- but it should not be
      * incorrect (return a non-related type).
@@ -215,11 +223,26 @@ public abstract class JsonDeserializer<T>
      * Default implementation will return null, which means almost same
      * same as returning <code>Object.class</code> would; that is, that
      * nothing is known about handled type.
-     *<p>
+     *
+     * @return Physical type of values this deserializer produces, if known;
+     *    {@code null} if not
+     *
      * @since 2.3
      */
     public Class<?> handledType() { return null; }
 
+    /**
+     * Method for accessing logical type of values this deserializer produces.
+     * Typically used for further configuring handling of values, for example,
+     * to find which coercions are legal.
+     *
+     * @return Logical type of values this deserializer produces, if known;
+     *    {@code null} if not
+     *
+     * @since 2.12
+     */
+    public LogicalType logicalType() { return null; }
+
     /**
      * Method called to see if deserializer instance is cachable and
      * usable for other properties of same type (type for which instance
@@ -241,10 +264,10 @@ public abstract class JsonDeserializer<T>
      * another deserializer for actual deserialization, by delegating
      * calls. If so, will return immediate delegate (which itself may
      * delegate to further deserializers); otherwise will return null.
-     * 
+     *
      * @return Deserializer this deserializer delegates calls to, if null;
      *   null otherwise.
-     * 
+     *
      * @since 2.1
      */
     public JsonDeserializer<?> getDelegatee() {
@@ -260,7 +283,7 @@ public abstract class JsonDeserializer<T>
      * This is only to be used for error reporting and diagnostics
      * purposes (most commonly, to accompany "unknown property"
      * exception).
-     * 
+     *
      * @since 2.0
      */
     public Collection<Object> getKnownPropertyNames() {
@@ -284,7 +307,7 @@ public abstract class JsonDeserializer<T>
      * {@link #getNullAccessPattern()} returns.
      *<p>
      * Default implementation simply returns null.
-     * 
+     *
      * @since 2.6 Added to replace earlier no-arguments variant
      */
     @Override
@@ -299,8 +322,9 @@ public abstract class JsonDeserializer<T>
      * to be called just once (static values), or each time empty value is
      * needed.
      *<p>
-     * Default implementation indicates that "null value" to use for input null
-     * is simply Java `null` for all deserializers, unless overridden by sub-classes.
+     * Default implementation indicates that the "null value" to use for input null
+     * does not vary across uses so that {@link #getNullValue(DeserializationContext)}
+     * need not be called more than once per deserializer instance.
      * This information may be used as optimization.
      */
     @Override
@@ -312,23 +336,33 @@ public abstract class JsonDeserializer<T>
     }
 
     /**
-     * This method may be called in conjunction with calls to
-     * {@link #getEmptyValue(DeserializationContext)}, to check whether it needs
-     * to be called just once (static values), or each time empty value is
-     * needed.
+     * Method called to determine placeholder value to be used for cases
+     * where no value was obtained from input but we must pass a value
+     * nonetheless: the common case is that of Creator methods requiring
+     * passing a value for every parameter.
+     * Usually this is same as {@link #getNullValue} (which in turn
+     * is usually simply Java {@code null}), but it can be overridden
+     * for specific types: most notable scalar types must use "default"
+     * values.
+     *<p>
+     * This method needs to be called every time a determination is made.
+     *<p>
+     * Default implementation simply calls {@link #getNullValue} and
+     * returns value.
      *
-     * @since 2.9
+     * @since 2.13
      */
-    public AccessPattern getEmptyAccessPattern() {
-        return AccessPattern.DYNAMIC;
+    @Override
+    public Object getAbsentValue(DeserializationContext ctxt) throws JsonMappingException {
+        return getNullValue(ctxt);
     }
 
     /*
     /**********************************************************
-    /* Other accessors
+    /* Accessors for other replacement/placeholder values
     /**********************************************************
      */
-    
+
     /**
      * Method called to determine value to be used for "empty" values
      * (most commonly when deserializing from empty JSON Strings).
@@ -349,6 +383,24 @@ public abstract class JsonDeserializer<T>
         return getNullValue(ctxt);
     }
 
+    /**
+     * This method may be called in conjunction with calls to
+     * {@link #getEmptyValue(DeserializationContext)}, to check whether it needs
+     * to be called just once (static values), or each time empty value is
+     * needed.
+     *
+     * @since 2.9
+     */
+    public AccessPattern getEmptyAccessPattern() {
+        return AccessPattern.DYNAMIC;
+    }
+
+    /*
+    /**********************************************************
+    /* Other accessors
+    /**********************************************************
+     */
+
     /**
      * Accessor that can be used to check whether this deserializer
      * is expecting to possibly get an Object Identifier value instead of full value
@@ -360,11 +412,11 @@ public abstract class JsonDeserializer<T>
      * {@link com.fasterxml.jackson.databind.deser.BeanDeserializer})
      * do implement this feature, and may return reader instance, depending on exact
      * configuration of instance (which is based on type, and referring property).
-     * 
+     *
      * @return ObjectIdReader used for resolving possible Object Identifier
      *    value, instead of full value serialization, if deserializer can do that;
      *    null if no Object Id is expected.
-     * 
+     *
      * @since 2.0
      */
     public ObjectIdReader getObjectIdReader() { return null; }
@@ -372,7 +424,7 @@ public abstract class JsonDeserializer<T>
     /**
      * Method needed by {@link BeanDeserializerFactory} to properly link
      * managed- and back-reference pairs.
-     * 
+     *
      * @since 2.2 (was moved out of <code>BeanDeserializerBase</code>)
      */
     public SettableBeanProperty findBackReference(String refName)
@@ -390,7 +442,7 @@ public abstract class JsonDeserializer<T>
      *<p>
      * Information gathered is typically used to either prevent merging update for
      * property (either by skipping, if based on global defaults; or by exception during
-     * deserialization construction if explicit attempt made) if {@link Boolean#FALSE}
+     * deserializer construction if explicit attempt made) if {@link Boolean#FALSE}
      * returned, or inclusion if {@link Boolean#TRUE} is specified. If "unknown" case
      * (<code>null</code> returned) behavior is to exclude property if global defaults
      * used; or to allow if explicit per-type or property merging is defined.
