@@ -5,8 +5,11 @@ import java.util.*;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
 import com.fasterxml.jackson.databind.deser.impl.*;
 import com.fasterxml.jackson.databind.deser.impl.ReadableObjectId.Referring;
+import com.fasterxml.jackson.databind.util.ClassUtil;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 import com.fasterxml.jackson.databind.util.TokenBuffer;
 
@@ -18,17 +21,6 @@ public class BeanDeserializer
     extends BeanDeserializerBase
     implements java.io.Serializable
 {
-    /* TODOs for future versions:
-     * 
-     * For 2.9?
-     *
-     * - New method in JsonDeserializer (deserializeNext()) to allow use of more
-     *   efficient 'nextXxx()' method `JsonParser` provides.
-     *
-     * Also: need to ensure efficient impl of those methods for Smile, CBOR
-     * at least (in addition to JSON)
-     */
-
     private static final long serialVersionUID = 1L;
 
     /**
@@ -55,14 +47,31 @@ public class BeanDeserializer
 
     /**
      * Constructor used by {@link BeanDeserializerBuilder}.
+     *
+     * @deprecated in 2.12, remove from 3.0
      */
+    @Deprecated
     public BeanDeserializer(BeanDeserializerBuilder builder, BeanDescription beanDesc,
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
             HashSet<String> ignorableProps, boolean ignoreAllUnknown,
             boolean hasViews)
     {
         super(builder, beanDesc, properties, backRefs,
-                ignorableProps, ignoreAllUnknown, hasViews);
+                ignorableProps, ignoreAllUnknown, null, hasViews);
+    }
+
+    /**
+     * Constructor used by {@link BeanDeserializerBuilder}.
+     *
+     * @since 2.12
+     */
+    public BeanDeserializer(BeanDeserializerBuilder builder, BeanDescription beanDesc,
+                            BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
+                            HashSet<String> ignorableProps, boolean ignoreAllUnknown, Set<String> includableProps,
+                            boolean hasViews)
+    {
+        super(builder, beanDesc, properties, backRefs,
+                ignorableProps, ignoreAllUnknown, includableProps, hasViews);
     }
 
     /**
@@ -85,10 +94,21 @@ public class BeanDeserializer
         super(src, oir);
     }
 
+    /**
+     * @deprecated in 2.12, remove from 3.0
+     */
+    @Deprecated
     public BeanDeserializer(BeanDeserializerBase src, Set<String> ignorableProps) {
         super(src, ignorableProps);
     }
 
+    /**
+     * @since 2.12
+     */
+    public BeanDeserializer(BeanDeserializerBase src, Set<String> ignorableProps, Set<String> includableProps) {
+        super(src, ignorableProps, includableProps);
+    }
+
     public BeanDeserializer(BeanDeserializerBase src, BeanPropertyMap props) {
         super(src, props);
     }
@@ -118,8 +138,14 @@ public class BeanDeserializer
     }
 
     @Override
-    public BeanDeserializer withIgnorableProperties(Set<String> ignorableProps) {
-        return new BeanDeserializer(this, ignorableProps);
+    public BeanDeserializer withByNameInclusion(Set<String> ignorableProps,
+            Set<String> includableProps) {
+        return new BeanDeserializer(this, ignorableProps, includableProps);
+    }
+
+    @Override
+    public BeanDeserializerBase withIgnoreAllUnknown(boolean ignoreUnknown) {
+        return new BeanDeserializer(this, ignoreUnknown);
     }
 
     @Override
@@ -158,7 +184,7 @@ public class BeanDeserializer
             }
             return deserializeFromObject(p, ctxt);
         }
-        return _deserializeOther(p, ctxt, p.getCurrentToken());
+        return _deserializeOther(p, ctxt, p.currentToken());
     }
 
     protected final Object _deserializeOther(JsonParser p, DeserializationContext ctxt,
@@ -181,8 +207,8 @@ public class BeanDeserializer
             case VALUE_NULL:
                 return deserializeFromNull(p, ctxt);
             case START_ARRAY:
-                // these only work if there's a (delegating) creator...
-                return deserializeFromArray(p, ctxt);
+                // these only work if there's a (delegating) creator, or UNWRAP_SINGLE_ARRAY
+                return _deserializeFromArray(p, ctxt);
             case FIELD_NAME:
             case END_OBJECT: // added to resolve [JACKSON-319], possible related issues
                 if (_vanillaProcessing) {
@@ -198,11 +224,6 @@ public class BeanDeserializer
         return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
-    @Deprecated // since 2.8; remove unless getting used
-    protected Object _missingToken(JsonParser p, DeserializationContext ctxt) throws IOException {
-        throw ctxt.endOfInputException(handledType());
-    }
-
     /**
      * Secondary deserialization method, called in cases where POJO
      * instance is created as part of deserialization, potentially
@@ -212,7 +233,7 @@ public class BeanDeserializer
     public Object deserialize(JsonParser p, DeserializationContext ctxt, Object bean) throws IOException
     {
         // [databind#631]: Assign current value, to be accessible by custom serializers
-        p.setCurrentValue(bean);
+        p.assignCurrentValue(bean);
         if (_injectables != null) {
             injectValues(ctxt, bean);
         }
@@ -232,7 +253,7 @@ public class BeanDeserializer
             }
         } else {
             if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-                propName = p.getCurrentName();
+                propName = p.currentName();
             } else {
                 return bean;
             }
@@ -275,10 +296,11 @@ public class BeanDeserializer
         throws IOException
     {
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
-        // [databind#631]: Assign current value, to be accessible by custom serializers
-        p.setCurrentValue(bean);
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            // [databind#631]: Assign current value, to be accessible by custom serializers
+            // [databind#4184]: but only if we have at least one property
+            p.assignCurrentValue(bean);
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 SettableBeanProperty prop = _beanProperties.find(propName);
@@ -312,7 +334,7 @@ public class BeanDeserializer
          */
         if ((_objectIdReader != null) && _objectIdReader.maySerializeAsObject()) {
             if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)
-                    && _objectIdReader.isValidReferencePropertyName(p.getCurrentName(), p)) {
+                    && _objectIdReader.isValidReferencePropertyName(p.currentName(), p)) {
                 return deserializeFromObjectId(p, ctxt);
             }
         }
@@ -324,9 +346,6 @@ public class BeanDeserializer
                 return deserializeWithExternalTypeId(p, ctxt);
             }
             Object bean = deserializeFromObjectUsingNonDefault(p, ctxt);
-            if (_injectables != null) {
-                injectValues(ctxt, bean);
-            }
             /* 27-May-2014, tatu: I don't think view processing would work
              *   at this point, so commenting it out; but leaving in place
              *   just in case I forgot something fundamental...
@@ -343,13 +362,24 @@ public class BeanDeserializer
         }
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
         // [databind#631]: Assign current value, to be accessible by custom deserializers
-        p.setCurrentValue(bean);
+        p.assignCurrentValue(bean);
+
+        // First: do we have native Object Ids (like YAML)?
         if (p.canReadObjectId()) {
             Object id = p.getObjectId();
             if (id != null) {
                 _handleTypedObjectId(p, ctxt, bean, id);
             }
         }
+        // [databind#3838]: since 2.16 Uniform handling of missing objectId
+        // only for the specific "empty JSON Object" case (and only for non-Native
+        // Object Ids, see [databind#4607]
+        else if (_objectIdReader != null && p.hasTokenId(JsonTokenId.ID_END_OBJECT)) {
+            // [databind#4610]: check if we are to skip failure
+            if (ctxt.isEnabled(DeserializationFeature.FAIL_ON_UNRESOLVED_OBJECT_IDS)) {
+                ctxt.reportUnresolvedObjectId(_objectIdReader, bean);
+            }
+        }
         if (_injectables != null) {
             injectValues(ctxt, bean);
         }
@@ -360,7 +390,7 @@ public class BeanDeserializer
             }
         }
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 SettableBeanProperty prop = _beanProperties.find(propName);
@@ -387,7 +417,6 @@ public class BeanDeserializer
      * as well.
      */
     @Override
-    @SuppressWarnings("resource")
     protected Object _deserializeUsingPropertyBased(final JsonParser p, final DeserializationContext ctxt)
         throws IOException
     {
@@ -396,17 +425,17 @@ public class BeanDeserializer
         TokenBuffer unknown = null;
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         List<BeanReferring> referrings = null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             // Object Id property?
-            if (buffer.readIdProperty(propName)) {
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
                 continue;
             }
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             if (creatorProp != null) {
                 // Last creator property to set?
                 Object value;
@@ -428,11 +457,11 @@ public class BeanDeserializer
                                 _creatorReturnedNullException());
                     }
                     // [databind#631]: Assign current value, to be accessible by custom serializers
-                    p.setCurrentValue(bean);
+                    p.assignCurrentValue(bean);
 
                     //  polymorphic?
                     if (bean.getClass() != _beanType.getRawClass()) {
-                        return handlePolymorphic(p, ctxt, bean, unknown);
+                        return handlePolymorphic(p, ctxt, p.streamReadConstraints(), bean, unknown);
                     }
                     if (unknown != null) { // nope, just extra unknown stuff...
                         bean = handleUnknownProperties(ctxt, bean, unknown);
@@ -444,7 +473,13 @@ public class BeanDeserializer
             }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
-            if (prop != null) {
+            // [databind#3724]: Special handling because Records' ignored creator props
+            // weren't removed (to help in creating constructor-backed PropertyCreator)
+            // so they ended up in _beanProperties, unlike POJO (whose ignored
+            // props are removed)
+            if ((prop != null) &&
+                // [databind#3938]: except if it's MethodProperty
+                (!_beanType.isRecordType() || (prop instanceof MethodProperty))) {
                 try {
                     buffer.bufferProperty(prop, _deserializeWithErrorWrapping(p, ctxt, prop));
                 } catch (UnresolvedForwardReference reference) {
@@ -461,7 +496,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -474,9 +509,18 @@ public class BeanDeserializer
                 }
                 continue;
             }
+
+            // 29-Mar-2021, tatu: [databind#3082] May skip collection if we know
+            //    they'd just get ignored (note: any-setter handled above; unwrapped
+            //    properties also separately handled)
+            if (_ignoreAllUnknown) {
+                // 22-Aug-2021, tatu: [databind#3252] must ensure we do skip the whole value
+                p.skipChildren();
+                continue;
+            }
             // Ok then, let's collect the whole field; name and value
             if (unknown == null) {
-                unknown = new TokenBuffer(p, ctxt);
+                unknown = ctxt.bufferForInputBuffering(p);
             }
             unknown.writeFieldName(propName);
             unknown.copyCurrentStructure(p);
@@ -485,11 +529,15 @@ public class BeanDeserializer
         // We hit END_OBJECT, so:
         Object bean;
         try {
-            bean =  creator.build(ctxt, buffer);
+            bean = creator.build(ctxt, buffer);
         } catch (Exception e) {
-            wrapInstantiationProblem(e, ctxt);
-            bean = null; // never gets here
+            return wrapInstantiationProblem(e, ctxt);
         }
+        // 13-Apr-2020, tatu: [databind#2678] need to handle injection here
+        if (_injectables != null) {
+            injectValues(ctxt, bean);
+        }
+
         if (referrings != null) {
             for (BeanReferring referring : referrings) {
                referring.setBean(bean);
@@ -497,8 +545,8 @@ public class BeanDeserializer
         }
         if (unknown != null) {
             // polymorphic?
-            if (bean.getClass() != _beanType.getRawClass()) {
-                return handlePolymorphic(null, ctxt, bean, unknown);
+            if (bean.getClass() != _beanType.getRawClass()) { // lgtm [java/dereferenced-value-may-be-null]
+                return handlePolymorphic(null, ctxt, p.streamReadConstraints(), bean, unknown);
             }
             // no, just some extra unknown properties
             return handleUnknownProperties(ctxt, bean, unknown);
@@ -527,9 +575,7 @@ public class BeanDeserializer
         try {
             return prop.deserialize(p, ctxt);
         } catch (Exception e) {
-            wrapAndThrow(e, _beanType.getRawClass(), prop.getName(), ctxt);
-            // never gets here, unless caller declines to throw an exception
-            return null;
+            return wrapAndThrow(e, _beanType.getRawClass(), prop.getName(), ctxt);
         }
     }
 
@@ -549,7 +595,7 @@ public class BeanDeserializer
         //   `VALUE_NULL` token.
         if (p.requiresCustomCodec()) { // not only XML module, but mostly it...
             @SuppressWarnings("resource")
-            TokenBuffer tb = new TokenBuffer(p, ctxt);
+            TokenBuffer tb = ctxt.bufferForInputBuffering(p);
             tb.writeEndObject();
             JsonParser p2 = tb.asParser(p);
             p2.nextToken(); // to point to END_OBJECT
@@ -562,6 +608,58 @@ public class BeanDeserializer
         return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
+    @Override
+    protected Object _deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException
+    {
+        // note: cannot call `_delegateDeserializer()` since order reversed here:
+        JsonDeserializer<Object> delegateDeser = _arrayDelegateDeserializer;
+        // fallback to non-array delegate
+        if ((delegateDeser != null) || ((delegateDeser = _delegateDeserializer) != null)) {
+            Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
+            if (_injectables != null) {
+                injectValues(ctxt, bean);
+            }
+            return bean;
+        }
+        final CoercionAction act = _findCoercionFromEmptyArray(ctxt);
+        final boolean unwrap = ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
+
+        if (unwrap || (act != CoercionAction.Fail)) {
+            JsonToken unwrappedToken = p.nextToken();
+            if (unwrappedToken == JsonToken.END_ARRAY) {
+                switch (act) {
+                case AsEmpty:
+                    return getEmptyValue(ctxt);
+                case AsNull:
+                case TryConvert:
+                    return getNullValue(ctxt);
+                default:
+                }
+                return ctxt.handleUnexpectedToken(getValueType(ctxt), JsonToken.START_ARRAY, p, null);
+            }
+            if (unwrap) {
+                // 23-Aug-2022, tatu: To prevent unbounded nested arrays, we better
+                //   check there is NOT another START_ARRAY lurking there..
+                if (unwrappedToken == JsonToken.START_ARRAY) {
+                    JavaType targetType = getValueType(ctxt);
+                    return ctxt.handleUnexpectedToken(targetType, JsonToken.START_ARRAY, p,
+"Cannot deserialize value of type %s from deeply-nested Array: only single wrapper allowed with `%s`",
+                            ClassUtil.getTypeDescription(targetType),
+                                    "DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS");
+                }
+                final Object value = deserialize(p, ctxt);
+                if (p.nextToken() != JsonToken.END_ARRAY) {
+                    handleMissingEndArrayForSingle(p, ctxt);
+                }
+                return value;
+            }
+            // 15-Nov-2022, tatu: ... we probably should pass original `JsonToken.START_ARRAY`
+            //     as unexpected token, since `p` now points to `unwrappedToken` instead...
+        }
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
+    }
+
     /*
     /**********************************************************
     /* Deserializing when we have to consider an active View
@@ -573,13 +671,20 @@ public class BeanDeserializer
         throws IOException
     {
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 // TODO: 06-Jan-2015, tatu: try streamlining call sequences here as well
                 SettableBeanProperty prop = _beanProperties.find(propName);
                 if (prop != null) {
                     if (!prop.visibleInView(activeView)) {
+                        // [databind#437]: fields in other views to be considered as unknown properties
+                        if (ctxt.isEnabled(DeserializationFeature.FAIL_ON_UNEXPECTED_VIEW_PROPERTIES)){
+                            ctxt.reportInputMismatch(handledType(),
+                                String.format("Input mismatch while deserializing %s. Property '%s' is not part of current active view '%s'" +
+                                        " (disable 'DeserializationFeature.FAIL_ON_UNEXPECTED_VIEW_PROPERTIES' to allow)",
+                                    ClassUtil.nameOf(handledType()), prop.getName(), activeView.getName()));
+                        }
                         p.skipChildren();
                         continue;
                     }
@@ -595,7 +700,7 @@ public class BeanDeserializer
         }
         return bean;
     }
-    
+
     /*
     /**********************************************************
     /* Handling for cases where we have "unwrapped" values
@@ -616,18 +721,18 @@ public class BeanDeserializer
         if (_propertyBasedCreator != null) {
             return deserializeUsingPropertyBasedWithUnwrapped(p, ctxt);
         }
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
 
         // [databind#631]: Assign current value, to be accessible by custom serializers
-        p.setCurrentValue(bean);
+        p.assignCurrentValue(bean);
 
         if (_injectables != null) {
             injectValues(ctxt, bean);
         }
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        String propName = p.hasTokenId(JsonTokenId.ID_FIELD_NAME) ? p.getCurrentName() : null;
+        String propName = p.hasTokenId(JsonTokenId.ID_FIELD_NAME) ? p.currentName() : null;
 
         for (; propName != null; propName = p.nextFieldName()) {
             p.nextToken();
@@ -645,7 +750,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -660,7 +765,7 @@ public class BeanDeserializer
                 continue;
             }
             // Need to copy to a separate buffer first
-            TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+            TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
             tokens.writeFieldName(propName);
             tokens.append(b2);
             try {
@@ -679,15 +784,15 @@ public class BeanDeserializer
             Object bean)
         throws IOException
     {
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         if (t == JsonToken.START_OBJECT) {
             t = p.nextToken();
         }
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             SettableBeanProperty prop = _beanProperties.find(propName);
             p.nextToken();
             if (prop != null) { // normal case
@@ -702,7 +807,7 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -716,7 +821,7 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+                TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
@@ -739,19 +844,23 @@ public class BeanDeserializer
         // 01-Dec-2016, tatu: Note: This IS legal to call, but only when unwrapped
         //    value itself is NOT passed via `CreatorProperty` (which isn't supported).
         //    Ok however to pass via setter or field.
-        
+
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
 
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 // Last creator property to set?
                 if (buffer.assignParameter(creatorProp,
@@ -764,7 +873,7 @@ public class BeanDeserializer
                         bean = wrapInstantiationProblem(e, ctxt);
                     }
                     // [databind#631]: Assign current value, to be accessible by custom serializers
-                    p.setCurrentValue(bean);
+                    p.assignCurrentValue(bean);
                     // if so, need to copy all remaining tokens into buffer
                     while (t == JsonToken.FIELD_NAME) {
                         // NOTE: do NOT skip name as it needs to be copied; `copyCurrentStructure` does that
@@ -774,7 +883,7 @@ public class BeanDeserializer
                     // 28-Aug-2018, tatu: Let's add sanity check here, easier to catch off-by-some
                     //    problems if we maintain invariants
                     if (t != JsonToken.END_OBJECT) {
-                        ctxt.reportWrongTokenException(this, JsonToken.END_OBJECT, 
+                        ctxt.reportWrongTokenException(this, JsonToken.END_OBJECT,
                                 "Attempted to unwrap '%s' value",
                                 handledType().getName());
                     }
@@ -782,18 +891,13 @@ public class BeanDeserializer
                     if (bean.getClass() != _beanType.getRawClass()) {
                         // !!! 08-Jul-2011, tatu: Could probably support; but for now
                         //   it's too complicated, so bail out
-                        ctxt.reportInputMismatch(creatorProp,
+                        return ctxt.reportInputMismatch(creatorProp,
                                 "Cannot create polymorphic instances with unwrapped values");
-                        return null;
                     }
                     return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
@@ -801,7 +905,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -815,7 +919,7 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+                TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
@@ -833,8 +937,7 @@ public class BeanDeserializer
         try {
             bean = creator.build(ctxt, buffer);
         } catch (Exception e) {
-            wrapInstantiationProblem(e, ctxt);
-            return null; // never gets here
+            return wrapInstantiationProblem(e, ctxt);
         }
         return _unwrappedPropertyHandler.processUnwrapped(p, ctxt, bean, tokens);
     }
@@ -869,11 +972,17 @@ public class BeanDeserializer
             Object bean)
         throws IOException
     {
-        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        final ExternalTypeHandler ext = _externalTypeIdHandler.start();
+        return _deserializeWithExternalTypeId(p, ctxt, bean,
+                _externalTypeIdHandler.start());
+    }
 
-        for (JsonToken t = p.getCurrentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+    protected Object _deserializeWithExternalTypeId(JsonParser p, DeserializationContext ctxt,
+            Object bean, ExternalTypeHandler ext)
+        throws IOException
+    {
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
+        for (JsonToken t = p.currentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.currentName();
             t = p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
@@ -893,7 +1002,7 @@ public class BeanDeserializer
                 continue;
             }
             // ignorable things should be ignored
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -918,22 +1027,25 @@ public class BeanDeserializer
     }
 
     @SuppressWarnings("resource")
-    protected Object deserializeUsingPropertyBasedWithExternalTypeId(JsonParser p, DeserializationContext ctxt)
+    protected Object deserializeUsingPropertyBasedWithExternalTypeId(JsonParser p,
+            DeserializationContext ctxt)
         throws IOException
     {
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
-        tokens.writeStartObject();
-
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
-            p.nextToken(); // to point to value
+            String propName = p.currentName();
+            t = p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 // first: let's check to see if this might be part of value with external type id:
                 // 11-Sep-2015, tatu: Important; do NOT pass buffer as last arg, but null,
@@ -942,7 +1054,8 @@ public class BeanDeserializer
                     ;
                 } else {
                     // Last creator property to set?
-                    if (buffer.assignParameter(creatorProp, _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
+                    if (buffer.assignParameter(creatorProp,
+                            _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
                         t = p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
                         Object bean;
                         try {
@@ -951,12 +1064,6 @@ public class BeanDeserializer
                             wrapAndThrow(e, _beanType.getRawClass(), propName, ctxt);
                             continue; // never gets here
                         }
-                        // if so, need to copy all remaining tokens into buffer
-                        while (t == JsonToken.FIELD_NAME) {
-                            p.nextToken(); // to skip name
-                            tokens.copyCurrentStructure(p);
-                            t = p.nextToken();
-                        }
                         if (bean.getClass() != _beanType.getRawClass()) {
                             // !!! 08-Jul-2011, tatu: Could theoretically support; but for now
                             //   it's too complicated, so bail out
@@ -964,19 +1071,24 @@ public class BeanDeserializer
                                     "Cannot create polymorphic instances with external type ids (%s -> %s)",
                                     _beanType, bean.getClass()));
                         }
-                        return ext.complete(p, ctxt, bean);
+                        return _deserializeWithExternalTypeId(p, ctxt, bean, ext);
                     }
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
-                buffer.bufferProperty(prop, prop.deserialize(p, ctxt));
+                // [databind#3045]: may have property AND be used as external type id:
+                if (t.isScalarValue()) {
+                    ext.handleTypePropertyValue(p, ctxt, propName, null);
+                }
+                // 19-Feb-2021, tatu: Should probably consider view too?
+                if (activeView != null && !prop.visibleInView(activeView)) {
+                    p.skipChildren();
+                } else {
+                    buffer.bufferProperty(prop, prop.deserialize(p, ctxt));
+                }
                 continue;
             }
             // external type id (or property that depends on it)?
@@ -984,7 +1096,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -992,9 +1104,11 @@ public class BeanDeserializer
             if (_anySetter != null) {
                 buffer.bufferAnyProperty(_anySetter, propName,
                         _anySetter.deserialize(p, ctxt));
+                continue;
             }
+            // Unknown: let's call handler method
+            handleUnknownProperty(p, ctxt, _valueClass, propName);
         }
-        tokens.writeEndObject();
 
         // We hit END_OBJECT; resolve the pieces:
         try {
