@@ -1,18 +1,12 @@
 package com.fasterxml.jackson.databind.deser.impl;
 
 import java.io.IOException;
-import java.lang.annotation.Annotation;
 import java.util.Collection;
 import java.util.Map;
 
 import com.fasterxml.jackson.core.JsonParser;
-import com.fasterxml.jackson.core.JsonProcessingException;
-import com.fasterxml.jackson.databind.DeserializationContext;
-import com.fasterxml.jackson.databind.JsonDeserializer;
-import com.fasterxml.jackson.databind.PropertyName;
+import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.deser.SettableBeanProperty;
-import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
-import com.fasterxml.jackson.databind.util.Annotations;
 
 /**
  * Wrapper property that is used to handle managed (forward) properties
@@ -20,73 +14,41 @@ import com.fasterxml.jackson.databind.util.Annotations;
  * then to back property.
  */
 public final class ManagedReferenceProperty
-    extends SettableBeanProperty
+    // Changed to extends delegating base class in 2.9
+    extends SettableBeanProperty.Delegating
 {
     private static final long serialVersionUID = 1L;
 
     protected final String _referenceName;
-    
+
     /**
      * Flag that indicates whether property to handle is a container type
      * (array, Collection, Map) or not.
      */
     protected final boolean _isContainer;
-    
-    protected final SettableBeanProperty _managedProperty;
 
     protected final SettableBeanProperty _backProperty;
-    
+
     public ManagedReferenceProperty(SettableBeanProperty forward, String refName,
-            SettableBeanProperty backward, Annotations contextAnnotations, boolean isContainer)
+            SettableBeanProperty backward, boolean isContainer)
     {
-        super(forward.getFullName(), forward.getType(), forward.getWrapperName(),
-                forward.getValueTypeDeserializer(), contextAnnotations,
-                forward.getMetadata());
+        super(forward);
         _referenceName = refName;
-        _managedProperty = forward;
         _backProperty = backward;
         _isContainer = isContainer;
     }
 
-    protected ManagedReferenceProperty(ManagedReferenceProperty src, JsonDeserializer<?> deser)
-    {
-        super(src, deser);
-        _referenceName = src._referenceName;
-        _isContainer = src._isContainer;
-        _managedProperty = src._managedProperty;
-        _backProperty = src._backProperty;
-    }
-
-    protected ManagedReferenceProperty(ManagedReferenceProperty src, PropertyName newName) {
-        super(src, newName);
-        _referenceName = src._referenceName;
-        _isContainer = src._isContainer;
-        _managedProperty = src._managedProperty;
-        _backProperty = src._backProperty;
+    @Override
+    protected SettableBeanProperty withDelegate(SettableBeanProperty d) {
+        throw new IllegalStateException("Should never try to reset delegate");
     }
 
+    // need to override to ensure both get fixed
     @Override
-    public ManagedReferenceProperty withName(PropertyName newName) {
-        return new ManagedReferenceProperty(this, newName);
+    public void fixAccess(DeserializationConfig config) {
+        delegate.fixAccess(config);
+        _backProperty.fixAccess(config);
     }
-    
-    @Override
-    public ManagedReferenceProperty withValueDeserializer(JsonDeserializer<?> deser) {
-        return new ManagedReferenceProperty(this, deser);
-    }
-    
-    /*
-    /**********************************************************
-    /* BeanProperty impl
-    /**********************************************************
-     */
-    
-    @Override
-    public <A extends Annotation> A getAnnotation(Class<A> acls) {
-        return _managedProperty.getAnnotation(acls);
-    }
-
-    @Override public AnnotatedMember getMember() {  return _managedProperty.getMember(); }
 
     /*
     /**********************************************************
@@ -95,20 +57,20 @@ public final class ManagedReferenceProperty
      */
 
     @Override
-    public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
-            throws IOException, JsonProcessingException {
-        set(instance, _managedProperty.deserialize(jp, ctxt));
+    public void deserializeAndSet(JsonParser p, DeserializationContext ctxt, Object instance)
+            throws IOException {
+        set(instance, delegate.deserialize(p, ctxt));
     }
 
     @Override
-    public Object deserializeSetAndReturn(JsonParser jp, DeserializationContext ctxt, Object instance)
-            throws IOException, JsonProcessingException {
-        return setAndReturn(instance, deserialize(jp, ctxt));
+    public Object deserializeSetAndReturn(JsonParser p, DeserializationContext ctxt, Object instance)
+            throws IOException {
+        return setAndReturn(instance, deserialize(p, ctxt));
     }
-    
+
     @Override
     public final void set(Object instance, Object value) throws IOException {
-    	setAndReturn(instance, value);
+        setAndReturn(instance, value);
     }
 
     @Override
@@ -140,6 +102,6 @@ public final class ManagedReferenceProperty
             }
         }
         // and then the forward reference itself
-        return _managedProperty.setAndReturn(instance, value);
+        return delegate.setAndReturn(instance, value);
 	}
 }
