@@ -27,7 +27,7 @@ abstract class NodeCursor
      * @since 2.5
      */
     protected java.lang.Object _currentValue;
-    
+
     public NodeCursor(int contextType, NodeCursor p)
     {
         super();
@@ -67,7 +67,7 @@ abstract class NodeCursor
     public void setCurrentValue(java.lang.Object v) {
         _currentValue = v;
     }
-    
+
     /*
     /**********************************************************
     /* Extended API
@@ -75,12 +75,11 @@ abstract class NodeCursor
      */
 
     public abstract JsonToken nextToken();
-    public abstract JsonToken nextValue();
-    public abstract JsonToken endToken();
-
     public abstract JsonNode currentNode();
-    public abstract boolean currentHasChildren();
-    
+
+    public abstract NodeCursor startObject();
+    public abstract NodeCursor startArray();
+
     /**
      * Method called to create a new context for iterating all
      * contents of the current structured value (JSON array or object)
@@ -104,9 +103,8 @@ abstract class NodeCursor
      */
 
     /**
-     * Context matching root-level value nodes (i.e. anything other
-     * than JSON Object and Array).
-     * Note that context is NOT created for leaf values.
+     * Context for all root-level value nodes (including Arrays and Objects):
+     * only context for scalar values.
      */
     protected final static class RootCursor
         extends NodeCursor
@@ -122,9 +120,9 @@ abstract class NodeCursor
 
         @Override
         public void overrideCurrentName(String name) {
-            
+
         }
-        
+
         @Override
         public JsonToken nextToken() {
             if (!_done) {
@@ -135,26 +133,28 @@ abstract class NodeCursor
             _node = null;
             return null;
         }
-        
+
         @Override
-        public JsonToken nextValue() { return nextToken(); }
+        public JsonNode currentNode() {
+            // May look weird, but is necessary so as not to expose current node
+            // before it has been traversed
+            return _done ? _node : null;
+        }
+
         @Override
-        public JsonToken endToken() { return null; }
+        public NodeCursor startArray() { return new ArrayCursor(_node, this); }
+
         @Override
-        public JsonNode currentNode() { return _node; }
-        @Override
-        public boolean currentHasChildren() { return false; }
+        public NodeCursor startObject() { return new ObjectCursor(_node, this); }
     }
 
-    /**
-     * Cursor used for traversing non-empty JSON Array nodes
-     */
+    // Cursor used for traversing JSON Array nodes
     protected final static class ArrayCursor
         extends NodeCursor
     {
         protected Iterator<JsonNode> _contents;
 
-        protected JsonNode _currentNode;
+        protected JsonNode _currentElement;
 
         public ArrayCursor(JsonNode n, NodeCursor p) {
             super(JsonStreamContext.TYPE_ARRAY, p);
@@ -165,31 +165,25 @@ abstract class NodeCursor
         public JsonToken nextToken()
         {
             if (!_contents.hasNext()) {
-                _currentNode = null;
-                return null;
+                _currentElement = null;
+                return JsonToken.END_ARRAY;
             }
             ++_index;
-            _currentNode = _contents.next();
-            return _currentNode.asToken();
+            _currentElement = _contents.next();
+            return _currentElement.asToken();
         }
 
         @Override
-        public JsonToken nextValue() { return nextToken(); }
-        @Override
-        public JsonToken endToken() { return JsonToken.END_ARRAY; }
+        public JsonNode currentNode() { return _currentElement; }
 
         @Override
-        public JsonNode currentNode() { return _currentNode; }
+        public NodeCursor startArray() { return new ArrayCursor(_currentElement, this); }
+
         @Override
-        public boolean currentHasChildren() {
-            // note: ONLY to be called for container nodes
-            return ((ContainerNode<?>) currentNode()).size() > 0;
-        }
+        public NodeCursor startObject() { return new ObjectCursor(_currentElement, this); }
     }
 
-    /**
-     * Cursor used for traversing non-empty JSON Object nodes
-     */
+    // Cursor used for traversing JSON Object nodes
     protected final static class ObjectCursor
         extends NodeCursor
     {
@@ -197,11 +191,11 @@ abstract class NodeCursor
         protected Map.Entry<String, JsonNode> _current;
 
         protected boolean _needEntry;
-        
+
         public ObjectCursor(JsonNode n, NodeCursor p)
         {
             super(JsonStreamContext.TYPE_OBJECT, p);
-            _contents = ((ObjectNode) n).fields();
+            _contents = n.fields();
             _needEntry = true;
         }
 
@@ -213,7 +207,7 @@ abstract class NodeCursor
                 if (!_contents.hasNext()) {
                     _currentName = null;
                     _current = null;
-                    return null;
+                    return JsonToken.END_OBJECT;
                 }
                 ++_index;
                 _needEntry = false;
@@ -225,27 +219,15 @@ abstract class NodeCursor
             return _current.getValue().asToken();
         }
 
-        @Override
-        public JsonToken nextValue()
-        {
-            JsonToken t = nextToken();
-            if (t == JsonToken.FIELD_NAME) {
-                t = nextToken();
-            }
-            return t;
-        }
-
-        @Override
-        public JsonToken endToken() { return JsonToken.END_OBJECT; }
-
         @Override
         public JsonNode currentNode() {
             return (_current == null) ? null : _current.getValue();
         }
+
         @Override
-        public boolean currentHasChildren() {
-            // note: ONLY to be called for container nodes
-            return ((ContainerNode<?>) currentNode()).size() > 0;
-        }
+        public NodeCursor startArray() { return new ArrayCursor(currentNode(), this); }
+
+        @Override
+        public NodeCursor startObject() { return new ObjectCursor(currentNode(), this); }
     }
 }
