@@ -1,6 +1,7 @@
 package com.fasterxml.jackson.databind.node;
 
 import java.io.IOException;
+import java.util.Objects;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.io.CharTypes;
@@ -38,7 +39,7 @@ public class TextNode
         if (v == null) {
             return null;
         }
-        if (v.length() == 0) {
+        if (v.isEmpty()) {
             return EMPTY_STRING_NODE;
         }
         return new TextNode(v);
@@ -65,7 +66,13 @@ public class TextNode
     public byte[] getBinaryValue(Base64Variant b64variant) throws IOException
     {
         final String str = _value.trim();
-        ByteArrayBuilder builder = new ByteArrayBuilder(4 + ((str.length() * 3) >> 2));
+        // 04-Sep-2020, tatu: Let's limit the size of the initial block to 64k,
+        //    no point in trying to exactly match the size beyond certain point
+        // (plus it could even lead to unnecessarily high retention with block
+        // recycling)
+        final int initBlockSize = 4 + ((str.length() >> 2) * 3);
+        ByteArrayBuilder builder = new ByteArrayBuilder(Math.max(16,
+                Math.min(0x10000, initBlockSize)));
         try {
             b64variant.decode(str, builder);
         } catch (IllegalArgumentException e) {
@@ -83,7 +90,7 @@ e.getMessage()),
         return getBinaryValue(Base64Variants.getDefaultVariant());
     }
 
-    /* 
+    /*
     /**********************************************************
     /* General type coercions
     /**********************************************************
@@ -98,7 +105,7 @@ e.getMessage()),
     public String asText(String defaultValue) {
         return (_value == null) ? defaultValue : _value;
     }
-    
+
     // note: neither fast nor elegant, but these work for now:
 
     @Override
@@ -114,7 +121,7 @@ e.getMessage()),
         }
         return defaultValue;
     }
-    
+
     @Override
     public int asInt(int defaultValue) {
         return NumberInput.parseAsInt(_value, defaultValue);
@@ -124,18 +131,18 @@ e.getMessage()),
     public long asLong(long defaultValue) {
         return NumberInput.parseAsLong(_value, defaultValue);
     }
-    
+
     @Override
     public double asDouble(double defaultValue) {
         return NumberInput.parseAsDouble(_value, defaultValue);
     }
-    
-    /* 
+
+    /*
     /**********************************************************
     /* Serialization
     /**********************************************************
      */
-    
+
     @Override
     public final void serialize(JsonGenerator g, SerializerProvider provider) throws IOException
     {
@@ -151,20 +158,22 @@ e.getMessage()),
     /* Overridden standard methods
     /**********************************************************
      */
-    
+
     @Override
     public boolean equals(Object o)
     {
         if (o == this) return true;
         if (o == null) return false;
         if (o instanceof TextNode) {
-            return ((TextNode) o)._value.equals(_value);
+            return Objects.equals(((TextNode) o)._value, _value);
         }
         return false;
     }
-    
+
     @Override
-    public int hashCode() { return _value.hashCode(); }
+    public int hashCode() {
+        return Objects.hashCode(_value);
+    }
 
     @Deprecated // since 2.10
     protected static void appendQuoted(StringBuilder sb, String content)
