@@ -5,10 +5,11 @@ import java.lang.reflect.Type;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
-import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.annotation.JsonInclude;
+
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.type.WritableTypeId;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
@@ -23,6 +24,7 @@ import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.databind.util.ArrayBuilders;
 import com.fasterxml.jackson.databind.util.BeanUtil;
 import com.fasterxml.jackson.databind.util.ClassUtil;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 
 /**
  * Standard serializer implementation for serializing {link java.util.Map} types.
@@ -49,7 +51,7 @@ public class MapSerializer
     /* Basic information about referring property, type
     /**********************************************************
      */
-    
+
     /**
      * Map-valued property being serialized with this instance
      */
@@ -76,7 +78,7 @@ public class MapSerializer
     /* Serializers used
     /**********************************************************
      */
-    
+
     /**
      * Key serializer to use, if it can be statically determined
      */
@@ -103,12 +105,19 @@ public class MapSerializer
     /* Config settings, filtering
     /**********************************************************
      */
-    
+
     /**
      * Set of entries to omit during serialization, if any
      */
     protected final Set<String> _ignoredEntries;
 
+    /**
+     * Set of entries to include during serialization, if null, it is ignored, empty will include nothing.
+     *
+     * @since 2.12
+     */
+    protected final Set<String> _includedEntries;
+
     /**
      * Id of the property filter to use, if any; null if none.
      *
@@ -123,7 +132,7 @@ public class MapSerializer
      * non-null values.
      * Note that inclusion value for Map instance itself is handled by caller (POJO
      * property that refers to the Map value).
-     * 
+     *
      * @since 2.5
      */
     protected final Object _suppressableValue;
@@ -136,6 +145,13 @@ public class MapSerializer
      */
     protected final boolean _suppressNulls;
 
+    /**
+     * Helper object used for name-based filtering
+     *
+     * @since 2.12
+     */
+    protected final IgnorePropertiesUtil.Checker _inclusionChecker;
+
     /*
     /**********************************************************
     /* Config settings, other
@@ -145,7 +161,7 @@ public class MapSerializer
     /**
      * Flag set if output is forced to be sorted by keys (usually due
      * to annotation).
-     * 
+     *
      * @since 2.4
      */
     protected final boolean _sortKeys;
@@ -155,12 +171,12 @@ public class MapSerializer
     /* Life-cycle
     /**********************************************************
      */
-    
+
     /**
-     * @since 2.5
+     * @since 2.12
      */
     @SuppressWarnings("unchecked")
-    protected MapSerializer(Set<String> ignoredEntries,
+    protected MapSerializer(Set<String> ignoredEntries, Set<String> includedEntries,
             JavaType keyType, JavaType valueType, boolean valueTypeIsStatic,
             TypeSerializer vts,
             JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer)
@@ -168,6 +184,7 @@ public class MapSerializer
         super(Map.class, false);
         _ignoredEntries = ((ignoredEntries == null) || ignoredEntries.isEmpty())
                 ? null : ignoredEntries;
+        _includedEntries = includedEntries;
         _keyType = keyType;
         _valueType = valueType;
         _valueTypeIsStatic = valueTypeIsStatic;
@@ -180,16 +197,38 @@ public class MapSerializer
         _sortKeys = false;
         _suppressableValue = null;
         _suppressNulls = false;
+
+        _inclusionChecker = IgnorePropertiesUtil.buildCheckerIfNeeded(_ignoredEntries, _includedEntries);
     }
 
+    /**
+     * @since 2.5
+     * @deprecated in 2.12, remove from 3.0
+     */
+    @Deprecated
+    protected MapSerializer(Set<String> ignoredEntries,
+            JavaType keyType, JavaType valueType, boolean valueTypeIsStatic,
+            TypeSerializer vts,
+            JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer)
+    {
+        this(ignoredEntries, null,
+                keyType, valueType, valueTypeIsStatic,
+                vts,
+                keySerializer, valueSerializer);
+    }
+
+    /**
+     * @since 2.12
+     */
     @SuppressWarnings("unchecked")
     protected MapSerializer(MapSerializer src, BeanProperty property,
             JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer,
-            Set<String> ignoredEntries)
+            Set<String> ignoredEntries, Set<String> includedEntries)
     {
         super(Map.class, false);
         _ignoredEntries = ((ignoredEntries == null) || ignoredEntries.isEmpty())
                 ? null : ignoredEntries;
+        _includedEntries = includedEntries;
         _keyType = src._keyType;
         _valueType = src._valueType;
         _valueTypeIsStatic = src._valueTypeIsStatic;
@@ -203,6 +242,19 @@ public class MapSerializer
         _sortKeys = src._sortKeys;
         _suppressableValue = src._suppressableValue;
         _suppressNulls = src._suppressNulls;
+
+        _inclusionChecker = IgnorePropertiesUtil.buildCheckerIfNeeded(_ignoredEntries, _includedEntries);
+    }
+
+    /**
+     * @deprecated in 2.12, remove from 3.0
+     */
+    @Deprecated
+    protected MapSerializer(MapSerializer src, BeanProperty property,
+                            JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer,
+                            Set<String> ignoredEntries)
+    {
+        this(src, property, keySerializer, valueSerializer, ignoredEntries, null);
     }
 
     /**
@@ -213,6 +265,7 @@ public class MapSerializer
     {
         super(Map.class, false);
         _ignoredEntries = src._ignoredEntries;
+        _includedEntries = src._includedEntries;
         _keyType = src._keyType;
         _valueType = src._valueType;
         _valueTypeIsStatic = src._valueTypeIsStatic;
@@ -227,12 +280,15 @@ public class MapSerializer
         _sortKeys = src._sortKeys;
         _suppressableValue = suppressableValue;
         _suppressNulls = suppressNulls;
+
+        _inclusionChecker = src._inclusionChecker;
     }
 
     protected MapSerializer(MapSerializer src, Object filterId, boolean sortKeys)
     {
         super(Map.class, false);
         _ignoredEntries = src._ignoredEntries;
+        _includedEntries = src._includedEntries;
         _keyType = src._keyType;
         _valueType = src._valueType;
         _valueTypeIsStatic = src._valueTypeIsStatic;
@@ -246,6 +302,8 @@ public class MapSerializer
         _sortKeys = sortKeys;
         _suppressableValue = src._suppressableValue;
         _suppressNulls = src._suppressNulls;
+
+        _inclusionChecker = src._inclusionChecker;
     }
 
     @Override
@@ -257,6 +315,21 @@ public class MapSerializer
         return new MapSerializer(this, vts, _suppressableValue, _suppressNulls);
     }
 
+    /**
+     * @since 2.12
+     */
+    public MapSerializer withResolved(BeanProperty property,
+             JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer,
+             Set<String> ignored, Set<String> included, boolean sortKeys)
+    {
+        _ensureOverride("withResolved");
+        MapSerializer ser = new MapSerializer(this, property, keySerializer, valueSerializer, ignored, included);
+        if (sortKeys != ser._sortKeys) {
+            ser = new MapSerializer(ser, _filterId, sortKeys);
+        }
+        return ser;
+    }
+
     /**
      * @since 2.4
      */
@@ -264,12 +337,7 @@ public class MapSerializer
             JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer,
             Set<String> ignored, boolean sortKeys)
     {
-        _ensureOverride("withResolved");
-        MapSerializer ser = new MapSerializer(this, property, keySerializer, valueSerializer, ignored);
-        if (sortKeys != ser._sortKeys) {
-            ser = new MapSerializer(ser, _filterId, sortKeys);
-        }
-        return ser;
+        return withResolved(property, keySerializer, valueSerializer, ignored, null, sortKeys);
     }
 
     @Override
@@ -284,7 +352,7 @@ public class MapSerializer
     /**
      * Mutant factory for constructing an instance with different inclusion strategy
      * for content (Map values).
-     * 
+     *
      * @since 2.9
      */
     public MapSerializer withContentInclusion(Object suppressableValue, boolean suppressNulls) {
@@ -296,20 +364,26 @@ public class MapSerializer
     }
 
     /**
-     * @since 2.8
+     * @since 2.12
      */
-    public static MapSerializer construct(Set<String> ignoredEntries, JavaType mapType,
+    public static MapSerializer construct(Set<String> ignoredEntries, Set<String> includedEntries, JavaType mapType,
             boolean staticValueType, TypeSerializer vts,
             JsonSerializer<Object> keySerializer, JsonSerializer<Object> valueSerializer,
             Object filterId)
     {
         JavaType keyType, valueType;
-        
+
         if (mapType == null) {
             keyType = valueType = UNSPECIFIED_TYPE;
-        } else { 
+        } else {
             keyType = mapType.getKeyType();
-            valueType = mapType.getContentType();
+            if (mapType.hasRawClass(java.util.Properties.class)) {
+                // 25-Mar-2020, tatu: [databind#2657] Since non-standard Properties may actually
+                //     contain non-Strings, demote value type to raw `Object`
+                valueType = TypeFactory.unknownType();
+            } else {
+                valueType = mapType.getContentType();
+            }
         }
         // If value type is final, it's same as forcing static value typing:
         if (!staticValueType) {
@@ -320,7 +394,7 @@ public class MapSerializer
                 staticValueType = false;
             }
         }
-        MapSerializer ser = new MapSerializer(ignoredEntries, keyType, valueType, staticValueType, vts,
+        MapSerializer ser = new MapSerializer(ignoredEntries, includedEntries, keyType, valueType, staticValueType, vts,
                 keySerializer, valueSerializer);
         if (filterId != null) {
             ser = ser.withFilterId(filterId);
@@ -328,6 +402,17 @@ public class MapSerializer
         return ser;
     }
 
+    /**
+     * @since 2.8
+     */
+    public static MapSerializer construct(Set<String> ignoredEntries, JavaType mapType,
+            boolean staticValueType, TypeSerializer vts,
+            JsonSerializer<Object> keySerializer, JsonSerializer<Object> valueSerializer,
+            Object filterId)
+    {
+        return construct(ignoredEntries, null, mapType, staticValueType, vts, keySerializer, valueSerializer, filterId);
+    }
+
     /**
      * @since 2.9
      */
@@ -366,7 +451,7 @@ public class MapSerializer
     @Deprecated // since 2.9
     public MapSerializer withContentInclusion(Object suppressableValue) {
         return new MapSerializer(this, _valueTypeSerializer, suppressableValue, _suppressNulls);
-    }                
+    }
 
     /**
      * @since 2.3
@@ -421,7 +506,7 @@ public class MapSerializer
             //   we can consider it a static case as well.
             // 20-Aug-2013, tatu: Need to avoid trying to access serializer for java.lang.Object tho
             if (_valueTypeIsStatic && !_valueType.isJavaLangObject()) {
-                ser = provider.findValueSerializer(_valueType, property);
+                ser = provider.findContentValueSerializer(_valueType, property);
             }
         }
         if (keySer == null) {
@@ -433,18 +518,27 @@ public class MapSerializer
             keySer = provider.handleSecondaryContextualization(keySer, property);
         }
         Set<String> ignored = _ignoredEntries;
+        Set<String> included = _includedEntries;
         boolean sortKeys = false;
         if (_neitherNull(propertyAcc, intr)) {
-            JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnorals(propertyAcc);
-            if (ignorals != null){
-                Set<String> newIgnored = ignorals.findIgnoredForSerialization();
-                if (_nonEmpty(newIgnored)) {
-                    ignored = (ignored == null) ? new HashSet<String>() : new HashSet<String>(ignored);
-                    for (String str : newIgnored) {
-                        ignored.add(str);
-                    }
+            final SerializationConfig config = provider.getConfig();
+            // ignorals
+            Set<String> newIgnored = intr.findPropertyIgnoralByName(config, propertyAcc).findIgnoredForSerialization();
+            if (_nonEmpty(newIgnored)) {
+                ignored = (ignored == null) ? new HashSet<String>() : new HashSet<String>(ignored);
+                for (String str : newIgnored) {
+                    ignored.add(str);
                 }
             }
+            // inclusions
+            Set<String> newIncluded = intr.findPropertyInclusionByName(config, propertyAcc).getIncluded();
+            if (newIncluded != null) {
+                included = (included == null) ? new HashSet<String>() : new HashSet<String>(included);
+                for (String str : newIncluded) {
+                    included.add(str);
+                }
+            }
+            // sort key
             Boolean b = intr.findSerializationSortAlphabetically(propertyAcc);
             sortKeys = Boolean.TRUE.equals(b);
         }
@@ -452,19 +546,16 @@ public class MapSerializer
         if (format != null) {
             Boolean B = format.getFeature(JsonFormat.Feature.WRITE_SORTED_MAP_ENTRIES);
             if (B != null) {
-                sortKeys = B.booleanValue();
+                sortKeys = B;
             }
         }
-        MapSerializer mser = withResolved(property, keySer, ser, ignored, sortKeys);
+        MapSerializer mser = withResolved(property, keySer, ser, ignored, included, sortKeys);
 
         // [databind#307]: allow filtering
-        if (property != null) {
-            AnnotatedMember m = property.getMember();
-            if (m != null) {
-                Object filterId = intr.findFilterId(m);
-                if (filterId != null) {
-                    mser = mser.withFilterId(filterId);
-                }
+        if (propertyAcc != null) {
+            Object filterId = intr.findFilterId(propertyAcc);
+            if (filterId != null) {
+                mser = mser.withFilterId(filterId);
             }
         }
         JsonInclude.Value inclV = findIncludeOverrides(provider, property, Map.class);
@@ -540,7 +631,7 @@ public class MapSerializer
         if (value.isEmpty()) {
             return true;
         }
-        
+
         // 05-Nove-2015, tatu: Simple cases are cheap, but for recursive
         //   emptiness checking we actually need to see if values are empty as well.
         Object supp = _suppressableValue;
@@ -577,7 +668,7 @@ public class MapSerializer
             }
             try {
                 valueSer = _findSerializer(prov, elemValue);
-            } catch (JsonMappingException e) { // Ugh... cannot just throw as-is, so...
+            } catch (DatabindException e) { // Ugh... cannot just throw as-is, so...
                 // 05-Nov-2015, tatu: For now, probably best not to assume empty then
                 return false;
             }
@@ -610,13 +701,11 @@ public class MapSerializer
      * (which can be overridden by custom implementations), but for some
      * dynamic types, it is possible that serializer is only resolved
      * during actual serialization.
-     * 
-     * @since 2.0
      */
     public JsonSerializer<?> getKeySerializer() {
         return _keySerializer;
     }
-    
+
     /*
     /**********************************************************
     /* JsonSerializer implementation
@@ -628,21 +717,7 @@ public class MapSerializer
         throws IOException
     {
         gen.writeStartObject(value);
-        if (!value.isEmpty()) {
-            if (_sortKeys || provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
-                value = _orderEntries(value, gen, provider);
-            }
-            PropertyFilter pf;
-            if ((_filterId != null) && (pf = findPropertyFilter(provider, _filterId, value)) != null) {
-                serializeFilteredFields(value, gen, provider, pf, _suppressableValue);
-            } else if ((_suppressableValue != null) || _suppressNulls) {
-                serializeOptionalFields(value, gen, provider, _suppressableValue);
-            } else if (_valueSerializer != null) {
-                serializeFieldsUsing(value, gen, provider, _valueSerializer);
-            } else {
-                serializeFields(value, gen, provider);
-            }
-        }
+        serializeWithoutTypeInfo(value, gen, provider);
         gen.writeEndObject();
     }
 
@@ -652,9 +727,28 @@ public class MapSerializer
         throws IOException
     {
         // [databind#631]: Assign current value, to be accessible by custom serializers
-        gen.setCurrentValue(value);
+        gen.assignCurrentValue(value);
         WritableTypeId typeIdDef = typeSer.writeTypePrefix(gen,
                 typeSer.typeId(value, JsonToken.START_OBJECT));
+        serializeWithoutTypeInfo(value, gen, provider);
+        typeSer.writeTypeSuffix(gen, typeIdDef);
+    }
+
+    /*
+    /**********************************************************
+    /* Secondary serialization methods
+    /**********************************************************
+     */
+
+    /**
+     * General-purpose serialization for contents without writing object type. Will suppress, filter and
+     * use custom serializers.
+     *<p>
+     * Public since it also is called by {@code AnyGetterWriter}.
+     *
+     * @since 2.11
+     */
+    public void serializeWithoutTypeInfo(Map<?, ?> value, JsonGenerator gen, SerializerProvider provider) throws IOException {
         if (!value.isEmpty()) {
             if (_sortKeys || provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
                 value = _orderEntries(value, gen, provider);
@@ -670,18 +764,11 @@ public class MapSerializer
                 serializeFields(value, gen, provider);
             }
         }
-        typeSer.writeTypeSuffix(gen, typeIdDef);
     }
 
-    /*
-    /**********************************************************
-    /* Secondary serialization methods
-    /**********************************************************
-     */
-    
     /**
      * General-purpose serialization for contents, where we do not necessarily know
-     * the value serialization, but 
+     * the value serialization, but
      * we do know that no value suppression is needed (which simplifies processing a bit)
      */
     public void serializeFields(Map<?,?> value, JsonGenerator gen, SerializerProvider provider)
@@ -693,7 +780,6 @@ public class MapSerializer
             return;
         }
         final JsonSerializer<Object> keySerializer = _keySerializer;
-        final Set<String> ignored = _ignoredEntries;
         Object keyElem = null;
 
         try {
@@ -705,7 +791,7 @@ public class MapSerializer
                     provider.findNullKeySerializer(_keyType, _property).serialize(null, gen, provider);
                 } else {
                     // One twist: is entry ignorable? If so, skip
-                    if ((ignored != null) && ignored.contains(keyElem)) {
+                    if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
                         continue;
                     }
                     keySerializer.serialize(keyElem, gen, provider);
@@ -738,7 +824,6 @@ public class MapSerializer
             serializeTypedFields(value, gen, provider, suppressableValue);
             return;
         }
-        final Set<String> ignored = _ignoredEntries;
         final boolean checkEmpty = (MARKER_FOR_EMPTY == suppressableValue);
 
         for (Map.Entry<?,?> entry : value.entrySet()) {
@@ -748,7 +833,9 @@ public class MapSerializer
             if (keyElem == null) {
                 keySerializer = provider.findNullKeySerializer(_keyType, _property);
             } else {
-                if (ignored != null && ignored.contains(keyElem)) continue;
+                if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
+                    continue;
+                }
                 keySerializer = _keySerializer;
             }
 
@@ -785,7 +872,7 @@ public class MapSerializer
             }
         }
     }
-    
+
     /**
      * Method called to serialize fields, when the value type is statically known,
      * so that value serializer is passed and does not need to be fetched from
@@ -796,12 +883,13 @@ public class MapSerializer
         throws IOException
     {
         final JsonSerializer<Object> keySerializer = _keySerializer;
-        final Set<String> ignored = _ignoredEntries;
         final TypeSerializer typeSer = _valueTypeSerializer;
 
         for (Map.Entry<?,?> entry : value.entrySet()) {
             Object keyElem = entry.getKey();
-            if (ignored != null && ignored.contains(keyElem)) continue;
+            if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
+                continue;
+            }
 
             if (keyElem == null) {
                 provider.findNullKeySerializer(_keyType, _property).serialize(null, gen, provider);
@@ -828,7 +916,7 @@ public class MapSerializer
     /**
      * Helper method used when we have a JSON Filter to use for potentially
      * filtering out Map entries.
-     * 
+     *
      * @since 2.5
      */
     public void serializeFilteredFields(Map<?,?> value, JsonGenerator gen, SerializerProvider provider,
@@ -836,14 +924,15 @@ public class MapSerializer
             Object suppressableValue) // since 2.5
         throws IOException
     {
-        final Set<String> ignored = _ignoredEntries;
         final MapProperty prop = new MapProperty(_valueTypeSerializer, _property);
         final boolean checkEmpty = (MARKER_FOR_EMPTY == suppressableValue);
 
         for (Map.Entry<?,?> entry : value.entrySet()) {
             // First, serialize key; unless ignorable by key
             final Object keyElem = entry.getKey();
-            if (ignored != null && ignored.contains(keyElem)) continue;
+            if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
+                continue;
+            }
 
             JsonSerializer<Object> keySerializer;
             if (keyElem == null) {
@@ -894,7 +983,6 @@ public class MapSerializer
             Object suppressableValue) // since 2.5
         throws IOException
     {
-        final Set<String> ignored = _ignoredEntries;
         final boolean checkEmpty = (MARKER_FOR_EMPTY == suppressableValue);
 
         for (Map.Entry<?,?> entry : value.entrySet()) {
@@ -904,11 +992,13 @@ public class MapSerializer
                 keySerializer = provider.findNullKeySerializer(_keyType, _property);
             } else {
                 // One twist: is entry ignorable? If so, skip
-                if (ignored != null && ignored.contains(keyElem)) continue;
+                if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
+                    continue;
+                }
                 keySerializer = _keySerializer;
             }
             final Object valueElem = entry.getValue();
-    
+
             // And then value
             JsonSerializer<Object> valueSer;
             if (valueElem == null) {
@@ -946,7 +1036,7 @@ public class MapSerializer
      * "any properties" of a POJO.
      *
      * @param bean Enclosing POJO that has any-getter used to obtain "any properties"
-     * 
+     *
      * @since 2.9
      */
     public void serializeFilteredAnyProperties(SerializerProvider provider, JsonGenerator gen,
@@ -954,14 +1044,15 @@ public class MapSerializer
             Object suppressableValue)
         throws IOException
     {
-        final Set<String> ignored = _ignoredEntries;
         final MapProperty prop = new MapProperty(_valueTypeSerializer, _property);
         final boolean checkEmpty = (MARKER_FOR_EMPTY == suppressableValue);
 
         for (Map.Entry<?,?> entry : value.entrySet()) {
             // First, serialize key; unless ignorable by key
             final Object keyElem = entry.getKey();
-            if (ignored != null && ignored.contains(keyElem)) continue;
+            if ((_inclusionChecker != null) && _inclusionChecker.shouldIgnore(keyElem)) {
+                continue;
+            }
 
             JsonSerializer<Object> keySerializer;
             if (keyElem == null) {
@@ -1011,6 +1102,10 @@ public class MapSerializer
     /**********************************************************
      */
 
+    /**
+     * @deprecated Since 2.15
+     */
+    @Deprecated
     @Override
     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
     {
@@ -1023,7 +1118,7 @@ public class MapSerializer
     public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint)
         throws JsonMappingException
     {
-        JsonMapFormatVisitor v2 = visitor.expectMapFormat(typeHint);        
+        JsonMapFormatVisitor v2 = visitor.expectMapFormat(typeHint);
         if (v2 != null) {
             v2.keyFormat(_keySerializer, _keyType);
             JsonSerializer<?> valueSer = _valueSerializer;
@@ -1042,7 +1137,8 @@ public class MapSerializer
      */
 
     protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map,
-            Class<?> type, SerializerProvider provider) throws JsonMappingException
+            Class<?> type, SerializerProvider provider)
+        throws JsonMappingException
     {
         PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSecondarySerializer(type, provider, _property);
         // did we get a new map of serializers? If so, start using it
@@ -1053,7 +1149,8 @@ public class MapSerializer
     }
 
     protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map,
-            JavaType type, SerializerProvider provider) throws JsonMappingException
+            JavaType type, SerializerProvider provider)
+        throws JsonMappingException
     {
         PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSecondarySerializer(type, provider, _property);
         if (map != result.map) {
@@ -1063,7 +1160,8 @@ public class MapSerializer
     }
 
     protected Map<?,?> _orderEntries(Map<?,?> input, JsonGenerator gen,
-            SerializerProvider provider) throws IOException
+            SerializerProvider provider)
+        throws IOException
     {
         // minor optimization: may already be sorted?
         if (input instanceof SortedMap<?,?>) {
@@ -1080,7 +1178,7 @@ public class MapSerializer
                 if (key == null) {
                     _writeNullKeyedEntry(gen, provider, entry.getValue());
                     continue;
-                } 
+                }
                 result.put(key, entry.getValue());
             }
             return result;
@@ -1100,12 +1198,13 @@ public class MapSerializer
         //   known good cases).
         //   While my first instinct was to do black-listing (remove Hashtable and ConcurrentHashMap),
         //   all in all it is probably better to just white list `HashMap` (and its sub-classes).
-        
+
         return (input instanceof HashMap) && input.containsKey(null);
     }
-    
+
     protected void _writeNullKeyedEntry(JsonGenerator gen, SerializerProvider provider,
-            Object value) throws IOException
+            Object value)
+        throws IOException
     {
         JsonSerializer<Object> keySerializer = provider.findNullKeySerializer(_keyType, _property);
         JsonSerializer<Object> valueSer;
@@ -1138,7 +1237,8 @@ public class MapSerializer
     }
 
     private final JsonSerializer<Object> _findSerializer(SerializerProvider provider,
-            Object value) throws JsonMappingException
+            Object value)
+        throws JsonMappingException
     {
         final Class<?> cc = value.getClass();
         JsonSerializer<Object> valueSer = _dynamicValueSerializers.serializerFor(cc);
