@@ -19,7 +19,7 @@ import com.fasterxml.jackson.databind.util.NameTransformer;
 /**
  * Base implementation for values of {@link ReferenceType}.
  * Implements most of functionality, only leaving couple of abstract
- * methods for sub-classes to implement
+ * methods for sub-classes to implement.
  *
  * @since 2.8
  */
@@ -33,7 +33,7 @@ public abstract class ReferenceTypeSerializer<T>
      * @since 2.9
      */
     public final static Object MARKER_FOR_EMPTY = JsonInclude.Include.NON_EMPTY;
-    
+
     /**
      * Value type
      */
@@ -116,7 +116,8 @@ public abstract class ReferenceTypeSerializer<T>
     {
         super(base);
         _referredType = base._referredType;
-        _dynamicSerializers = base._dynamicSerializers;
+        // [databind#2181]: may not be safe to reuse, start from empty
+        _dynamicSerializers = PropertySerializerMap.emptyForProperties();
         _property = property;
         _valueTypeSerializer = vts;
         _valueSerializer = (JsonSerializer<Object>) valueSer;
@@ -129,7 +130,12 @@ public abstract class ReferenceTypeSerializer<T>
     public JsonSerializer<T> unwrappingSerializer(NameTransformer transformer) {
         JsonSerializer<Object> valueSer = _valueSerializer;
         if (valueSer != null) {
+            // 09-Dec-2019, tatu: [databind#2565] Can not assume that serializer in
+            //    question actually can unwrap
             valueSer = valueSer.unwrappingSerializer(transformer);
+            if (valueSer == _valueSerializer) {
+                return this;
+            }
         }
         NameTransformer unwrapper = (_unwrapper == null) ? transformer
                 : NameTransformer.chainedTransformer(transformer, _unwrapper);
@@ -210,6 +216,9 @@ public abstract class ReferenceTypeSerializer<T>
                 ser = provider.handlePrimaryContextualization(ser, property);
             }
         }
+        // 23-Jan-2024, tatu: [databind#4337]: May have a content converter
+        ser = findContextualConvertingSerializer(provider, property, ser);
+
         // First, resolve wrt property, resolved serializers
         ReferenceTypeSerializer<?> refSer;
         if ((_property == property)
@@ -397,7 +406,7 @@ public abstract class ReferenceTypeSerializer<T>
         // 19-Apr-2016, tatu: In order to basically "skip" the whole wrapper level
         //    (which is what non-polymorphic serialization does too), we will need
         //    to simply delegate call, I think, and NOT try to use it here.
-        
+
         // Otherwise apply type-prefix/suffix, then std serialize:
         /*
         typeSer.writeTypePrefixForScalar(ref, g);
@@ -436,7 +445,7 @@ public abstract class ReferenceTypeSerializer<T>
     /* Helper methods
     /**********************************************************
      */
-    
+
     /**
      * Helper method that encapsulates logic of retrieving and caching required
      * serializer.
@@ -453,9 +462,12 @@ public abstract class ReferenceTypeSerializer<T>
                 // [databind#1673] Must ensure we will resolve all available type information
                 //  so as not to miss generic declaration of, say, `List<GenericPojo>`...
                 JavaType fullType = provider.constructSpecializedType(_referredType, rawType);
-                ser = provider.findValueSerializer(fullType, _property);
+                // 23-Oct-2019, tatu: I _think_ we actually need to consider referenced
+                //    type as "primary" to allow applying various handlers -- done since 2.11
+
+                ser = provider.findPrimaryPropertySerializer(fullType, _property);
             } else {
-                ser = provider.findValueSerializer(rawType, _property);
+                ser = provider.findPrimaryPropertySerializer(rawType, _property);
             }
             if (_unwrapper != null) {
                 ser = ser.unwrappingSerializer(_unwrapper);
@@ -473,6 +485,6 @@ public abstract class ReferenceTypeSerializer<T>
         // 15-Jan-2017, tatu: ... possibly because we need to access "secondary" serializer,
         //   not primary (primary being one for Reference type itself, not value)
 //        return provider.findTypedValueSerializer(type, true, prop);
-        return provider.findValueSerializer(type, prop);
+        return provider.findPrimaryPropertySerializer(type, prop);
     }
 }
