@@ -5,7 +5,6 @@ import com.fasterxml.jackson.core.JsonGenerator;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitable;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
-import com.fasterxml.jackson.databind.jsonschema.SchemaAware;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.ser.ContextualSerializer;
 import com.fasterxml.jackson.databind.ser.ResolvableSerializer;
@@ -29,7 +28,7 @@ import java.lang.reflect.Type;
 public class StdDelegatingSerializer
     extends StdSerializer<Object>
     implements ContextualSerializer, ResolvableSerializer,
-        JsonFormatVisitable, SchemaAware
+        JsonFormatVisitable
 {
     protected final Converter<Object,?> _converter;
 
@@ -37,12 +36,12 @@ public class StdDelegatingSerializer
      * Fully resolved delegate type, with generic information if any available.
      */
     protected final JavaType _delegateType;
-    
+
     /**
      * Underlying serializer for type <code>T</code>.
      */
     protected final JsonSerializer<Object> _delegateSerializer;
-    
+
     /*
     /**********************************************************
     /* Life-cycle
@@ -66,7 +65,7 @@ public class StdDelegatingSerializer
         _delegateType = null;
         _delegateSerializer = null;
     }
-    
+
     @SuppressWarnings("unchecked")
     public StdDelegatingSerializer(Converter<Object,?> converter,
             JavaType delegateType, JsonSerializer<?> delegateSerializer)
@@ -87,7 +86,7 @@ public class StdDelegatingSerializer
         ClassUtil.verifyMustOverride(StdDelegatingSerializer.class, this, "withDelegate");
         return new StdDelegatingSerializer(converter, delegateType, delegateSerializer);
     }
-    
+
     /*
     /**********************************************************
     /* Contextualization
@@ -144,7 +143,7 @@ public class StdDelegatingSerializer
     public JsonSerializer<?> getDelegatee() {
         return _delegateSerializer;
     }
-    
+
     /*
     /**********************************************************
     /* Serialization
@@ -172,10 +171,14 @@ public class StdDelegatingSerializer
     public void serializeWithType(Object value, JsonGenerator gen, SerializerProvider provider,
             TypeSerializer typeSer) throws IOException
     {
-        /* 03-Oct-2012, tatu: This is actually unlikely to work ok... but for now,
-         *    let's give it a chance?
-         */
+        // 03-Oct-2012, tatu: This is actually unlikely to work ok... but for now,
+        //    let's give it a chance?
         Object delegateValue = convertValue(value);
+        // consider null (to be consistent with serialize method above)
+        if (delegateValue == null) {
+            provider.defaultSerializeNull(gen);
+            return;
+        }
         JsonSerializer<Object> ser = _delegateSerializer;
         if (ser == null) {
             ser = _findSerializer(value, provider);
@@ -202,22 +205,32 @@ public class StdDelegatingSerializer
     /**********************************************************
      */
 
+    /**
+     * @deprecated Since 2.15
+     */
+    @Deprecated
     @Override
     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
         throws JsonMappingException
     {
-        if (_delegateSerializer instanceof SchemaAware) {
-            return ((SchemaAware) _delegateSerializer).getSchema(provider, typeHint);
+        if (_delegateSerializer instanceof com.fasterxml.jackson.databind.jsonschema.SchemaAware) {
+            return ((com.fasterxml.jackson.databind.jsonschema.SchemaAware) _delegateSerializer)
+                .getSchema(provider, typeHint);
         }
         return super.getSchema(provider, typeHint);
     }
 
+    /**
+     * @deprecated Since 2.15
+     */
+    @Deprecated
     @Override
     public JsonNode getSchema(SerializerProvider provider, Type typeHint,
         boolean isOptional) throws JsonMappingException
     {
-        if (_delegateSerializer instanceof SchemaAware) {
-            return ((SchemaAware) _delegateSerializer).getSchema(provider, typeHint, isOptional);
+        if (_delegateSerializer instanceof com.fasterxml.jackson.databind.jsonschema.SchemaAware) {
+            return ((com.fasterxml.jackson.databind.jsonschema.SchemaAware) _delegateSerializer)
+                .getSchema(provider, typeHint, isOptional);
         }
         return super.getSchema(provider, typeHint);
     }
@@ -247,9 +260,9 @@ public class StdDelegatingSerializer
      *<P>
      * The default implementation uses configured {@link Converter} to do
      * conversion.
-     * 
+     *
      * @param value Value to convert
-     * 
+     *
      * @return Result of conversion
      */
     protected Object convertValue(Object value) {
