@@ -16,13 +16,16 @@ public class BeanUtil
 {
     /*
     /**********************************************************
-    /* Handling property names
+    /* Property name mangling methods: deprecated
     /**********************************************************
      */
 
     /**
      * @since 2.5
+     *
+     * @deprecated Since 2.12 replaced with {@link com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy}
      */
+    @Deprecated
     public static String okNameForGetter(AnnotatedMethod am, boolean stdNaming) {
         String name = am.getName();
         String str = okNameForIsGetter(am, name, stdNaming);
@@ -31,10 +34,13 @@ public class BeanUtil
         }
         return str;
     }
-    
+
     /**
      * @since 2.5
+     *
+     * @deprecated Since 2.12 replaced with {@link com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy}
      */
+    @Deprecated
     public static String okNameForRegularGetter(AnnotatedMethod am, String name,
             boolean stdNaming)
     {
@@ -42,7 +48,7 @@ public class BeanUtil
             /* 16-Feb-2009, tatu: To handle [JACKSON-53], need to block
              *   CGLib-provided method "getCallbacks". Not sure of exact
              *   safe criteria to get decent coverage without false matches;
-             *   but for now let's assume there's no reason to use any 
+             *   but for now let's assume there's no reason to use any
              *   such getter from CGLib.
              *   But let's try this approach...
              */
@@ -65,7 +71,10 @@ public class BeanUtil
 
     /**
      * @since 2.5
+     *
+     * @deprecated Since 2.12 replaced with {@link com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy}
      */
+    @Deprecated
     public static String okNameForIsGetter(AnnotatedMethod am, String name,
             boolean stdNaming)
     {
@@ -80,23 +89,19 @@ public class BeanUtil
         return null;
     }
 
-    /**
-     * @since 2.5
-     */
-    @Deprecated // since 2.9, not used any more
+    // since 2.9, not used any more by databind itself but somehow seems as if
+    // it may have been used by JAXB module during 2.11
+    @Deprecated
     public static String okNameForSetter(AnnotatedMethod am, boolean stdNaming) {
-        String name = okNameForMutator(am, "set", stdNaming);
-        if ((name != null) 
-            // 26-Nov-2009, tatu: need to suppress this internal groovy method
-                && (!"metaClass".equals(name) || !isGroovyMetaClassSetter(am))) {
-            return name;
-        }
-        return null;
+        return okNameForMutator(am, "set", stdNaming);
     }
 
     /**
      * @since 2.5
+     *
+     * @deprecated Since 2.12 replaced with {@link com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy}
      */
+    @Deprecated
     public static String okNameForMutator(AnnotatedMethod am, String prefix,
             boolean stdNaming) {
         String name = am.getName();
@@ -113,7 +118,7 @@ public class BeanUtil
     /* Value defaulting helpers
     /**********************************************************
      */
-    
+
     /**
      * Accessor used to find out "default value" to use for comparing values to
      * serialize, to determine whether to exclude value from serialization with
@@ -166,7 +171,7 @@ public class BeanUtil
 
     /**
      * This method was added to address the need to weed out
-     * CGLib-injected "getCallbacks" method. 
+     * CGLib-injected "getCallbacks" method.
      * At this point caller has detected a potential getter method
      * with name "getCallbacks" and we need to determine if it is
      * indeed injectect by Cglib. We do this by verifying that the
@@ -177,44 +182,28 @@ public class BeanUtil
         Class<?> rt = am.getRawType();
         // Ok, first: must return an array type
         if (rt.isArray()) {
-            /* And that type needs to be "net.sf.cglib.proxy.Callback".
-             * Theoretically could just be a type that implements it, but
-             * for now let's keep things simple, fix if need be.
-             */
+            // And that type needs to be "net.sf.cglib.proxy.Callback".
+            // Theoretically could just be a type that implements it, but
+            // for now let's keep things simple, fix if need be.
             Class<?> compType = rt.getComponentType();
             // Actually, let's just verify it's a "net.sf.cglib.*" class/interface
-            String pkgName = ClassUtil.getPackageName(compType);
-            if (pkgName != null) {
-                if (pkgName.contains(".cglib")) {
-                    return pkgName.startsWith("net.sf.cglib")
-                        // also, as per [JACKSON-177]
-                        || pkgName.startsWith("org.hibernate.repackage.cglib")
-                        // and [core#674]
-                        || pkgName.startsWith("org.springframework.cglib");
-                }
+            final String className = compType.getName();
+            if (className.contains(".cglib")) {
+                return className.startsWith("net.sf.cglib")
+                    // also, as per [JACKSON-177]
+                    || className.startsWith("org.hibernate.repackage.cglib")
+                    // and [core#674]
+                    || className.startsWith("org.springframework.cglib");
             }
         }
         return false;
     }
 
-    /**
-     * Similar to {@link #isCglibGetCallbacks}, need to suppress
-     * a cyclic reference.
-     */
-    protected static boolean isGroovyMetaClassSetter(AnnotatedMethod am)
-    {
-        Class<?> argType = am.getRawParameterType(0);
-        String pkgName = ClassUtil.getPackageName(argType);
-        return (pkgName != null) && pkgName.startsWith("groovy.lang");
-    }
-
     /**
      * Another helper method to deal with Groovy's problematic metadata accessors
      */
-    protected static boolean isGroovyMetaClassGetter(AnnotatedMethod am)
-    {
-        String pkgName = ClassUtil.getPackageName(am.getRawType());
-        return (pkgName != null) && pkgName.startsWith("groovy.lang");
+    protected static boolean isGroovyMetaClassGetter(AnnotatedMethod am) {
+        return am.getRawType().getName().startsWith("groovy.lang");
     }
 
     /*
@@ -224,7 +213,7 @@ public class BeanUtil
      */
 
     /**
-     * Method called to figure out name of the property, given 
+     * Method called to figure out name of the property, given
      * corresponding suggested name based on a method or field name.
      *
      * @param basename Name of accessor/mutator method, not including prefix
@@ -239,7 +228,7 @@ public class BeanUtil
         // next check: is the first character upper case? If not, return as is
         char c = basename.charAt(offset);
         char d = Character.toLowerCase(c);
-        
+
         if (c == d) {
             return basename.substring(offset);
         }
@@ -260,9 +249,11 @@ public class BeanUtil
     }
 
     /**
+     * Note: public only since 2.11
+     *
      * @since 2.5
      */
-    protected static String stdManglePropertyName(final String basename, final int offset)
+    public static String stdManglePropertyName(final String basename, final int offset)
     {
         final int end = basename.length();
         if (end == offset) { // empty name, nope
@@ -287,4 +278,84 @@ public class BeanUtil
         sb.append(basename, offset+1, end);
         return sb.toString();
     }
+
+    /*
+    /**********************************************************
+    /* Package-specific type detection for error handling
+    /**********************************************************
+     */
+
+    /**
+     * Helper method called by {@link com.fasterxml.jackson.databind.deser.BeanDeserializerFactory}
+     * and {@link com.fasterxml.jackson.databind.ser.BeanSerializerFactory} to check
+     * if given unrecognized type (to be (de)serialized as general POJO) is one of
+     * "well-known" types for which there would be a datatype module; and if so,
+     * return appropriate failure message to give to caller.
+     *
+     * @since 2.12
+     */
+    public static String checkUnsupportedType(JavaType type) {
+        final String className = type.getRawClass().getName();
+        String typeName, moduleName;
+
+        if (isJava8TimeClass(className)) {
+            // [modules-java8#207]: do NOT check/block helper types in sub-packages,
+            // but only main-level types (to avoid issues with module)
+            if (className.indexOf('.', 10) >= 0) {
+                return null;
+            }
+            // [databind#4718]: Also don't worry about Exception type(s)
+            if (type.isTypeOrSubTypeOf(Throwable.class)) {
+                return null;
+            }
+            typeName =  "Java 8 date/time";
+            moduleName = "com.fasterxml.jackson.datatype:jackson-datatype-jsr310";
+        } else if (isJodaTimeClass(className)) {
+            typeName =  "Joda date/time";
+            moduleName = "com.fasterxml.jackson.datatype:jackson-datatype-joda";
+        } else if (isJava8OptionalClass(className)) {
+            typeName =  "Java 8 optional";
+            moduleName = "com.fasterxml.jackson.datatype:jackson-datatype-jdk8";
+        } else {
+            return null;
+        }
+        return String.format("%s type %s not supported by default: add Module \"%s\" to enable handling",
+                typeName, ClassUtil.getTypeDescription(type), moduleName);
+    }
+
+    /**
+     * @since 2.12
+     */
+    public static boolean isJava8TimeClass(Class<?> rawType) {
+        return isJava8TimeClass(rawType.getName());
+    }
+
+    // @since 2.12
+    private static boolean isJava8TimeClass(String className) {
+        return className.startsWith("java.time.");
+    }
+
+    /**
+     * @since 2.16
+     */
+    public static boolean isJava8OptionalClass(Class<?> rawType) {
+        return isJava8OptionalClass(rawType.getName());
+    }
+
+    // @since 2.16
+    private static boolean isJava8OptionalClass(String className) {
+        return className.startsWith("java.util.Optional");
+    }
+
+    /**
+     * @since 2.12
+     */
+    public static boolean isJodaTimeClass(Class<?> rawType) {
+        return isJodaTimeClass(rawType.getName());
+    }
+
+    // @since 2.12
+    private static boolean isJodaTimeClass(String className) {
+        return className.startsWith("org.joda.time.");
+    }
 }
