@@ -5,11 +5,8 @@ import java.util.*;
 
 import static org.junit.Assert.*;
 
-import com.fasterxml.jackson.annotation.JsonCreator;
-
 import com.fasterxml.jackson.core.*;
 
-import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
 import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.ser.std.StdScalarSerializer;
 import com.fasterxml.jackson.databind.type.TypeFactory;
@@ -18,31 +15,13 @@ public abstract class BaseMapTest
     extends BaseTest
 {
     private final static Object SINGLETON_OBJECT = new Object();
-    
+
     /*
     /**********************************************************
     /* Shared helper classes
     /**********************************************************
      */
 
-    public static class BogusSchema implements FormatSchema {
-        @Override
-        public String getSchemaType() {
-            return "TestFormat";
-        }
-    }
-
-    /**
-     * Simple wrapper around boolean types, usually to test value
-     * conversions or wrapping
-     */
-    protected static class BooleanWrapper {
-        public Boolean b;
-
-        public BooleanWrapper() { }
-        public BooleanWrapper(Boolean value) { b = value; }
-    }
-
     protected static class IntWrapper {
         public int i;
 
@@ -57,6 +36,13 @@ public abstract class BaseMapTest
         public LongWrapper(long value) { l = value; }
     }
 
+    protected static class FloatWrapper {
+        public float f;
+
+        public FloatWrapper() { }
+        public FloatWrapper(float value) { f = value; }
+    }
+
     protected static class DoubleWrapper {
         public double d;
 
@@ -77,18 +63,6 @@ public abstract class BaseMapTest
         }
     }
 
-    protected static class ObjectWrapper {
-        final Object object;
-        protected ObjectWrapper(final Object object) {
-            this.object = object;
-        }
-        public Object getObject() { return object; }
-        @JsonCreator
-        static ObjectWrapper jsonValue(final Object object) {
-            return new ObjectWrapper(object);
-        }
-    }
-
     protected static class ListWrapper<T>
     {
         public List<T> list;
@@ -114,7 +88,7 @@ public abstract class BaseMapTest
             map.put(key, value);
         }
     }
-    
+
     protected static class ArrayWrapper<T>
     {
         public T[] array;
@@ -123,17 +97,6 @@ public abstract class BaseMapTest
             array = v;
         }
     }
-    
-    /**
-     * Enumeration type with sub-classes per value.
-     */
-    protected enum EnumWithSubClass {
-        A { @Override public void foobar() { } }
-        ,B { @Override public void foobar() { } }
-        ;
-
-        public abstract void foobar();
-    }
 
     public enum ABC { A, B, C; }
 
@@ -146,7 +109,7 @@ public abstract class BaseMapTest
             x = x0;
             y = y0;
         }
-    
+
         @Override
         public boolean equals(Object o) {
             if (!(o instanceof Point)) {
@@ -180,24 +143,12 @@ public abstract class BaseMapTest
         }
     }
 
-    @SuppressWarnings("serial")
-    public static class LowerCasingDeserializer extends StdScalarDeserializer<String>
-    {
-        public LowerCasingDeserializer() { super(String.class); }
-
-        @Override
-        public String deserialize(JsonParser p, DeserializationContext ctxt)
-                throws IOException {
-            return p.getText().toLowerCase();
-        }
-    }
-
     /*
     /**********************************************************
     /* Construction
     /**********************************************************
      */
-    
+
     protected BaseMapTest() { super(); }
 
     /*
@@ -226,18 +177,16 @@ public abstract class BaseMapTest
     protected ObjectReader objectReader() {
         return sharedMapper().reader();
     }
-    
-    protected ObjectReader objectReader(Class<?> cls) {
-        return sharedMapper().readerFor(cls);
-    }
 
+    // `public` since 2.16, was only `protected` before then.
     // @since 2.10
-    protected static ObjectMapper newJsonMapper() {
+    public static ObjectMapper newJsonMapper() {
         return new JsonMapper();
     }
 
+    // `public` since 2.16, was only `protected` before then.
     // @since 2.10
-    protected static JsonMapper.Builder jsonMapperBuilder() {
+    public static JsonMapper.Builder jsonMapperBuilder() {
         return JsonMapper.builder();
     }
 
@@ -290,7 +239,7 @@ public abstract class BaseMapTest
         String str = m.writeValueAsString(value);
         return (Map<String,Object>) m.readValue(str, LinkedHashMap.class);
     }
-    
+
     protected String serializeAsString(ObjectMapper m, Object value)
         throws IOException
     {
@@ -324,7 +273,7 @@ public abstract class BaseMapTest
     /* Helper methods, deserialization
     /**********************************************************
      */
-    
+
     protected <T> T readAndMapFromString(String input, Class<T> cls)
         throws IOException
     {
@@ -342,10 +291,6 @@ public abstract class BaseMapTest
     /**********************************************************
      */
 
-    protected TimeZone getUTCTimeZone() {
-        return TimeZone.getTimeZone("GMT");
-    }
-
     protected byte[] utf8Bytes(String str) {
         try {
             return str.getBytes("UTF-8");
