@@ -3,22 +3,31 @@ package com.fasterxml.jackson.databind;
 import java.io.*;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonSetter;
 import com.fasterxml.jackson.annotation.Nulls;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.json.JsonWriteFeature;
 import com.fasterxml.jackson.core.util.MinimalPrettyPrinter;
-
+import com.fasterxml.jackson.databind.cfg.EnumFeature;
 import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
 import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
+import com.fasterxml.jackson.databind.json.JsonMapper;
+import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.databind.node.*;
 
-public class ObjectMapperTest extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
+public class ObjectMapperTest
 {
     static class Bean {
         int value = 3;
-        
+
         public void setX(int v) { value = v; }
 
         protected Bean() { }
@@ -48,7 +57,7 @@ public class ObjectMapperTest extends BaseMapTest
     @SuppressWarnings("serial")
     static class NoCopyMapper extends ObjectMapper { }
 
-    final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     /*
     /**********************************************************
@@ -56,32 +65,43 @@ public class ObjectMapperTest extends BaseMapTest
     /**********************************************************
      */
 
-    public void testFactorFeatures()
+    @Test
+    public void testFactoryFeatures()
     {
         assertTrue(MAPPER.isEnabled(JsonFactory.Feature.CANONICALIZE_FIELD_NAMES));
     }
 
+    @Test
     public void testGeneratorFeatures()
     {
         // and also for mapper
-        ObjectMapper mapper = new ObjectMapper();
-        assertFalse(mapper.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
-        assertTrue(mapper.isEnabled(JsonGenerator.Feature.QUOTE_FIELD_NAMES));
-        mapper.disable(JsonGenerator.Feature.FLUSH_PASSED_TO_STREAM,
-                JsonGenerator.Feature.QUOTE_FIELD_NAMES);
+        JsonMapper mapper = new JsonMapper();
+        assertTrue(mapper.isEnabled(JsonGenerator.Feature.AUTO_CLOSE_TARGET));
+        assertTrue(mapper.isEnabled(StreamWriteFeature.AUTO_CLOSE_TARGET));
+        assertFalse(mapper.isEnabled(JsonWriteFeature.ESCAPE_NON_ASCII));
+        assertTrue(mapper.isEnabled(JsonWriteFeature.WRITE_NAN_AS_STRINGS));
+        mapper = JsonMapper.builder()
+                .disable(StreamWriteFeature.FLUSH_PASSED_TO_STREAM)
+                .disable(JsonWriteFeature.WRITE_NAN_AS_STRINGS)
+                .build();
+        assertFalse(mapper.isEnabled(StreamWriteFeature.FLUSH_PASSED_TO_STREAM));
+        assertFalse(mapper.isEnabled(JsonWriteFeature.WRITE_NAN_AS_STRINGS));
     }
 
+    @Test
     public void testParserFeatures()
     {
         // and also for mapper
         ObjectMapper mapper = new ObjectMapper();
-                
+
         assertTrue(mapper.isEnabled(JsonParser.Feature.AUTO_CLOSE_SOURCE));
-        assertFalse(mapper.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        assertTrue(mapper.isEnabled(StreamReadFeature.AUTO_CLOSE_SOURCE));
+        assertFalse(mapper.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
 
         mapper.disable(JsonParser.Feature.AUTO_CLOSE_SOURCE,
                 JsonParser.Feature.STRICT_DUPLICATE_DETECTION);
         assertFalse(mapper.isEnabled(JsonParser.Feature.AUTO_CLOSE_SOURCE));
+        assertFalse(mapper.isEnabled(StreamReadFeature.AUTO_CLOSE_SOURCE));
     }
 
     /*
@@ -91,6 +111,7 @@ public class ObjectMapperTest extends BaseMapTest
      */
 
     // [databind#28]: ObjectMapper.copy()
+    @Test
     public void testCopy() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
@@ -99,12 +120,12 @@ public class ObjectMapperTest extends BaseMapTest
         assertFalse(m.isEnabled(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES));
         InjectableValues inj = new InjectableValues.Std();
         m.setInjectableValues(inj);
-        assertFalse(m.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
-        m.enable(JsonParser.Feature.ALLOW_COMMENTS);
-        assertTrue(m.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        assertFalse(m.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
+        m.enable(JsonParser.Feature.IGNORE_UNDEFINED);
+        assertTrue(m.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
 
         // // First: verify that handling of features is decoupled:
-        
+
         ObjectMapper m2 = m.copy();
         assertFalse(m2.isEnabled(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES));
         m2.enable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
@@ -122,9 +143,11 @@ public class ObjectMapperTest extends BaseMapTest
         assertFalse(m2.isEnabled(DeserializationFeature.UNWRAP_ROOT_VALUE));
 
         // // Also, underlying JsonFactory instances should be distinct
-        
         assertNotSame(m.getFactory(), m2.getFactory());
 
+        // [databind#2755]: also need to copy this:
+        assertNotSame(m.getSubtypeResolver(), m2.getSubtypeResolver());
+
         // [databind#122]: Need to ensure mix-ins are not shared
         assertEquals(0, m.getSerializationConfig().mixInCount());
         assertEquals(0, m2.getSerializationConfig().mixInCount());
@@ -138,10 +161,11 @@ public class ObjectMapperTest extends BaseMapTest
         assertEquals(0, m2.getDeserializationConfig().mixInCount());
 
         // [databind#913]: Ensure JsonFactory Features copied
-        assertTrue(m2.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        assertTrue(m2.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
     }
 
     // [databind#1580]
+    @Test
     public void testCopyOfConfigOverrides() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
@@ -172,6 +196,51 @@ public class ObjectMapperTest extends BaseMapTest
         assertSame(customVis, config2.getDefaultVisibilityChecker());
     }
 
+    @Test
+    public void testCopyWith() throws Exception {
+        ObjectMapper mapper = new ObjectMapper();
+        //configuring some settings to non-defaults
+        mapper.configure(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES, true);
+        mapper.configure(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL, true);
+        mapper.configure(SerializationFeature.INDENT_OUTPUT, true);
+        mapper.configure(SerializationFeature.FAIL_ON_SELF_REFERENCES, true);
+        JsonFactory newFactory = JsonFactory.builder()
+            .configure(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING, false)
+            .build();
+        ObjectMapper copiedMapper = mapper.copyWith(newFactory);
+        String json = "{ \"color\" : \"Black\", \"free\" : \"true\", \"pages\" : \"204.04\" }";
+        JsonNode readResult = copiedMapper.readTree(json);
+        // validate functionality
+        assertEquals("Black", readResult.get("color").asText());
+        assertEquals(true, readResult.get("free").asBoolean());
+        assertEquals(204, readResult.get("pages").asInt());
+        String readResultAsString = _unifyLFs("{\n  \"color\" : \"Black\",\n  \"free\" : \"true\",\n  \"pages\" : \"204.04\"\n}");
+        assertEquals(readResultAsString, _unifyLFs(mapper.writeValueAsString(readResult)));
+
+        // validate properties
+        Boolean mapperConfig1 = mapper._deserializationConfig.isEnabled(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES);
+        Boolean copiedMapperConfig1 = copiedMapper._deserializationConfig.isEnabled(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES);
+        Boolean mapperConfig2 = mapper._deserializationConfig.isEnabled(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL);
+        Boolean copiedMapperConfig2 = copiedMapper._deserializationConfig.isEnabled(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL);
+        Boolean mapperConfig3 = mapper._serializationConfig.isEnabled(SerializationFeature.INDENT_OUTPUT);
+        Boolean copiedMapperConfig3 = copiedMapper._serializationConfig.isEnabled(SerializationFeature.INDENT_OUTPUT);
+        Boolean mapperConfig4 = mapper._serializationConfig.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES);
+        Boolean copiedMapperConfig4 = copiedMapper._serializationConfig.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES);
+        assertNotSame(mapper.getFactory(), copiedMapper.getFactory());
+        assertSame(mapperConfig1, copiedMapperConfig1);
+        assertSame(mapperConfig2, copiedMapperConfig2);
+        assertSame(mapperConfig3, copiedMapperConfig3);
+        assertSame(mapperConfig4, copiedMapperConfig4);
+        assertNotSame(mapper.getFactory().isEnabled(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING),
+            copiedMapper.getFactory().isEnabled(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING)
+        );
+    }
+
+    private String _unifyLFs(String doc) {
+        return doc.replaceAll("[\r\n]+", "\n");
+    }
+
+    @Test
     public void testFailedCopy() throws Exception
     {
         NoCopyMapper src = new NoCopyMapper();
@@ -183,7 +252,8 @@ public class ObjectMapperTest extends BaseMapTest
         }
     }
 
-    public void testAnnotationIntrospectorCopyin() 
+    @Test
+    public void testAnnotationIntrospectorCopying()
     {
         ObjectMapper m = new ObjectMapper();
         m.setAnnotationIntrospector(new MyAnnotationIntrospector());
@@ -197,12 +267,34 @@ public class ObjectMapperTest extends BaseMapTest
                 m2.getSerializationConfig().getAnnotationIntrospector().getClass());
     }
 
+    /*
+    /**********************************************************
+    /* Test methods, JsonMapper.rebuild()
+    /**********************************************************
+     */
+
+    @Test
+    public void jsonMapperRebuildTest()
+    {
+        JsonMapper m = JsonMapper.builder().build();
+        JsonMapper m2 = m.copy();
+        assertNotSame(m, m2);
+
+        JsonMapper m3 = m2.rebuild()
+                .propertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE)
+                .enable(MapperFeature.ACCEPT_CASE_INSENSITIVE_ENUMS)
+                .enable(EnumFeature.WRITE_ENUMS_TO_LOWERCASE)
+                .build();
+        assertNotSame(m2, m3);
+    }
+
     /*
     /**********************************************************
     /* Test methods, other
     /**********************************************************
      */
 
+    @Test
     public void testProps()
     {
         ObjectMapper m = new ObjectMapper();
@@ -215,10 +307,11 @@ public class ObjectMapperTest extends BaseMapTest
     }
 
     // Test to ensure that we can check property ordering defaults...
+    @Test
     public void testConfigForPropertySorting() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
-        
+
         // sort-alphabetically is disabled by default:
         assertFalse(m.isEnabled(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY));
         SerializationConfig sc = m.getSerializationConfig();
@@ -228,7 +321,9 @@ public class ObjectMapperTest extends BaseMapTest
         assertFalse(dc.shouldSortPropertiesAlphabetically());
 
         // but when enabled, should be visible:
-        m.enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY);
+        m = jsonMapperBuilder()
+                .enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY)
+                .build();
         sc = m.getSerializationConfig();
         assertTrue(sc.isEnabled(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY));
         assertTrue(sc.shouldSortPropertiesAlphabetically());
@@ -238,7 +333,31 @@ public class ObjectMapperTest extends BaseMapTest
         assertTrue(dc.shouldSortPropertiesAlphabetically());
     }
 
+    // Test to ensure that we can check forced property ordering defaults...
+    @Test
+    public void testConfigForForcedPropertySorting() throws Exception
+    {
+        ObjectMapper m = new ObjectMapper();
 
+        // sort-alphabetically is disabled by default:
+        assertTrue(m.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        SerializationConfig sc = m.getSerializationConfig();
+        assertTrue(sc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        DeserializationConfig dc = m.getDeserializationConfig();
+        assertTrue(dc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+
+        // but when enabled, should be visible:
+        m = jsonMapperBuilder()
+                .disable(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST)
+                .build();
+        sc = m.getSerializationConfig();
+        assertFalse(sc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        dc = m.getDeserializationConfig();
+        // and not just via SerializationConfig, but also via DeserializationConfig
+        assertFalse(dc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+    }
+
+    @Test
     public void testJsonFactoryLinkage()
     {
         // first, implicit factory, giving implicit linkage
@@ -251,7 +370,8 @@ public class ObjectMapperTest extends BaseMapTest
         assertSame(m, f.getCodec());
     }
 
-    public void testProviderConfig() throws Exception   
+    @Test
+    public void testProviderConfig() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
         final String JSON = "{ \"x\" : 3 }";
@@ -275,6 +395,7 @@ public class ObjectMapperTest extends BaseMapTest
     }
 
     // For [databind#689]
+    @Test
     public void testCustomDefaultPrettyPrinter() throws Exception
     {
         final ObjectMapper m = new ObjectMapper();
@@ -299,8 +420,9 @@ public class ObjectMapperTest extends BaseMapTest
         assertEquals("[1,2]", m.writer().without(SerializationFeature.INDENT_OUTPUT)
                 .writeValueAsString(input));
     }
-    
+
     // For [databind#703], [databind#978]
+    @Test
     public void testNonSerializabilityOfObject()
     {
         ObjectMapper m = new ObjectMapper();
@@ -319,6 +441,7 @@ public class ObjectMapperTest extends BaseMapTest
     }
 
     // for [databind#756]
+    @Test
     public void testEmptyBeanSerializability()
     {
         // with default settings, error
@@ -329,7 +452,16 @@ public class ObjectMapperTest extends BaseMapTest
                 .canSerialize(EmptyBean.class));
     }
 
+    // for [databind#2749]: just to check there's no NPE; method really not useful
+    @Test
+    public void testCanDeserialize()
+    {
+        assertTrue(MAPPER.canDeserialize(MAPPER.constructType(EmptyBean.class)));
+        assertTrue(MAPPER.canDeserialize(MAPPER.constructType(Object.class)));
+    }
+
     // for [databind#898]
+    @Test
     public void testSerializerProviderAccess() throws Exception
     {
         // ensure we have "fresh" instance, just in case
@@ -341,43 +473,48 @@ public class ObjectMapperTest extends BaseMapTest
     }
 
     // for [databind#1074]
+    @Test
     public void testCopyOfParserFeatures() throws Exception
     {
         // ensure we have "fresh" instance to start with
         ObjectMapper mapper = new ObjectMapper();
-        assertFalse(mapper.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
-        mapper.configure(JsonParser.Feature.ALLOW_COMMENTS, true);
-        assertTrue(mapper.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        assertFalse(mapper.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
+        mapper.configure(JsonParser.Feature.IGNORE_UNDEFINED, true);
+        assertTrue(mapper.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
 
         ObjectMapper copy = mapper.copy();
-        assertTrue(copy.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        assertTrue(copy.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
 
         // also verify there's no back-linkage
-        copy.configure(JsonParser.Feature.ALLOW_COMMENTS, false);
-        assertFalse(copy.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
-        assertTrue(mapper.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
+        copy.configure(JsonParser.Feature.IGNORE_UNDEFINED, false);
+        assertFalse(copy.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
+        assertTrue(mapper.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
     }
 
     // since 2.8
+    @Test
     public void testDataOutputViaMapper() throws Exception
     {
         ByteArrayOutputStream bytes = new ByteArrayOutputStream();
         ObjectNode input = MAPPER.createObjectNode();
         input.put("a", 1);
-        DataOutput data = new DataOutputStream(bytes);
         final String exp = "{\"a\":1}";
-        MAPPER.writeValue(data, input);
+        try (DataOutputStream data = new DataOutputStream(bytes)) {
+            MAPPER.writeValue((DataOutput) data, input);
+        }
         assertEquals(exp, bytes.toString("UTF-8"));
 
         // and also via ObjectWriter...
         bytes.reset();
-        data = new DataOutputStream(bytes);
-        MAPPER.writer().writeValue(data, input);
+        try (DataOutputStream data = new DataOutputStream(bytes)) {
+            MAPPER.writer().writeValue((DataOutput) data, input);
+        }
         assertEquals(exp, bytes.toString("UTF-8"));
     }
 
     // since 2.8
     @SuppressWarnings("unchecked")
+    @Test
     public void testDataInputViaMapper() throws Exception
     {
         byte[] src = "{\"a\":1}".getBytes("UTF-8");
@@ -396,4 +533,89 @@ public class ObjectMapperTest extends BaseMapTest
                 .readTree(input);
         assertNotNull(n);
     }
+
+    @SuppressWarnings("serial")
+    @Test
+    public void testRegisterDependentModules() {
+        ObjectMapper objectMapper = newJsonMapper();
+
+        final SimpleModule secondModule = new SimpleModule() {
+            @Override
+            public Object getTypeId() {
+                return "second";
+            }
+        };
+
+        final SimpleModule thirdModule = new SimpleModule() {
+            @Override
+            public Object getTypeId() {
+                return "third";
+            }
+        };
+
+        final SimpleModule firstModule = new SimpleModule() {
+            @Override
+            public Iterable<? extends Module> getDependencies() {
+                return Arrays.asList(secondModule, thirdModule);
+            }
+
+            @Override
+            public Object getTypeId() {
+                return "main";
+            }
+        };
+
+        objectMapper.registerModule(firstModule);
+
+        assertEquals(
+            new HashSet<>(Arrays.asList("second", "third", "main")),
+            objectMapper.getRegisteredModuleIds()
+        );
+    }
+
+    // since 2.12
+    @Test
+    public void testHasExplicitTimeZone() throws Exception
+    {
+        final TimeZone DEFAULT_TZ = TimeZone.getTimeZone("UTC");
+
+        // By default, not explicitly set
+        assertFalse(MAPPER.getSerializationConfig().hasExplicitTimeZone());
+        assertFalse(MAPPER.getDeserializationConfig().hasExplicitTimeZone());
+        assertEquals(DEFAULT_TZ, MAPPER.getSerializationConfig().getTimeZone());
+        assertEquals(DEFAULT_TZ, MAPPER.getDeserializationConfig().getTimeZone());
+        assertFalse(MAPPER.reader().getConfig().hasExplicitTimeZone());
+        assertFalse(MAPPER.writer().getConfig().hasExplicitTimeZone());
+
+        final TimeZone TZ = TimeZone.getTimeZone("GMT+4");
+
+        // should be able to set it via mapper
+        ObjectMapper mapper = JsonMapper.builder()
+                .defaultTimeZone(TZ)
+                .build();
+        assertSame(TZ, mapper.getSerializationConfig().getTimeZone());
+        assertSame(TZ, mapper.getDeserializationConfig().getTimeZone());
+        assertTrue(mapper.getSerializationConfig().hasExplicitTimeZone());
+        assertTrue(mapper.getDeserializationConfig().hasExplicitTimeZone());
+        assertTrue(mapper.reader().getConfig().hasExplicitTimeZone());
+        assertTrue(mapper.writer().getConfig().hasExplicitTimeZone());
+
+        // ... as well as via ObjectReader/-Writer
+        {
+            final ObjectReader r = MAPPER.reader().with(TZ);
+            assertTrue(r.getConfig().hasExplicitTimeZone());
+            assertSame(TZ, r.getConfig().getTimeZone());
+            final ObjectWriter w = MAPPER.writer().with(TZ);
+            assertTrue(w.getConfig().hasExplicitTimeZone());
+            assertSame(TZ, w.getConfig().getTimeZone());
+
+            // but can also remove explicit definition
+            final ObjectReader r2 = r.with((TimeZone) null);
+            assertFalse(r2.getConfig().hasExplicitTimeZone());
+            assertEquals(DEFAULT_TZ, r2.getConfig().getTimeZone());
+            final ObjectWriter w2 = w.with((TimeZone) null);
+            assertFalse(w2.getConfig().hasExplicitTimeZone());
+            assertEquals(DEFAULT_TZ, w2.getConfig().getTimeZone());
+        }
+    }
 }
