@@ -6,6 +6,8 @@ import java.io.IOException;
 import java.io.StringWriter;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.annotation.JsonTypeName;
 
@@ -14,19 +16,24 @@ import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.core.json.JsonWriteFeature;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.a2q;
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.BogusSchema;
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.verifyException;
+
 /**
  * Unit tests for checking features added to {@link ObjectWriter}, such
  * as adding of explicit pretty printer.
  */
 public class ObjectWriterTest
-    extends BaseMapTest
 {
     static class CloseableValue implements Closeable
     {
         public int x;
 
         public boolean closed;
-        
+
         @Override
         public void close() throws IOException {
             closed = true;
@@ -42,14 +49,14 @@ public class ObjectWriterTest
     @JsonTypeName("A")
     static class ImplA extends PolyBase {
         public int value;
-        
+
         public ImplA(int v) { value = v; }
     }
 
     @JsonTypeName("B")
     static class ImplB extends PolyBase {
         public int b;
-        
+
         public ImplB(int v) { b = v; }
     }
 
@@ -59,12 +66,13 @@ public class ObjectWriterTest
     /**********************************************************
      */
 
+    @Test
     public void testPrettyPrinter() throws Exception
     {
         ObjectWriter writer = MAPPER.writer();
         HashMap<String, Integer> data = new HashMap<String,Integer>();
         data.put("a", 1);
-        
+
         // default: no indentation
         assertEquals("{\"a\":1}", writer.writeValueAsString(data));
 
@@ -80,6 +88,7 @@ public class ObjectWriterTest
         assertEquals("{\"a\":1}", writer.writeValueAsString(data));
     }
 
+    @Test
     public void testPrefetch() throws Exception
     {
         ObjectWriter writer = MAPPER.writer();
@@ -88,10 +97,11 @@ public class ObjectWriterTest
         assertTrue(writer.hasPrefetchedSerializer());
     }
 
+    @Test
     public void testObjectWriterFeatures() throws Exception
     {
         ObjectWriter writer = MAPPER.writer()
-                .without(JsonWriteFeature.QUOTE_FIELD_NAMES);                
+                .without(JsonWriteFeature.QUOTE_FIELD_NAMES);
         Map<String,Integer> map = new HashMap<String,Integer>();
         map.put("a", 1);
         assertEquals("{a:1}", writer.writeValueAsString(map));
@@ -100,6 +110,7 @@ public class ObjectWriterTest
                 .writeValueAsString(map));
     }
 
+    @Test
     public void testObjectWriterWithNode() throws Exception
     {
         ObjectNode stuff = MAPPER.createObjectNode();
@@ -109,23 +120,26 @@ public class ObjectWriterTest
         assertEquals("{\"a\":5}", json);
     }
 
+    @Test
     public void testPolymorphicWithTyping() throws Exception
     {
         ObjectWriter writer = MAPPER.writerFor(PolyBase.class);
         String json;
 
         json = writer.writeValueAsString(new ImplA(3));
-        assertEquals(aposToQuotes("{'type':'A','value':3}"), json);
+        assertEquals(a2q("{'type':'A','value':3}"), json);
         json = writer.writeValueAsString(new ImplB(-5));
-        assertEquals(aposToQuotes("{'type':'B','b':-5}"), json);
+        assertEquals(a2q("{'type':'B','b':-5}"), json);
     }
 
+    @Test
     public void testCanSerialize() throws Exception
     {
         assertTrue(MAPPER.writer().canSerialize(String.class));
         assertTrue(MAPPER.writer().canSerialize(String.class, null));
     }
 
+    @Test
     public void testNoPrefetch() throws Exception
     {
         ObjectWriter w = MAPPER.writer()
@@ -136,6 +150,7 @@ public class ObjectWriterTest
         assertEquals("3", out.toString("UTF-8"));
     }
 
+    @Test
     public void testWithCloseCloseable() throws Exception
     {
         ObjectWriter w = MAPPER.writer()
@@ -149,7 +164,7 @@ public class ObjectWriterTest
         input.close();
 
         // and via explicitly passed generator
-        JsonGenerator g = MAPPER.getFactory().createGenerator(new StringWriter());
+        JsonGenerator g = MAPPER.createGenerator(new StringWriter());
         input = new CloseableValue();
         assertFalse(input.closed);
         w.writeValue(g, input);
@@ -158,6 +173,7 @@ public class ObjectWriterTest
         input.close();
     }
 
+    @Test
     public void testViewSettings() throws Exception
     {
         ObjectWriter w = MAPPER.writer();
@@ -170,6 +186,7 @@ public class ObjectWriterTest
         assertSame(newW, newW.with(Locale.CANADA));
     }
 
+    @Test
     public void testMiscSettings() throws Exception
     {
         ObjectWriter w = MAPPER.writer();
@@ -183,7 +200,7 @@ public class ObjectWriterTest
         ObjectWriter newW = w.with(Base64Variants.MODIFIED_FOR_URL);
         assertNotSame(w, newW);
         assertSame(newW, newW.with(Base64Variants.MODIFIED_FOR_URL));
-        
+
         w = w.withAttributes(Collections.emptyMap());
         w = w.withAttribute("a", "b");
         assertEquals("b", w.getAttributes().getAttribute("a"));
@@ -199,10 +216,11 @@ public class ObjectWriterTest
         }
     }
 
+    @Test
     public void testRootValueSettings() throws Exception
     {
         ObjectWriter w = MAPPER.writer();
-        
+
         // First, root name:
         ObjectWriter newW = w.withRootName("foo");
         assertNotSame(w, newW);
@@ -229,11 +247,14 @@ public class ObjectWriterTest
         assertNotSame(w, newW);
     }
 
+    @Test
     public void testFeatureSettings() throws Exception
     {
         ObjectWriter w = MAPPER.writer();
         assertFalse(w.isEnabled(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES));
         assertFalse(w.isEnabled(JsonGenerator.Feature.STRICT_DUPLICATE_DETECTION));
+        assertFalse(w.isEnabled(StreamWriteFeature.STRICT_DUPLICATE_DETECTION));
+
         ObjectWriter newW = w.with(SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS,
                 SerializationFeature.INDENT_OUTPUT);
         assertNotSame(w, newW);
@@ -258,6 +279,7 @@ public class ObjectWriterTest
                 SerializationFeature.EAGER_SERIALIZER_FETCH));
     }
 
+    @Test
     public void testGeneratorFeatures() throws Exception
     {
         ObjectWriter w = MAPPER.writer();
@@ -267,14 +289,18 @@ public class ObjectWriterTest
         assertTrue(w.isEnabled(JsonGenerator.Feature.AUTO_CLOSE_TARGET));
         assertNotSame(w, w.without(JsonGenerator.Feature.AUTO_CLOSE_TARGET));
         assertNotSame(w, w.withoutFeatures(JsonGenerator.Feature.AUTO_CLOSE_TARGET));
+
+        assertFalse(w.isEnabled(StreamWriteFeature.STRICT_DUPLICATE_DETECTION));
+        assertNotSame(w, w.with(StreamWriteFeature.STRICT_DUPLICATE_DETECTION));
     }
-    
+
     /*
     /**********************************************************
     /* Test methods, failures
     /**********************************************************
      */
 
+    @Test
     public void testArgumentChecking() throws Exception
     {
         final ObjectWriter w = MAPPER.writer();
@@ -286,6 +312,7 @@ public class ObjectWriterTest
         }
     }
 
+    @Test
     public void testSchema() throws Exception
     {
         try {
