@@ -1,11 +1,17 @@
 package com.fasterxml.jackson.databind.cfg;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 
-public class DatabindContextTest extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.assertNull;
+
+public class DatabindContextTest
 {
-    private final ObjectMapper MAPPER = objectMapper();
+    private final ObjectMapper MAPPER = new JsonMapper();
 
+    @Test
     public void testDeserializationContext() throws Exception
     {
         DeserializationContext ctxt = MAPPER.getDeserializationContext();
@@ -14,6 +20,7 @@ public class DatabindContextTest extends BaseMapTest
         assertNull(ctxt.constructType((java.lang.reflect.Type) null));
     }
 
+    @Test
     public void testSerializationContext() throws Exception
     {
         SerializerProvider ctxt = MAPPER.getSerializerProvider();
