@@ -5,6 +5,8 @@ import java.io.IOException;
 import java.util.List;
 import java.util.Map;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
@@ -13,7 +15,12 @@ import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
 import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 import com.fasterxml.jackson.databind.exc.ValueInstantiationException;
 
-public class TestCreators2 extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.q;
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.verifyException;
+
+public class TestCreators2
 {
     static class HashTest
     {
@@ -43,7 +50,7 @@ public class TestCreators2 extends BaseMapTest
         protected int x = 3;
         protected double d = -0.5;
         protected boolean b = true;
-        
+
         @JsonCreator
         public Primitives(@JsonProperty("x") int x,
                 @JsonProperty("d") double d,
@@ -54,7 +61,7 @@ public class TestCreators2 extends BaseMapTest
             this.b = b;
         }
     }
-    
+
     protected static class Test431Container {
         protected final List<Item431> items;
 
@@ -62,7 +69,7 @@ public class TestCreators2 extends BaseMapTest
         public Test431Container(@JsonProperty("items") final List<Item431> i) {
             items = i;
         }
-    }    
+    }
 
     @JsonIgnoreProperties(ignoreUnknown = true)
     protected static class Item431 {
@@ -86,13 +93,13 @@ public class TestCreators2 extends BaseMapTest
     static class BrokenCreatorBean
     {
         protected String bar;
-        
+
         @JsonCreator
         public BrokenCreatorBean(@JsonProperty("bar") String bar1, @JsonProperty("bar") String bar2) {
             bar = ""+bar1+"/"+bar2;
         }
     }
-    
+
     // For [JACKSON-541]: should not need @JsonCreator if SerializationFeature.AUTO_DETECT_CREATORS is on.
     static class AutoDetectConstructorBean
     {
@@ -134,12 +141,12 @@ public class TestCreators2 extends BaseMapTest
     static class AbstractBaseImpl extends AbstractBase
     {
         protected Map<String,Object> props;
-        
+
         public AbstractBaseImpl(Map<String,Object> props) {
             this.props = props;
         }
     }
-    
+
     static interface Issue700Set extends java.util.Set<Object> { }
 
     static class Issue700Bean
@@ -188,6 +195,7 @@ public class TestCreators2 extends BaseMapTest
 
     private final ObjectMapper MAPPER = new ObjectMapper();
 
+    @Test
     public void testExceptionFromConstructor() throws Exception
     {
         try {
@@ -208,6 +216,7 @@ public class TestCreators2 extends BaseMapTest
         }
     }
 
+    @Test
     public void testSimpleConstructor() throws Exception
     {
         HashTest test = MAPPER.readValue("{\"type\":\"custom\",\"bytes\":\"abc\" }", HashTest.class);
@@ -215,6 +224,7 @@ public class TestCreators2 extends BaseMapTest
         assertEquals("abc", new String(test.bytes, "UTF-8"));
     }
 
+    @Test
     public void testMissingPrimitives() throws Exception
     {
         Primitives p = MAPPER.readValue("{}", Primitives.class);
@@ -223,26 +233,28 @@ public class TestCreators2 extends BaseMapTest
         assertEquals(0.0, p.d);
     }
 
+    @Test
     public void testJackson431() throws Exception
     {
         final Test431Container foo = MAPPER.readValue(
                 "{\"items\":\n"
                 +"[{\"bar\": 0,\n"
                 +"\"id\": \"id123\",\n"
-                +"\"foo\": 1\n" 
+                +"\"foo\": 1\n"
                 +"}]}",
                 Test431Container.class);
         assertNotNull(foo);
     }
 
     // Catch and re-throw exceptions that Creator methods throw
+    @Test
     public void testJackson438() throws Exception
     {
         Exception e = null;
         try {
             MAPPER.readValue("{ \"name\":\"foobar\" }", BeanFor438.class);
             fail("Should have failed");
-        } catch (JsonMappingException e0) {
+        } catch (Exception e0) {
             e = e0;
         }
         if (!(e instanceof ValueInstantiationException)) {
@@ -258,6 +270,7 @@ public class TestCreators2 extends BaseMapTest
         verifyException(e, "don't like that name");
     }
 
+    @Test
     public void testCreatorWithDupNames() throws Exception
     {
         try {
@@ -270,6 +283,7 @@ public class TestCreators2 extends BaseMapTest
         }
     }
 
+    @Test
     public void testCreatorMultipleArgumentWithoutAnnotation() throws Exception {
         AutoDetectConstructorBean value = MAPPER.readValue("{\"bar\":\"bar\",\"foo\":\"foo\"}",
                 AutoDetectConstructorBean.class);
@@ -277,16 +291,18 @@ public class TestCreators2 extends BaseMapTest
         assertEquals("foo", value.foo);
     }
 
+    @Test
     public void testIgnoredSingleArgCtor() throws Exception
     {
         try {
-            MAPPER.readValue(quote("abc"), IgnoredCtor.class);
+            MAPPER.readValue(q("abc"), IgnoredCtor.class);
             fail("Should have caught missing constructor problem");
         } catch (MismatchedInputException e) {
             verifyException(e, "no String-argument constructor/factory method");
         }
     }
 
+    @Test
     public void testAbstractFactory() throws Exception
     {
         AbstractBase bean = MAPPER.readValue("{\"a\":3}", AbstractBase.class);
@@ -296,6 +312,7 @@ public class TestCreators2 extends BaseMapTest
         assertEquals(Integer.valueOf(3), impl.props.get("a"));
     }
 
+    @Test
     public void testCreatorProperties() throws Exception
     {
         Issue700Bean value = MAPPER.readValue("{ \"item\" : \"foo\" }", Issue700Bean.class);
@@ -303,6 +320,7 @@ public class TestCreators2 extends BaseMapTest
     }
 
     // [databind#1476]
+    @Test
     public void testConstructorChoice() throws Exception {
         ObjectMapper mapper = new ObjectMapper();
         MultiPropCreator1476 pojo = mapper.readValue("{ \"intField\": 1, \"stringField\": \"foo\" }",
