@@ -1,13 +1,17 @@
 package com.fasterxml.jackson.databind.deser.creators;
 
 import com.fasterxml.jackson.annotation.*;
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.databind.*;
 
-public class TestPolymorphicDelegating extends BaseMapTest
-{
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 
+public class TestPolymorphicDelegating
+{
     // For [databind#580]
-    
+
     @JsonTypeInfo(use = JsonTypeInfo.Id.CLASS)
     static abstract class Issue580Base {
     }
@@ -39,12 +43,12 @@ public class TestPolymorphicDelegating extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testAbstractDelegateWithCreator() throws Exception
     {
         Issue580Bean input = new Issue580Bean(new Issue580Impl(13));
         ObjectMapper mapper = new ObjectMapper();
         String json = mapper.writeValueAsString(input);
-
         Issue580Bean result = mapper.readValue(json, Issue580Bean.class);
         assertNotNull(result);
         assertNotNull(result.value);
