@@ -1,9 +1,9 @@
 package com.fasterxml.jackson.databind.deser.creators;
 
 import java.io.IOException;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Map;
+import java.util.*;
+
+import org.junit.jupiter.api.Test;
 
 import com.fasterxml.jackson.core.Version;
 import com.fasterxml.jackson.databind.*;
@@ -14,11 +14,15 @@ import com.fasterxml.jackson.databind.introspect.AnnotatedWithParams;
 import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 /**
  * Test custom value instantiators.
  */
 @SuppressWarnings("serial")
-public class TestValueInstantiator extends BaseMapTest
+public class TestValueInstantiator
 {
     static class MyBean
     {
@@ -61,9 +65,9 @@ public class TestValueInstantiator extends BaseMapTest
         @Override
         public boolean canCreateUsingDelegate() { return false; }
     }
-    
+
     static abstract class PolymorphicBeanBase { }
-    
+
     static class PolymorphicBean extends PolymorphicBeanBase
     {
         public String name;
@@ -82,14 +86,14 @@ public class TestValueInstantiator extends BaseMapTest
             put(name, name);
         }
     }
-    
+
     static class MyBeanInstantiator extends InstantiatorBase
     {
         @Override
         public String getValueTypeDesc() {
             return MyBean.class.getName();
         }
-        
+
         @Override
         public boolean canCreateUsingDefault() { return true; }
 
@@ -110,14 +114,14 @@ public class TestValueInstantiator extends BaseMapTest
         public String getValueTypeDesc() {
             return Object.class.getName();
         }
-        
+
         @Override
         public boolean canCreateFromObjectWith() { return true; }
-        
+
         @Override
         public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
             return  new CreatorProperty[] {
-                    new CreatorProperty(new PropertyName("type"), config.constructType(Class.class), null,
+                    CreatorProperty.construct(new PropertyName("type"), config.constructType(Class.class), null,
                             null, null, null, 0, null,
                             PropertyMetadata.STD_REQUIRED)
             };
@@ -133,21 +137,21 @@ public class TestValueInstantiator extends BaseMapTest
             }
         }
     }
-    
+
     static class CreatorMapInstantiator extends InstantiatorBase
     {
         @Override
         public String getValueTypeDesc() {
             return MyMap.class.getName();
         }
-        
+
         @Override
         public boolean canCreateFromObjectWith() { return true; }
 
         @Override
         public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
             return  new CreatorProperty[] {
-                    new CreatorProperty(new PropertyName("name"), config.constructType(String.class), null,
+                    CreatorProperty.construct(new PropertyName("name"), config.constructType(String.class), null,
                             null, null, null, 0, null,
                             PropertyMetadata.STD_REQUIRED)
             };
@@ -158,14 +162,14 @@ public class TestValueInstantiator extends BaseMapTest
             return new MyMap((String) args[0]);
         }
     }
-    
+
     static class MyDelegateBeanInstantiator extends ValueInstantiator.Base
     {
         public MyDelegateBeanInstantiator() { super(Object.class); }
 
         @Override
         public String getValueTypeDesc() { return "xxx"; }
-        
+
         @Override
         public boolean canCreateUsingDelegate() { return true; }
 
@@ -173,20 +177,20 @@ public class TestValueInstantiator extends BaseMapTest
         public JavaType getDelegateType(DeserializationConfig config) {
             return config.constructType(Object.class);
         }
-        
+
         @Override
         public Object createUsingDelegate(DeserializationContext ctxt, Object delegate) {
             return new MyBean(""+delegate, true);
         }
     }
-    
+
     static class MyListInstantiator extends InstantiatorBase
     {
         @Override
         public String getValueTypeDesc() {
             return MyList.class.getName();
         }
-        
+
         @Override
         public boolean canCreateUsingDefault() { return true; }
 
@@ -202,7 +206,7 @@ public class TestValueInstantiator extends BaseMapTest
 
         @Override
         public String getValueTypeDesc() { return "xxx"; }
-        
+
         @Override
         public boolean canCreateUsingDelegate() { return true; }
 
@@ -210,7 +214,7 @@ public class TestValueInstantiator extends BaseMapTest
         public JavaType getDelegateType(DeserializationConfig config) {
             return config.constructType(Object.class);
         }
-        
+
         @Override
         public Object createUsingDelegate(DeserializationContext ctxt, Object delegate) {
             MyList list = new MyList(true);
@@ -218,14 +222,14 @@ public class TestValueInstantiator extends BaseMapTest
             return list;
         }
     }
-    
+
     static class MyMapInstantiator extends InstantiatorBase
     {
         @Override
         public String getValueTypeDesc() {
             return MyMap.class.getName();
         }
-        
+
         @Override
         public boolean canCreateUsingDefault() { return true; }
 
@@ -241,7 +245,7 @@ public class TestValueInstantiator extends BaseMapTest
 
         @Override
         public String getValueTypeDesc() { return "xxx"; }
-        
+
         @Override
         public boolean canCreateUsingDelegate() { return true; }
 
@@ -249,7 +253,7 @@ public class TestValueInstantiator extends BaseMapTest
         public JavaType getDelegateType(DeserializationConfig config) {
             return TypeFactory.defaultInstance().constructType(Object.class);
         }
-        
+
         @Override
         public Object createUsingDelegate(DeserializationContext ctxt, Object delegate) {
             MyMap map = new MyMap(true);
@@ -262,7 +266,7 @@ public class TestValueInstantiator extends BaseMapTest
     static class AnnotatedBean {
         protected final String a;
         protected final int b;
-        
+
         public AnnotatedBean(String a, int b) {
             this.a = a;
             this.b = b;
@@ -275,7 +279,7 @@ public class TestValueInstantiator extends BaseMapTest
         public String getValueTypeDesc() {
             return AnnotatedBean.class.getName();
         }
-        
+
         @Override
         public boolean canCreateUsingDefault() { return true; }
 
@@ -309,7 +313,7 @@ public class TestValueInstantiator extends BaseMapTest
         public String getValueTypeDesc() {
             return AnnotatedBeanDelegating.class.getName();
         }
-        
+
         @Override
         public boolean canCreateUsingDelegate() { return true; }
 
@@ -322,7 +326,7 @@ public class TestValueInstantiator extends BaseMapTest
         public AnnotatedWithParams getDelegateCreator() {
             return null;
         }
-        
+
         @Override
         public Object createUsingDelegate(DeserializationContext ctxt, Object delegate) throws IOException {
             return new AnnotatedBeanDelegating(delegate, false);
@@ -335,8 +339,9 @@ public class TestValueInstantiator extends BaseMapTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = objectMapper();
-    
+    private final ObjectMapper MAPPER = sharedMapper();
+
+    @Test
     public void testCustomBeanInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -346,6 +351,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals("secret!", bean._secret);
     }
 
+    @Test
     public void testCustomListInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -356,6 +362,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(0, result.size());
     }
 
+    @Test
     public void testCustomMapInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -365,13 +372,14 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(MyMap.class, result.getClass());
         assertEquals(1, result.size());
     }
-    
+
     /*
     /**********************************************************
     /* Unit tests for delegate creators
     /**********************************************************
      */
 
+    @Test
     public void testDelegateBeanInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -381,6 +389,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals("123", bean._secret);
     }
 
+    @Test
     public void testDelegateListInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -390,7 +399,8 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(1, result.size());
         assertEquals(Integer.valueOf(123), result.get(0));
     }
-    
+
+    @Test
     public void testDelegateMapInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -401,6 +411,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(Integer.valueOf(123), result.values().iterator().next());
     }
 
+    @Test
     public void testCustomDelegateInstantiator() throws Exception
     {
         AnnotatedBeanDelegating value = MAPPER.readValue("{\"a\":3}", AnnotatedBeanDelegating.class);
@@ -416,6 +427,7 @@ public class TestValueInstantiator extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testPropertyBasedBeanInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -423,16 +435,16 @@ public class TestValueInstantiator extends BaseMapTest
                 new InstantiatorBase() {
                     @Override
                     public boolean canCreateFromObjectWith() { return true; }
-        
+
                     @Override
                     public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
                         return  new CreatorProperty[] {
-                                new CreatorProperty(new PropertyName("secret"), config.constructType(String.class), null,
+                                CreatorProperty.construct(new PropertyName("secret"), config.constructType(String.class), null,
                                         null, null, null, 0, null,
                                         PropertyMetadata.STD_REQUIRED)
                         };
                     }
-        
+
                     @Override
                     public Object createFromObjectWith(DeserializationContext ctxt, Object[] args) {
                         return new CreatorBean((String) args[0]);
@@ -443,6 +455,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals("123", bean._secret);
     }
 
+    @Test
     public void testPropertyBasedMapInstantiator() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -460,6 +473,7 @@ public class TestValueInstantiator extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testBeanFromString() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -467,17 +481,18 @@ public class TestValueInstantiator extends BaseMapTest
                 new InstantiatorBase() {
                     @Override
                     public boolean canCreateFromString() { return true; }
-                    
+
                     @Override
                     public Object createFromString(DeserializationContext ctxt, String value) {
                         return new MysteryBean(value);
                     }
         }));
-        MysteryBean result = mapper.readValue(quote("abc"), MysteryBean.class);
+        MysteryBean result = mapper.readValue(q("abc"), MysteryBean.class);
         assertNotNull(result);
         assertEquals("abc", result.value);
     }
 
+    @Test
     public void testBeanFromInt() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -485,7 +500,7 @@ public class TestValueInstantiator extends BaseMapTest
                 new InstantiatorBase() {
                     @Override
                     public boolean canCreateFromInt() { return true; }
-                    
+
                     @Override
                     public Object createFromInt(DeserializationContext ctxt, int value) {
                         return new MysteryBean(value+1);
@@ -496,6 +511,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(Integer.valueOf(38), result.value);
     }
 
+    @Test
     public void testBeanFromLong() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -503,7 +519,7 @@ public class TestValueInstantiator extends BaseMapTest
                 new InstantiatorBase() {
                     @Override
                     public boolean canCreateFromLong() { return true; }
-                    
+
                     @Override
                     public Object createFromLong(DeserializationContext ctxt, long value) {
                         return new MysteryBean(value+1L);
@@ -514,6 +530,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(Long.valueOf(9876543211L), result.value);
     }
 
+    @Test
     public void testBeanFromDouble() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -532,6 +549,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(Double.valueOf(0.5), result.value);
     }
 
+    @Test
     public void testBeanFromBoolean() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -539,7 +557,7 @@ public class TestValueInstantiator extends BaseMapTest
                 new InstantiatorBase() {
                     @Override
                     public boolean canCreateFromBoolean() { return true; }
-                    
+
                     @Override
                     public Object createFromBoolean(DeserializationContext ctxt, boolean value) {
                         return new MysteryBean(Boolean.valueOf(value));
@@ -549,7 +567,7 @@ public class TestValueInstantiator extends BaseMapTest
         assertNotNull(result);
         assertEquals(Boolean.TRUE, result.value);
     }
-    
+
     /*
     /**********************************************************
     /* Other tests
@@ -560,17 +578,19 @@ public class TestValueInstantiator extends BaseMapTest
      * Beyond basic features, it should be possible to even implement
      * polymorphic handling...
      */
+    @Test
     public void testPolymorphicCreatorBean() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
         mapper.registerModule(new MyModule(PolymorphicBeanBase.class, new PolymorphicBeanInstantiator()));
-        String JSON = "{\"type\":"+quote(PolymorphicBean.class.getName())+",\"name\":\"Axel\"}";
+        String JSON = "{\"type\":"+q(PolymorphicBean.class.getName())+",\"name\":\"Axel\"}";
         PolymorphicBeanBase result = mapper.readValue(JSON, PolymorphicBeanBase.class);
         assertNotNull(result);
         assertSame(PolymorphicBean.class, result.getClass());
         assertEquals("Axel", ((PolymorphicBean) result).name);
     }
 
+    @Test
     public void testEmptyBean() throws Exception
     {
         AnnotatedBean bean = MAPPER.readValue("{}", AnnotatedBean.class);
@@ -579,33 +599,29 @@ public class TestValueInstantiator extends BaseMapTest
         assertEquals(3, bean.b);
     }
 
-    // @since 2.8
+    @Test
     public void testErrorMessageForMissingCtor() throws Exception
     {
         // first fail, check message from JSON Object (no default ctor)
         try {
             MAPPER.readValue("{ }", MyBean.class);
             fail("Should not succeed");
-        } catch (JsonMappingException e) {
+        } catch (InvalidDefinitionException e) {
             verifyException(e, "Cannot construct instance of");
             verifyException(e, "no Creators");
-            // as per [databind#1414], is definition problem
-            assertEquals(InvalidDefinitionException.class, e.getClass());
         }
     }
 
-    // @since 2.8
+    @Test
     public void testErrorMessageForMissingStringCtor() throws Exception
     {
         // then from JSON String
         try {
             MAPPER.readValue("\"foo\"", MyBean.class);
             fail("Should not succeed");
-        } catch (JsonMappingException e) {
+        } catch (InvalidDefinitionException e) {
             verifyException(e, "Cannot construct instance of");
             verifyException(e, "no String-argument constructor/factory");
-            // as per [databind#1414], is definition problem
-            assertEquals(InvalidDefinitionException.class, e.getClass());
         }
     }
 }
