@@ -2,15 +2,21 @@ package com.fasterxml.jackson.databind.deser.filter;
 
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonSetter;
 import com.fasterxml.jackson.annotation.Nulls;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.exc.InvalidNullException;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 // For [databind#1402]; configurable null handling, for contents of
 // Collections, Maps, arrays
-public class NullConversionsForContentTest extends BaseMapTest
+public class NullConversionsForContentTest
 {
     static class NullContentFail<T> {
         public T nullsOk;
@@ -43,9 +49,10 @@ public class NullConversionsForContentTest extends BaseMapTest
     private final ObjectMapper MAPPER = newJsonMapper();
 
     // Tests to verify that we can set default settings for failure
+    @Test
     public void testFailOnNullFromDefaults() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
         TypeReference<NullContentUndefined<List<String>>> listType = new TypeReference<NullContentUndefined<List<String>>>() { };
 
         // by default fine to get nulls
@@ -77,22 +84,23 @@ public class NullConversionsForContentTest extends BaseMapTest
             assertEquals(String.class, e.getTargetType());
         }
     }
-    
+
+    @Test
     public void testFailOnNullWithCollections() throws Exception
     {
         TypeReference<NullContentFail<List<Integer>>> typeRef = new TypeReference<NullContentFail<List<Integer>>>() { };
 
         // first, ok if assigning non-null to not-nullable, null for nullable
-        NullContentFail<List<Integer>> result = MAPPER.readValue(aposToQuotes("{'nullsOk':[null]}"),
+        NullContentFail<List<Integer>> result = MAPPER.readValue(a2q("{'nullsOk':[null]}"),
                 typeRef);
         assertNotNull(result.nullsOk);
         assertEquals(1, result.nullsOk.size());
         assertNull(result.nullsOk.get(0));
 
         // and then see that nulls are not ok for non-nullable.
-        
+
         // List<Integer>
-        final String JSON = aposToQuotes("{'noNulls':[null]}");
+        final String JSON = a2q("{'noNulls':[null]}");
         try {
             MAPPER.readValue(JSON, typeRef);
             fail("Should not pass");
@@ -111,9 +119,10 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testFailOnNullWithArrays() throws Exception
     {
-        final String JSON = aposToQuotes("{'noNulls':[null]}");
+        final String JSON = a2q("{'noNulls':[null]}");
         // Object[]
         try {
             MAPPER.readValue(JSON, new TypeReference<NullContentFail<Object[]>>() { });
@@ -133,9 +142,10 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testFailOnNullWithPrimitiveArrays() throws Exception
     {
-        final String JSON = aposToQuotes("{'noNulls':[null]}");
+        final String JSON = a2q("{'noNulls':[null]}");
 
         // boolean[]
         try {
@@ -163,11 +173,12 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testFailOnNullWithMaps() throws Exception
     {
         // Then: Map<String,String>
         try {
-            final String MAP_JSON = aposToQuotes("{'noNulls':{'a':null}}");
+            final String MAP_JSON = a2q("{'noNulls':{'a':null}}");
             MAPPER.readValue(MAP_JSON, new TypeReference<NullContentFail<Map<String,String>>>() { });
             fail("Should not pass");
         } catch (InvalidNullException e) {
@@ -177,7 +188,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // Then: EnumMap<Enum,String>
         try {
-            final String MAP_JSON = aposToQuotes("{'noNulls':{'A':null}}");
+            final String MAP_JSON = a2q("{'noNulls':{'A':null}}");
             MAPPER.readValue(MAP_JSON, new TypeReference<NullContentFail<EnumMap<ABC,String>>>() { });
             fail("Should not pass");
         } catch (InvalidNullException e) {
@@ -191,10 +202,10 @@ public class NullConversionsForContentTest extends BaseMapTest
     /* Test methods, null-as-empty
     /**********************************************************
      */
-
+    @Test
     public void testNullsAsEmptyWithCollections() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
 
         // List<Integer>
         {
@@ -213,9 +224,10 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testNullsAsEmptyUsingDefaults() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
         TypeReference<NullContentUndefined<List<Integer>>> listType = new TypeReference<NullContentUndefined<List<Integer>>>() { };
 
         // Let's see defaulting in action
@@ -232,12 +244,13 @@ public class NullConversionsForContentTest extends BaseMapTest
         result = mapper.readValue(JSON, listType);
         assertEquals(1, result.values.size());
         assertEquals(Integer.valueOf(0), result.values.get(0));
-    }        
-    
+    }
+
+    @Test
     public void testNullsAsEmptyWithArrays() throws Exception
     {
         // Note: skip `Object[]`, no default empty value at this point
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
 
         // Then: String[]
         {
@@ -248,9 +261,10 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testNullsAsEmptyWithPrimitiveArrays() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
 
         // int[]
         {
@@ -276,13 +290,14 @@ public class NullConversionsForContentTest extends BaseMapTest
             assertEquals(false, result.values[0]);
         }
 }
-    
+
+    @Test
     public void testNullsAsEmptyWithMaps() throws Exception
     {
         // Then: Map<String,String>
-        final String MAP_JSON = aposToQuotes("{'values':{'A':null}}");
+        final String MAP_JSON = a2q("{'values':{'A':null}}");
         {
-            NullContentAsEmpty<Map<String,String>> result 
+            NullContentAsEmpty<Map<String,String>> result
                 = MAPPER.readValue(MAP_JSON, new TypeReference<NullContentAsEmpty<Map<String,String>>>() { });
             assertEquals(1, result.values.size());
             assertEquals("A", result.values.entrySet().iterator().next().getKey());
@@ -291,7 +306,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // Then: EnumMap<Enum,String>
         {
-            NullContentAsEmpty<EnumMap<ABC,String>> result 
+            NullContentAsEmpty<EnumMap<ABC,String>> result
                 = MAPPER.readValue(MAP_JSON, new TypeReference<NullContentAsEmpty<EnumMap<ABC,String>>>() { });
             assertEquals(1, result.values.size());
             assertEquals(ABC.A, result.values.entrySet().iterator().next().getKey());
@@ -305,9 +320,10 @@ public class NullConversionsForContentTest extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testNullsSkipUsingDefaults() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
         TypeReference<NullContentUndefined<List<Long>>> listType = new TypeReference<NullContentUndefined<List<Long>>>() { };
 
         // Let's see defaulting in action
@@ -322,12 +338,13 @@ public class NullConversionsForContentTest extends BaseMapTest
                 .setSetterInfo(JsonSetter.Value.forContentNulls(Nulls.SKIP));
         result = mapper.readValue(JSON, listType);
         assertEquals(0, result.values.size());
-    }        
+    }
 
     // Test to verify that per-property setting overrides defaults:
+    @Test
     public void testNullsSkipWithOverrides() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':[null]}");
+        final String JSON = a2q("{'values':[null]}");
         TypeReference<NullContentSkip<List<Long>>> listType = new TypeReference<NullContentSkip<List<Long>>>() { };
 
         ObjectMapper mapper = newJsonMapper();
@@ -342,13 +359,14 @@ public class NullConversionsForContentTest extends BaseMapTest
                 .setSetterInfo(JsonSetter.Value.forContentNulls(Nulls.FAIL));
         result = mapper.readValue(JSON, listType);
         assertEquals(0, result.values.size());
-    }        
+    }
 
+    @Test
     public void testNullsSkipWithCollections() throws Exception
     {
         // List<Integer>
         {
-            final String JSON = aposToQuotes("{'values':[1,null,2]}");
+            final String JSON = a2q("{'values':[1,null,2]}");
             NullContentSkip<List<Integer>> result = MAPPER.readValue(JSON,
                     new TypeReference<NullContentSkip<List<Integer>>>() { });
             assertEquals(2, result.values.size());
@@ -358,7 +376,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // List<String>
         {
-            final String JSON = aposToQuotes("{'values':['ab',null,'xy']}");
+            final String JSON = a2q("{'values':['ab',null,'xy']}");
             NullContentSkip<List<String>> result = MAPPER.readValue(JSON,
                     new TypeReference<NullContentSkip<List<String>>>() { });
             assertEquals(2, result.values.size());
@@ -367,9 +385,10 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testNullsSkipWithArrays() throws Exception
     {
-        final String JSON = aposToQuotes("{'values':['a',null,'xy']}");
+        final String JSON = a2q("{'values':['a',null,'xy']}");
         // Object[]
         {
             NullContentSkip<Object[]> result = MAPPER.readValue(JSON,
@@ -388,11 +407,12 @@ public class NullConversionsForContentTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testNullsSkipWithPrimitiveArrays() throws Exception
     {
         // int[]
         {
-            final String JSON = aposToQuotes("{'values':[3,null,7]}");
+            final String JSON = a2q("{'values':[3,null,7]}");
             NullContentSkip<int[]> result = MAPPER.readValue(JSON,
                     new TypeReference<NullContentSkip<int[]>>() { });
             assertEquals(2, result.values.length);
@@ -402,7 +422,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // long[]
         {
-            final String JSON = aposToQuotes("{'values':[-13,null,999]}");
+            final String JSON = a2q("{'values':[-13,null,999]}");
             NullContentSkip<long[]> result = MAPPER.readValue(JSON,
                     new TypeReference<NullContentSkip<long[]>>() { });
             assertEquals(2, result.values.length);
@@ -412,7 +432,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // boolean[]
         {
-            final String JSON = aposToQuotes("{'values':[true,null,true]}");
+            final String JSON = a2q("{'values':[true,null,true]}");
             NullContentSkip<boolean[]> result = MAPPER.readValue(JSON,
                     new TypeReference<NullContentSkip<boolean[]>>() { });
             assertEquals(2, result.values.length);
@@ -420,13 +440,14 @@ public class NullConversionsForContentTest extends BaseMapTest
             assertEquals(true, result.values[1]);
         }
     }
-    
+
+    @Test
     public void testNullsSkipWithMaps() throws Exception
     {
         // Then: Map<String,String>
-        final String MAP_JSON = aposToQuotes("{'values':{'A':'foo','B':null,'C':'bar'}}");
+        final String MAP_JSON = a2q("{'values':{'A':'foo','B':null,'C':'bar'}}");
         {
-            NullContentSkip<Map<String,String>> result 
+            NullContentSkip<Map<String,String>> result
                 = MAPPER.readValue(MAP_JSON, new TypeReference<NullContentSkip<Map<String,String>>>() { });
             assertEquals(2, result.values.size());
             assertEquals("foo", result.values.get("A"));
@@ -435,7 +456,7 @@ public class NullConversionsForContentTest extends BaseMapTest
 
         // Then: EnumMap<Enum,String>
         {
-            NullContentSkip<EnumMap<ABC,String>> result 
+            NullContentSkip<EnumMap<ABC,String>> result
                 = MAPPER.readValue(MAP_JSON, new TypeReference<NullContentSkip<EnumMap<ABC,String>>>() { });
             assertEquals(2, result.values.size());
             assertEquals("foo", result.values.get(ABC.A));
