@@ -3,15 +3,22 @@ package com.fasterxml.jackson.databind.deser.filter;
 import java.util.List;
 import java.util.Map;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonSetter;
 import com.fasterxml.jackson.annotation.Nulls;
 import com.fasterxml.jackson.core.type.TypeReference;
-
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil.Point;
+
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
 
 // for [databind#1402]; configurable null handling, for values themselves,
 // using generic types
-public class NullConversionsGenericTest extends BaseMapTest
+public class NullConversionsGenericTest
 {
     static class GeneralEmpty<T> {
         // 09-Feb-2017, tatu: Should only need annotation either for field OR setter, not both:
@@ -41,9 +48,10 @@ public class NullConversionsGenericTest extends BaseMapTest
 
     private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testNullsToEmptyPojo() throws Exception
     {
-        GeneralEmpty<Point> result = MAPPER.readValue(aposToQuotes("{'value':null}"),
+        GeneralEmpty<Point> result = MAPPER.readValue(a2q("{'value':null}"),
                 new TypeReference<GeneralEmpty<Point>>() { });
         assertNotNull(result.value);
         Point p = result.value;
@@ -52,51 +60,55 @@ public class NullConversionsGenericTest extends BaseMapTest
 
         // and then also failing case with no suitable creator:
         try {
-            /* NoCtorWrapper nogo =*/ MAPPER.readValue(aposToQuotes("{'value':null}"),
+            /* NoCtorWrapper nogo =*/ MAPPER.readValue(a2q("{'value':null}"),
                     NoCtorWrapper.class);
             fail("Should not pass");
-        } catch (JsonMappingException e) {
+        } catch (InvalidDefinitionException e) {
             verifyException(e, "Cannot create empty instance");
         }
     }
 
     // [databind#2023] two-part coercion from "" to `null` to skip/empty/exception should work
+    @Test
     public void testEmptyStringToNullToEmptyPojo() throws Exception
     {
         GeneralEmpty<Point> result = MAPPER.readerFor(new TypeReference<GeneralEmpty<Point>>() { })
                 .with(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT)
-                .readValue(aposToQuotes("{'value':''}"));
+                .readValue(a2q("{'value':''}"));
         assertNotNull(result.value);
         Point p = result.value;
         assertEquals(0, p.x);
         assertEquals(0, p.y);
     }
 
+    @Test
     public void testNullsToEmptyCollection() throws Exception
     {
-        GeneralEmpty<List<String>> result = MAPPER.readValue(aposToQuotes("{'value':null}"),
+        GeneralEmpty<List<String>> result = MAPPER.readValue(a2q("{'value':null}"),
                 new TypeReference<GeneralEmpty<List<String>>>() { });
         assertNotNull(result.value);
         assertEquals(0, result.value.size());
 
         // but also non-String type, since impls vary
-        GeneralEmpty<List<Integer>> result2 = MAPPER.readValue(aposToQuotes("{'value':null}"),
+        GeneralEmpty<List<Integer>> result2 = MAPPER.readValue(a2q("{'value':null}"),
                 new TypeReference<GeneralEmpty<List<Integer>>>() { });
         assertNotNull(result2.value);
         assertEquals(0, result2.value.size());
     }
 
+    @Test
     public void testNullsToEmptyMap() throws Exception
     {
-        GeneralEmpty<Map<String,String>> result = MAPPER.readValue(aposToQuotes("{'value':null}"),
+        GeneralEmpty<Map<String,String>> result = MAPPER.readValue(a2q("{'value':null}"),
                 new TypeReference<GeneralEmpty<Map<String,String>>>() { });
         assertNotNull(result.value);
         assertEquals(0, result.value.size());
     }
 
+    @Test
     public void testNullsToEmptyArrays() throws Exception
     {
-        final String json = aposToQuotes("{'value':null}");
+        final String json = a2q("{'value':null}");
 
         GeneralEmpty<Object[]> result = MAPPER.readValue(json,
                 new TypeReference<GeneralEmpty<Object[]>>() { });
