@@ -4,6 +4,8 @@ import java.io.IOException;
 import java.util.Map;
 import java.util.UUID;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.core.JsonParser;
 import com.fasterxml.jackson.core.JsonToken;
@@ -11,15 +13,20 @@ import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.deser.DeserializationProblemHandler;
 import com.fasterxml.jackson.databind.deser.ValueInstantiator;
 import com.fasterxml.jackson.databind.exc.InvalidTypeIdException;
+import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 import com.fasterxml.jackson.databind.exc.ValueInstantiationException;
 import com.fasterxml.jackson.databind.jsontype.TypeIdResolver;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 /**
  * Tests to exercise handler methods of {@link DeserializationProblemHandler}.
  *
  * @since 2.8
  */
-public class ProblemHandlerTest extends BaseMapTest
+public class ProblemHandlerTest
 {
     /*
     /**********************************************************
@@ -69,11 +76,11 @@ public class ProblemHandlerTest extends BaseMapTest
         extends DeserializationProblemHandler
     {
         protected final Object value;
-    
+
         public WeirdStringHandler(Object v0) {
             value = v0;
         }
-    
+
         @Override
         public Object handleWeirdStringValue(DeserializationContext ctxt,
                 Class<?> targetType, String v,
@@ -88,7 +95,7 @@ public class ProblemHandlerTest extends BaseMapTest
         extends DeserializationProblemHandler
     {
         protected final Object value;
-    
+
         public InstantiationProblemHandler(Object v0) {
             value = v0;
         }
@@ -149,7 +156,7 @@ public class ProblemHandlerTest extends BaseMapTest
         protected final Class<?> raw;
 
         public UnknownTypeIdHandler(Class<?> r) { raw = r; }
-        
+
         @Override
         public JavaType handleUnknownTypeId(DeserializationContext ctxt,
                 JavaType baseType, String subTypeId, TypeIdResolver idResolver,
@@ -164,9 +171,9 @@ public class ProblemHandlerTest extends BaseMapTest
         extends DeserializationProblemHandler
     {
         protected final Class<?> raw;
-    
+
         public MissingTypeIdHandler(Class<?> r) { raw = r; }
-        
+
         @Override
         public JavaType handleMissingTypeId(DeserializationContext ctxt,
                 JavaType baseType, TypeIdResolver idResolver,
@@ -206,7 +213,7 @@ public class ProblemHandlerTest extends BaseMapTest
     static class Base2Wrapper {
         public Base2 value;
     }
-    
+
     enum SingleValuedEnum {
         A;
     }
@@ -234,10 +241,12 @@ public class ProblemHandlerTest extends BaseMapTest
 
     private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testWeirdKeyHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new WeirdKeyHandler(7));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new WeirdKeyHandler(7))
+            .build();
         IntKeyMapWrapper w = mapper.readValue("{\"stuff\":{\"foo\":\"abc\"}}",
                 IntKeyMapWrapper.class);
         Map<Integer,String> map = w.stuff;
@@ -246,44 +255,67 @@ public class ProblemHandlerTest extends BaseMapTest
         assertEquals(Integer.valueOf(7), map.keySet().iterator().next());
     }
 
+    @Test
     public void testWeirdNumberHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
+        ObjectMapper mapper = jsonMapperBuilder()
             .addHandler(new WeirdNumberHandler(SingleValuedEnum.A))
-            ;
+            .build();
         SingleValuedEnum result = mapper.readValue("3", SingleValuedEnum.class);
         assertEquals(SingleValuedEnum.A, result);
     }
 
+    @Test
     public void testWeirdStringHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
+        ObjectMapper mapper = jsonMapperBuilder()
             .addHandler(new WeirdStringHandler(SingleValuedEnum.A))
-            ;
+            .build();
         SingleValuedEnum result = mapper.readValue("\"B\"", SingleValuedEnum.class);
         assertEquals(SingleValuedEnum.A, result);
 
         // also, write [databind#1629] try this
         mapper = new ObjectMapper()
                 .addHandler(new WeirdStringHandler(null));
-        UUID result2 = mapper.readValue(quote("not a uuid!"), UUID.class);
+        UUID result2 = mapper.readValue(q("not a uuid!"), UUID.class);
         assertNull(result2);
     }
 
+    // [databind#3784]: Base64 decoding
+    @Test
+    public void testWeirdStringForBase64() throws Exception
+    {
+        ObjectMapper mapper = jsonMapperBuilder()
+                .addHandler(new WeirdStringHandler(new byte[0]))
+                .build();
+        byte[] binary = mapper.readValue(q("foobar"), byte[].class);
+        assertNotNull(binary);
+        assertEquals(0, binary.length);
+
+        JsonNode tree = mapper.readTree(q("foobar"));
+        binary = mapper.treeToValue(tree, byte[].class);
+        assertNotNull(binary);
+        assertEquals(0, binary.length);
+    }
+
+    @Test
     public void testInvalidTypeId() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new UnknownTypeIdHandler(BaseImpl.class));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new UnknownTypeIdHandler(BaseImpl.class))
+            .build();
         BaseWrapper w = mapper.readValue("{\"value\":{\"type\":\"foo\",\"a\":4}}",
                 BaseWrapper.class);
         assertNotNull(w);
         assertEquals(BaseImpl.class, w.value.getClass());
     }
 
+    @Test
     public void testInvalidClassAsId() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new UnknownTypeIdHandler(Base2Impl.class));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new UnknownTypeIdHandler(Base2Impl.class))
+            .build();
         Base2Wrapper w = mapper.readValue("{\"value\":{\"clazz\":\"com.fizz\",\"a\":4}}",
                 Base2Wrapper.class);
         assertNotNull(w);
@@ -292,27 +324,32 @@ public class ProblemHandlerTest extends BaseMapTest
 
     // 2.9: missing type id, distinct from unknown
 
+    @Test
     public void testMissingTypeId() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new MissingTypeIdHandler(BaseImpl.class));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new MissingTypeIdHandler(BaseImpl.class))
+            .build();
         BaseWrapper w = mapper.readValue("{\"value\":{\"a\":4}}",
                 BaseWrapper.class);
         assertNotNull(w);
         assertEquals(BaseImpl.class, w.value.getClass());
     }
 
+    @Test
     public void testMissingClassAsId() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new MissingTypeIdHandler(Base2Impl.class));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new MissingTypeIdHandler(Base2Impl.class))
+            .build();
         Base2Wrapper w = mapper.readValue("{\"value\":{\"a\":4}}",
                 Base2Wrapper.class);
         assertNotNull(w);
         assertEquals(Base2Impl.class, w.value.getClass());
     }
-    
+
     // verify that by default we get special exception type
+    @Test
     public void testInvalidTypeIdFail() throws Exception
     {
         try {
@@ -326,31 +363,46 @@ public class ProblemHandlerTest extends BaseMapTest
         }
     }
 
+    @Test
     public void testInstantiationExceptionHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
-            .addHandler(new InstantiationProblemHandler(BustedCtor.INST));
+        ObjectMapper mapper = jsonMapperBuilder()
+            .addHandler(new InstantiationProblemHandler(BustedCtor.INST))
+            .build();
         BustedCtor w = mapper.readValue("{ }",
                 BustedCtor.class);
         assertNotNull(w);
     }
 
+    @Test
     public void testMissingInstantiatorHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
+        ObjectMapper mapper = jsonMapperBuilder()
             .addHandler(new MissingInstantiationHandler(new NoDefaultCtor(13)))
-            ;
+            .build();
         NoDefaultCtor w = mapper.readValue("{ \"x\" : true }", NoDefaultCtor.class);
         assertNotNull(w);
         assertEquals(13, w.value);
     }
 
+    @Test
     public void testUnexpectedTokenHandling() throws Exception
     {
-        ObjectMapper mapper = newJsonMapper()
+        ObjectMapper mapper = jsonMapperBuilder()
             .addHandler(new WeirdTokenHandler(Integer.valueOf(13)))
-        ;
+            .build();
         Integer v = mapper.readValue("true", Integer.class);
         assertEquals(Integer.valueOf(13), v);
+
+        // Just for code coverage really...
+        mapper = newJsonMapper();
+        mapper.addHandler(new WeirdTokenHandler(Integer.valueOf(13)));
+        mapper.clearProblemHandlers();
+        try {
+            mapper.readValue("true", Integer.class);
+            fail("Should not pass");
+        } catch (MismatchedInputException e) {
+            verifyException(e, "from Boolean value (token `JsonToken.VALUE_TRUE`)");
+        }
     }
 }
