@@ -5,15 +5,21 @@ import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.JsonIgnore;
 import com.fasterxml.jackson.annotation.OptBoolean;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 import com.fasterxml.jackson.databind.exc.InvalidFormatException;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 public class DateDeserializationTest
-    extends BaseMapTest
 {
     static class DateAsStringBean
     {
@@ -26,7 +32,7 @@ public class DateDeserializationTest
         @JsonFormat(shape=JsonFormat.Shape.STRING, pattern="/yyyy/MM/dd/", locale="fr_FR")
         public Date date;
     }
-    
+
     static class CalendarAsStringBean
     {
         @JsonFormat(shape=JsonFormat.Shape.STRING, pattern=";yyyy/MM/dd;")
@@ -47,7 +53,7 @@ public class DateDeserializationTest
         @JsonFormat(lenient=OptBoolean.TRUE)
         public Calendar value;
     }
-    
+
     static class StrictCalendarBean {
         @JsonFormat(lenient=OptBoolean.FALSE)
         public Calendar value;
@@ -92,6 +98,7 @@ public class DateDeserializationTest
 
     private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testDateUtil() throws Exception
     {
         long now = 123456789L;
@@ -104,27 +111,29 @@ public class DateDeserializationTest
         String dateStr = dateToString(value);
         java.util.Date result = MAPPER.readValue("\""+dateStr+"\"", java.util.Date.class);
 
-        assertEquals("Date: expect "+value+" ("+value.getTime()+"), got "+result+" ("+result.getTime()+")",
-                value.getTime(), result.getTime());
+        assertEquals(value.getTime(), result.getTime(),
+            "Date: expect "+value+" ("+value.getTime()+"), got "+result+" ("+result.getTime()+")");
     }
 
+    @Test
     public void testDateUtilWithStringTimestamp() throws Exception
     {
         long now = 1321992375446L;
         /* Should be ok to pass as JSON String, as long
          * as it is plain timestamp (all numbers, 64-bit)
          */
-        String json = quote(String.valueOf(now));
+        String json = q(String.valueOf(now));
         java.util.Date value = MAPPER.readValue(json, java.util.Date.class);
         assertEquals(now, value.getTime());
 
         // #267: should handle negative timestamps too; like 12 hours before 1.1.1970
         long before = - (24 * 3600 * 1000L);
-        json = quote(String.valueOf(before));
+        json = q(String.valueOf(before));
         value = MAPPER.readValue(json, java.util.Date.class);
         assertEquals(before, value.getTime());
     }
 
+    @Test
     public void testDateUtilRFC1123() throws Exception
     {
         DateFormat fmt = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
@@ -134,6 +143,7 @@ public class DateDeserializationTest
         assertEquals(inputDate, MAPPER.readValue("\""+inputStr+"\"", java.util.Date.class));
     }
 
+    @Test
     public void testDateUtilRFC1123OnNonUSLocales() throws Exception
     {
         Locale old = Locale.getDefault();
@@ -149,6 +159,7 @@ public class DateDeserializationTest
     /**
      * ISO8601 is supported as well
      */
+    @Test
     public void testDateUtilISO8601() throws Exception
     {
         /* let's use simple baseline value, arbitrary date in GMT,
@@ -164,7 +175,7 @@ public class DateDeserializationTest
 
         // And then the same, but using 'Z' as alias for +0000 (very common)
         inputStr = "1972-12-28T00:00:00.000Z";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1972, c.get(Calendar.YEAR));
         assertEquals(Calendar.DECEMBER, c.get(Calendar.MONTH));
@@ -172,7 +183,7 @@ public class DateDeserializationTest
 
         // Same but using colon in timezone
         inputStr = "1972-12-28T00:00:00.000+00:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1972, c.get(Calendar.YEAR));
         assertEquals(Calendar.DECEMBER, c.get(Calendar.MONTH));
@@ -180,14 +191,14 @@ public class DateDeserializationTest
 
         // Same but only passing hour difference as timezone
         inputStr = "1972-12-28T00:00:00.000+00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1972, c.get(Calendar.YEAR));
         assertEquals(Calendar.DECEMBER, c.get(Calendar.MONTH));
         assertEquals(28, c.get(Calendar.DAY_OF_MONTH));
 
         inputStr = "1984-11-30T00:00:00.000Z";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1984, c.get(Calendar.YEAR));
         assertEquals(Calendar.NOVEMBER, c.get(Calendar.MONTH));
@@ -195,14 +206,15 @@ public class DateDeserializationTest
     }
 
     // [Databind#570]
+    @Test
     public void testISO8601PartialMilliseconds() throws Exception
     {
         String inputStr;
         Date inputDate;
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
-        
+
         inputStr = "2014-10-03T18:00:00.6-05:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(2014, c.get(Calendar.YEAR));
         assertEquals(Calendar.OCTOBER, c.get(Calendar.MONTH));
@@ -210,7 +222,7 @@ public class DateDeserializationTest
         assertEquals(600, c.get(Calendar.MILLISECOND));
 
         inputStr = "2014-10-03T18:00:00.61-05:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(2014, c.get(Calendar.YEAR));
         assertEquals(Calendar.OCTOBER, c.get(Calendar.MONTH));
@@ -221,7 +233,7 @@ public class DateDeserializationTest
         assertEquals(610, c.get(Calendar.MILLISECOND));
 
         inputStr = "1997-07-16T19:20:30.45+01:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -233,7 +245,7 @@ public class DateDeserializationTest
 
         // 14-Sep-2015, tatu: Colon for timezone offset is optional, verify
         inputStr = "1997-07-16T19:20:30.45+0100";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -245,7 +257,7 @@ public class DateDeserializationTest
 
         // plus may also just have hour part
         inputStr = "1997-07-16T19:20:30.45+01";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -257,10 +269,11 @@ public class DateDeserializationTest
     }
 
     // Also: minutes-part of offset need not be all zeroes: [databind#1788]
+    @Test
     public void testISO8601FractionalTimezoneOffset() throws Exception
     {
         String inputStr = "1997-07-16T19:20:30.45+01:30";
-        java.util.Date inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        java.util.Date inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
@@ -273,6 +286,7 @@ public class DateDeserializationTest
     }
 
     // [databind#1745]
+    @Test
     public void testISO8601FractSecondsLong() throws Exception
     {
         String inputStr;
@@ -280,7 +294,7 @@ public class DateDeserializationTest
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
 
         inputStr = "2014-10-03T18:00:00.3456-05:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(2014, c.get(Calendar.YEAR));
         assertEquals(Calendar.OCTOBER, c.get(Calendar.MONTH));
@@ -290,13 +304,14 @@ public class DateDeserializationTest
 
         // But! Still limit to 9 digits (nanoseconds)
         try {
-            MAPPER.readValue(quote("2014-10-03T18:00:00.1234567890-05:00"), java.util.Date.class);
+            MAPPER.readValue(q("2014-10-03T18:00:00.1234567890-05:00"), java.util.Date.class);
         } catch (InvalidFormatException e) {
             verifyException(e, "invalid fractional seconds");
             verifyException(e, "can use at most 9 digits");
         }
     }
 
+    @Test
     public void testISO8601MissingSeconds() throws Exception
     {
         String inputStr;
@@ -306,7 +321,7 @@ public class DateDeserializationTest
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
 
         inputStr = "1997-07-16T19:20+01:00";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -317,7 +332,7 @@ public class DateDeserializationTest
 
         // 14-Sep-2015, tatu: Colon for timezone offset is optional, verify
         inputStr = "1997-07-16T19:20+0200";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -328,7 +343,7 @@ public class DateDeserializationTest
 
         // or just hour
         inputStr = "1997-07-16T19:20+04";
-        inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         c.setTime(inputDate);
         assertEquals(1997, c.get(Calendar.YEAR));
         assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
@@ -338,11 +353,12 @@ public class DateDeserializationTest
         assertEquals(0, c.get(Calendar.MILLISECOND));
     }
 
+    @Test
     public void testDateUtilISO8601NoTimezone() throws Exception
     {
-        // Timezone itself is optional as well... 
+        // Timezone itself is optional as well...
         String inputStr = "1984-11-13T00:00:09";
-        Date inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        Date inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         c.setTime(inputDate);
         assertEquals(1984, c.get(Calendar.YEAR));
@@ -355,16 +371,17 @@ public class DateDeserializationTest
     }
 
     // [databind#1657]: no timezone should use configured default
+    @Test
     public void testDateUtilISO8601NoTimezoneNonDefault() throws Exception
     {
         // In first case, no timezone -> SHOULD use configured timezone
         ObjectReader r = MAPPER.readerFor(Date.class);
         TimeZone tz = TimeZone.getTimeZone("GMT-2");
         Date date1 = r.with(tz)
-                .readValue(quote("1970-01-01T00:00:00.000"));
+                .readValue(q("1970-01-01T00:00:00.000"));
         // Second case, should use specified timezone, not configured
         Date date2 = r.with(TimeZone.getTimeZone("GMT+5"))
-                .readValue(quote("1970-01-01T00:00:00.000-02:00"));
+                .readValue(q("1970-01-01T00:00:00.000-02:00"));
         assertEquals(date1, date2);
 
         // also verify actual value, in GMT
@@ -378,6 +395,7 @@ public class DateDeserializationTest
 
     // [databind#1722]: combination of `@ConstructorProperties` and `@JsonIgnore`
     //  should work fine.
+    @Test
     public void testFormatAndCtors1722() throws Exception
     {
         Date1722 input = new Date1722(new Date(0L), "bogus");
@@ -387,13 +405,14 @@ public class DateDeserializationTest
     }
 
     // [databind#338]
+    @Test
     public void testDateUtilISO8601NoMilliseconds() throws Exception
     {
         final String INPUT_STR = "2013-10-31T17:27:00";
         Date inputDate;
         Calendar c;
-        
-        inputDate = MAPPER.readValue(quote(INPUT_STR), java.util.Date.class);
+
+        inputDate = MAPPER.readValue(q(INPUT_STR), java.util.Date.class);
         c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         c.setTime(inputDate);
         assertEquals(2013, c.get(Calendar.YEAR));
@@ -420,11 +439,12 @@ public class DateDeserializationTest
         */
     }
 
+    @Test
     public void testDateUtilISO8601JustDate() throws Exception
     {
         // Plain date (no time)
         String inputStr = "1972-12-28";
-        Date inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+        Date inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
         Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         c.setTime(inputDate);
         assertEquals(1972, c.get(Calendar.YEAR));
@@ -432,44 +452,7 @@ public class DateDeserializationTest
         assertEquals(28, c.get(Calendar.DAY_OF_MONTH));
     }
 
-    @SuppressWarnings("deprecation")
-    public void testDateSql() throws Exception
-    {
-        java.sql.Date value = new java.sql.Date(0L);
-        value.setYear(99); // 1999
-        value.setDate(19);
-        value.setMonth(Calendar.APRIL);
-        long now = value.getTime();
-
-        // First from long
-        assertEquals(value, MAPPER.readValue(String.valueOf(now), java.sql.Date.class));
-
-        // then from default java.sql.Date String serialization:
-        
-        java.sql.Date result = MAPPER.readValue(quote(value.toString()), java.sql.Date.class);
-        Calendar c = gmtCalendar(result.getTime());
-        assertEquals(1999, c.get(Calendar.YEAR));
-        assertEquals(Calendar.APRIL, c.get(Calendar.MONTH));
-        assertEquals(19, c.get(Calendar.DAY_OF_MONTH));
-
-        /* [JACKSON-200]: looks like we better add support for regular date
-         *   formats as well
-         */
-        String expStr = "1981-07-13";
-        result = MAPPER.readValue(quote(expStr), java.sql.Date.class);
-        c.setTimeInMillis(result.getTime());
-        assertEquals(1981, c.get(Calendar.YEAR));
-        assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
-        assertEquals(13, c.get(Calendar.DAY_OF_MONTH));
-
-        /* 20-Nov-2009, tatus: I'll be damned if I understand why string serialization
-         *   is off-by-one, but day-of-month does seem to be one less. My guess is
-         *   that something is funky with timezones (i.e. somewhere local TZ is
-         *   being used), but just can't resolve it. Hence, need to comment this:
-         */
-//        assertEquals(expStr, result.toString());
-    }
-
+    @Test
     public void testCalendar() throws Exception
     {
         // not ideal, to use (ever-changing) current date, but...
@@ -483,7 +466,7 @@ public class DateDeserializationTest
 
         // Then from serialized String
         String dateStr = dateToString(new Date(l));
-        result = MAPPER.readValue(quote(dateStr), Calendar.class);
+        result = MAPPER.readValue(q(dateStr), Calendar.class);
 
         // note: representation may differ (wrt timezone etc), but underlying value must remain the same:
         if (l != result.getTimeInMillis()) {
@@ -492,6 +475,7 @@ public class DateDeserializationTest
         }
     }
 
+    @Test
     public void testCustom() throws Exception
     {
         final ObjectMapper mapper = new ObjectMapper();
@@ -509,13 +493,14 @@ public class DateDeserializationTest
      * Test for [JACKSON-203]: make empty Strings deserialize as nulls by default,
      * without need to turn on feature (which may be added in future)
      */
+    @Test
     public void testDatesWithEmptyStrings() throws Exception
     {
-        assertNull(MAPPER.readValue(quote(""), java.util.Date.class));
-        assertNull(MAPPER.readValue(quote(""), java.util.Calendar.class));
-        assertNull(MAPPER.readValue(quote(""), java.sql.Date.class));
+        assertNull(MAPPER.readValue(q(""), java.util.Date.class));
+        assertNull(MAPPER.readValue(q(""), java.util.Calendar.class));
     }
 
+    @Test
     public void test8601DateTimeNoMilliSecs() throws Exception
     {
         // ok, Zebra, no milliseconds
@@ -525,7 +510,7 @@ public class DateDeserializationTest
                "2010-06-28T23:34:22+00:00",
                "2010-06-28T23:34:22+00",
         }) {
-            Date inputDate = MAPPER.readValue(quote(inputStr), java.util.Date.class);
+            Date inputDate = MAPPER.readValue(q(inputStr), java.util.Date.class);
             Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
             c.setTime(inputDate);
             assertEquals(2010, c.get(Calendar.YEAR));
@@ -538,12 +523,14 @@ public class DateDeserializationTest
         }
     }
 
+    @Test
     public void testTimeZone() throws Exception
     {
-        TimeZone result = MAPPER.readValue(quote("PST"), TimeZone.class);
+        TimeZone result = MAPPER.readValue(q("PST"), TimeZone.class);
         assertEquals("PST", result.getID());
     }
 
+    @Test
     public void testCustomDateWithAnnotation() throws Exception
     {
         final String INPUT = "{\"date\":\"/2005/05/25/\"}";
@@ -590,6 +577,7 @@ public class DateDeserializationTest
         assertEquals(25, c.get(Calendar.DAY_OF_MONTH));
     }
 
+    @Test
     public void testCustomCalendarWithAnnotation() throws Exception
     {
         CalendarAsStringBean cbean = MAPPER.readValue("{\"cal\":\";2007/07/13;\"}",
@@ -602,6 +590,7 @@ public class DateDeserializationTest
         assertEquals(13, c.get(Calendar.DAY_OF_MONTH));
     }
 
+    @Test
     public void testCustomCalendarWithTimeZone() throws Exception
     {
         // And then with different TimeZone: CET is +01:00 from GMT -- read as CET
@@ -617,21 +606,22 @@ public class DateDeserializationTest
     }
 
     // [databind#1651]
+    @Test
     public void testDateEndingWithZNonDefTZ1651() throws Exception
     {
-        String json = quote("1970-01-01T00:00:00.000Z");
+        String json = q("1970-01-01T00:00:00.000Z");
 
         // Standard mapper with timezone UTC: shared instance should be ok.
         // ... but, Travis manages to have fails, so insist on newly created
         ObjectMapper mapper = newJsonMapper();
         Date dateUTC = mapper.readValue(json, Date.class);  // 1970-01-01T00:00:00.000+00:00
-    
+
         // Mapper with timezone GMT-2
         // note: must construct new one, not share
         mapper = new ObjectMapper();
         mapper.setTimeZone(TimeZone.getTimeZone("GMT-2"));
         Date dateGMT1 = mapper.readValue(json, Date.class);  // 1970-01-01T00:00:00.000-02:00
-    
+
         // Underlying timestamps should be the same
         assertEquals(dateUTC.getTime(), dateGMT1.getTime());
     }
@@ -641,8 +631,9 @@ public class DateDeserializationTest
     /* Context timezone use (or not)
     /**********************************************************
      */
-    
+
     // for [databind#204]
+    @Test
     public void testContextTimezone() throws Exception
     {
         String inputStr = "1997-07-16T19:20:30.45+0100";
@@ -655,7 +646,7 @@ public class DateDeserializationTest
                 .with(TimeZone.getTimeZone(tzId));
 
         // by default use contextual timezone:
-        Calendar cal = r.readValue(quote(inputStr));
+        Calendar cal = r.readValue(q(inputStr));
         TimeZone tz = cal.getTimeZone();
         assertEquals(tzId, tz.getID());
 
@@ -670,12 +661,12 @@ public class DateDeserializationTest
 
         // but if disabled, should use what's been sent in:
         cal = r.without(DeserializationFeature.ADJUST_DATES_TO_CONTEXT_TIME_ZONE)
-                .readValue(quote(inputStr));
+                .readValue(q(inputStr));
 
         // 23-Jun-2017, tatu: Actually turns out to be hard if not impossible to do ...
         //    problem being SimpleDateFormat does not really retain timezone offset.
         //    But if we match fields... we perhaps could use it?
-        
+
         // !!! TODO: would not yet pass
 /*
         System.err.println("CAL/2 == "+cal);
@@ -689,20 +680,21 @@ public class DateDeserializationTest
     /* Test(s) for array unwrapping
     /**********************************************************
      */
-    
+
+    @Test
     public void testCalendarArrayUnwrap() throws Exception
     {
         ObjectReader reader = new ObjectMapper()
                 .readerFor(CalendarBean.class)
                 .without(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
         final String inputDate = "1972-12-28T00:00:00.000+0000";
-        final String input = aposToQuotes("{'v':['"+inputDate+"']}");
+        final String input = a2q("{'v':['"+inputDate+"']}");
         try {
             reader.readValue(input);
             fail("Did not throw exception when reading a value from a single value array with the UNWRAP_SINGLE_VALUE_ARRAYS feature disabled");
         } catch (MismatchedInputException exp) {
             verifyException(exp, "Cannot deserialize");
-            verifyException(exp, "out of START_ARRAY");
+            verifyException(exp, "from Array value (token `JsonToken.START_ARRAY`)");
         }
 
         reader = reader.with(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
@@ -712,9 +704,9 @@ public class DateDeserializationTest
 
         // and finally, a fail due to multiple values:
         try {
-            reader.readValue(aposToQuotes("{'v':['"+inputDate+"','"+inputDate+"']}"));
+            reader.readValue(a2q("{'v':['"+inputDate+"','"+inputDate+"']}"));
             fail("Did not throw exception while reading a value from a multi value array with UNWRAP_SINGLE_VALUE_ARRAY feature enabled");
-        } catch (JsonMappingException exp) {
+        } catch (MismatchedInputException exp) {
             verifyException(exp, "Attempted to unwrap");
         }
     }
@@ -725,9 +717,10 @@ public class DateDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testLenientJDKDateTypes() throws Exception
     {
-        final String JSON = aposToQuotes("{'value':'2015-11-32'}");
+        final String JSON = a2q("{'value':'2015-11-32'}");
 
         // with lenient, can parse fine
         LenientCalendarBean lenBean = MAPPER.readValue(JSON, LenientCalendarBean.class);
@@ -745,13 +738,14 @@ public class DateDeserializationTest
         }
     }
 
+    @Test
     public void testLenientJDKDateTypesViaTypeOverride() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
         mapper.configOverride(java.util.Date.class)
             .setFormat(JsonFormat.Value.forLeniency(Boolean.FALSE));
         try {
-            mapper.readValue(quote("2015-11-32"), java.util.Date.class);
+            mapper.readValue(q("2015-11-32"), java.util.Date.class);
             fail("Should not pass with invalid (with strict) date value");
         } catch (MismatchedInputException e) {
             verifyException(e, "Cannot deserialize value of type `java.util.Date`");
@@ -760,9 +754,10 @@ public class DateDeserializationTest
         }
     }
 
+    @Test
     public void testLenientJDKDateTypesViaGlobal() throws Exception
     {
-        final String JSON = quote("2015-11-32");
+        final String JSON = q("2015-11-32");
 
         // with lenient, can parse fine
         Calendar value = MAPPER.readValue(JSON, Calendar.class);
@@ -780,7 +775,7 @@ public class DateDeserializationTest
             verifyException(e, "from String \"2015-11-32\"");
             verifyException(e, "expected format");
         }
-    
+
         // Unless we actually had per-type override too
         mapper = new ObjectMapper();
         mapper.configOverride(Calendar.class)
@@ -798,10 +793,11 @@ public class DateDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testInvalidFormat() throws Exception
     {
         try {
-            MAPPER.readValue(quote("foobar"), Date.class);
+            MAPPER.readValue(q("foobar"), Date.class);
             fail("Should have failed with an exception");
         } catch (InvalidFormatException e) {
             verifyException(e, "Cannot deserialize value of type `java.util.Date` from String");
@@ -812,7 +808,7 @@ public class DateDeserializationTest
                     +InvalidFormatException.class.getName());
         }
     }
-    
+
     /*
     /**********************************************************
     /* Helper methods
@@ -821,18 +817,9 @@ public class DateDeserializationTest
 
     private String dateToString(java.util.Date value)
     {
-        /* Then from String. This is bit tricky, since JDK does not really
-         * suggest a 'standard' format. So let's try using something...
-         */
+        // Then from String. This is bit tricky, since JDK does not really
+        // suggest a 'standard' format. So let's try using something...
         DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
         return df.format(value);
     }
-
-    private static Calendar gmtCalendar(long time)
-    {
-        Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
-        c.setTimeInMillis(time);
-        return c;
-    }
-
 }
