@@ -5,6 +5,8 @@ import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.core.*;
@@ -15,9 +17,12 @@ import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
 import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
 import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 @SuppressWarnings("serial")
 public class MapDeserializationTest
-    extends BaseMapTest
 {
     static enum Key {
         KEY1, KEY2, WHATEVER;
@@ -48,7 +53,7 @@ public class MapDeserializationTest
 
     static class KeyType {
         protected String value;
-        
+
         private KeyType(String v, boolean bogus) {
             value = v;
         }
@@ -79,7 +84,7 @@ public class MapDeserializationTest
     }
 
     static class ClassStringMap extends HashMap<Class<?>,String> { }
-    
+
     static class AbstractMapWrapper {
         public AbstractMap<String, Integer> values;
     }
@@ -90,8 +95,9 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testBigUntypedMap() throws Exception
     {
         Map<String,Object> map = new LinkedHashMap<String,Object>();
@@ -106,13 +112,17 @@ public class MapDeserializationTest
         }
         String json = MAPPER.writeValueAsString(map);
         Object bound = MAPPER.readValue(json, Object.class);
+
+        assertTrue(bound instanceof Map<?,?>);
+        assertEquals(map.size(), ((Map<?,?>) bound).size());
         assertEquals(map, bound);
     }
-    
+
     /**
      * Let's also try another way to express "gimme a Map" deserialization;
      * this time by specifying a Map class, to reduce need to cast
      */
+    @Test
     public void testUntypedMap2() throws Exception
     {
         // to get "untyped" default map-to-map, pass Object.class
@@ -131,6 +141,7 @@ public class MapDeserializationTest
     /**
      * Unit test for [JACKSON-185]
      */
+    @Test
     public void testUntypedMap3() throws Exception
     {
         String JSON = "{\"a\":[{\"a\":\"b\"},\"value\"]}";
@@ -157,9 +168,22 @@ public class MapDeserializationTest
             "{ \"double\":42.0, \"string\":\"string\","
             +"\"boolean\":true, \"list\":[\"list0\"],"
             +"\"null\":null }";
-    
+
+    protected static class ObjectWrapper {
+        final Object object;
+        protected ObjectWrapper(final Object object) {
+            this.object = object;
+        }
+        public Object getObject() { return object; }
+        @JsonCreator
+        static ObjectWrapper jsonValue(final Object object) {
+            return new ObjectWrapper(object);
+        }
+    }
+
     static class ObjectWrapperMap extends HashMap<String, ObjectWrapper> { }
-    
+
+    @Test
     public void testSpecialMap() throws IOException
     {
        final ObjectWrapperMap map = MAPPER.readValue(UNTYPED_MAP_JSON, ObjectWrapperMap.class);
@@ -167,6 +191,7 @@ public class MapDeserializationTest
        _doTestUntyped(map);
     }
 
+    @Test
     public void testGenericMap() throws IOException
     {
         final Map<String, ObjectWrapper> map = MAPPER.readValue
@@ -174,7 +199,7 @@ public class MapDeserializationTest
              new TypeReference<Map<String, ObjectWrapper>>() { });
        _doTestUntyped(map);
     }
-    
+
     private void _doTestUntyped(final Map<String, ObjectWrapper> map)
     {
         ObjectWrapper w = map.get("double");
@@ -187,8 +212,9 @@ public class MapDeserializationTest
         assertNull(map.get("null"));
         assertEquals(5, map.size());
     }
-    
+
     // [JACKSON-620]: allow "" to mean 'null' for Maps
+    @Test
     public void testFromEmptyString() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
@@ -203,6 +229,7 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testExactStringIntMap() throws Exception
     {
         // to get typing, must use type reference
@@ -225,6 +252,7 @@ public class MapDeserializationTest
      * Let's also check that it is possible to do type conversions
      * to allow use of non-String Map keys.
      */
+    @Test
     public void testIntBooleanMap() throws Exception
     {
         // to get typing, must use type reference
@@ -242,6 +270,7 @@ public class MapDeserializationTest
         assertNull(result.get(0));
     }
 
+    @Test
     public void testExactStringStringMap() throws Exception
     {
         // to get typing, must use type reference
@@ -264,6 +293,7 @@ public class MapDeserializationTest
      * be replaced by something like <code>HashMap.class</code>,
      * if given.
      */
+    @Test
     public void testGenericStringIntMap() throws Exception
     {
         // to get typing, must use type reference; but with abstract type
@@ -281,6 +311,7 @@ public class MapDeserializationTest
         assertNull(result.get(""));
     }
 
+    @Test
     public void testAbstractMapDefault() throws Exception
     {
         final AbstractMapWrapper result = MAPPER.readValue("{\"values\":{\"foo\":42}}",
@@ -295,6 +326,7 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testEnumMap() throws Exception
     {
         String JSON = "{ \"KEY1\" : \"\", \"WHATEVER\" : null }";
@@ -317,6 +349,7 @@ public class MapDeserializationTest
         assertNull(result.get(Key.KEY2));
     }
 
+    @Test
     public void testMapWithEnums() throws Exception
     {
         String JSON = "{ \"KEY2\" : \"WHATEVER\" }";
@@ -334,7 +367,8 @@ public class MapDeserializationTest
         assertNull(result.get(Key.KEY1));
     }
 
-    public void testEnumPolymorphicSerializationTest() throws Exception 
+    @Test
+    public void testEnumPolymorphicSerializationTest() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
         List<ITestType> testTypesList = new ArrayList<ITestType>();
@@ -349,7 +383,7 @@ public class MapDeserializationTest
         testTypesMap.put(KeyEnum.A, ConcreteType.ONE);
         testTypesMap.put(KeyEnum.B, ConcreteType.TWO);
         enumMapContainer.testTypes = testTypesMap;
-        
+
         json = mapper.writeValueAsString(enumMapContainer);
         enumMapContainer = mapper.readValue(json, EnumMapContainer.class);
     }
@@ -359,19 +393,20 @@ public class MapDeserializationTest
     /* Test methods, maps with Date
     /**********************************************************
      */
+    @Test
     public void testDateMap() throws Exception
     {
     	 Date date1=new Date(123456000L);
     	 DateFormat fmt = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
-         
+
     	 String JSON = "{ \""+  fmt.format(date1)+"\" : \"\", \""+new Date(0).getTime()+"\" : null }";
     	 HashMap<Date,String> result=  MAPPER.readValue
     	            (JSON, new TypeReference<HashMap<Date,String>>() { });
-    	 
+
     	 assertNotNull(result);
     	 assertEquals(HashMap.class, result.getClass());
     	 assertEquals(2, result.size());
-    	 
+
     	 assertTrue(result.containsKey(date1));
     	 assertEquals("", result.get(new Date(123456000L)));
 
@@ -385,10 +420,11 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testCalendarMap() throws Exception
     {
         // 18-Jun-2015, tatu: Should be safest to use default timezone that mapper would use
-        TimeZone tz = MAPPER.getSerializationConfig().getTimeZone();        
+        TimeZone tz = MAPPER.getSerializationConfig().getTimeZone();
         Calendar c = Calendar.getInstance(tz);
 
         c.setTimeInMillis(123456000L);
@@ -408,6 +444,7 @@ public class MapDeserializationTest
         assertNull(result.get(c));
     }
 
+    @Test
     public void testUUIDKeyMap() throws Exception
     {
          UUID key = UUID.nameUUIDFromBytes("foobar".getBytes("UTF-8"));
@@ -421,6 +458,7 @@ public class MapDeserializationTest
          assertEquals(key, ob);
     }
 
+    @Test
     public void testLocaleKeyMap() throws Exception {
         Locale key = Locale.CHINA;
         String JSON = "{ \"" + key + "\":4}";
@@ -434,6 +472,7 @@ public class MapDeserializationTest
         assertEquals(key, ob);
     }
 
+    @Test
     public void testCurrencyKeyMap() throws Exception {
         Currency key = Currency.getInstance("USD");
         String JSON = "{ \"" + key + "\":4}";
@@ -448,6 +487,7 @@ public class MapDeserializationTest
     }
 
     // Test confirming that @JsonCreator may be used with Map Key types
+    @Test
     public void testKeyWithCreator() throws Exception
     {
         // first, key should deserialize normally:
@@ -460,6 +500,7 @@ public class MapDeserializationTest
         assertEquals("foo", key.value);
     }
 
+    @Test
     public void testClassKeyMap() throws Exception {
         ClassStringMap map = MAPPER.readValue(a2q("{'java.lang.String':'foo'}"),
                 ClassStringMap.class);
@@ -468,6 +509,7 @@ public class MapDeserializationTest
         assertEquals("foo", map.get(String.class));
     }
 
+    @Test
     public void testcharSequenceKeyMap() throws Exception {
         String JSON = a2q("{'a':'b'}");
         Map<CharSequence,String> result = MAPPER.readValue(JSON, new TypeReference<Map<CharSequence,String>>() { });
@@ -486,6 +528,7 @@ public class MapDeserializationTest
      * Simple test to ensure that @JsonDeserialize.using is
      * recognized
      */
+    @Test
     public void testMapWithDeserializer() throws Exception
     {
         CustomMap result = MAPPER.readValue(q("xyz"), CustomMap.class);
@@ -499,10 +542,11 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
+    @Test
     public void testMapError() throws Exception
     {
         try {
-            Object result = MAPPER.readValue("[ 1, 2 ]", 
+            Object result = MAPPER.readValue("[ 1, 2 ]",
                     new TypeReference<Map<String,String>>() { });
             fail("Expected an exception, but got result value: "+result);
         } catch (MismatchedInputException jex) {
@@ -510,6 +554,7 @@ public class MapDeserializationTest
         }
     }
 
+    @Test
     public void testNoCtorMap() throws Exception
     {
         try {
