@@ -5,6 +5,8 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
@@ -18,13 +20,16 @@ import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
 import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 
+import static org.junit.jupiter.api.Assertions.*;
+
+import static com.fasterxml.jackson.databind.testutil.DatabindTestUtil.*;
+
 /**
  * Unit tests for verifying "raw" (or "untyped") data binding from JSON to JDK objects;
  * one that only uses core JDK types; wrappers, Maps and Lists.
  */
 @SuppressWarnings("serial")
 public class UntypedDeserializationTest
-    extends BaseMapTest
 {
     static class UCStringDeserializer
         extends StdScalarDeserializer<String>
@@ -41,7 +46,7 @@ public class UntypedDeserializationTest
         extends StdScalarDeserializer<Number>
     {
         protected final Integer value;
-        
+
         public CustomNumberDeserializer(int nr) {
             super(Number.class);
             value = nr;
@@ -75,7 +80,7 @@ public class UntypedDeserializationTest
                 BeanProperty property) throws JsonMappingException
         {
             // For now, we just need to access "untyped" deserializer; not use it.
-            
+
             /*JsonDeserializer<Object> ob = */
             ctxt.findContextualValueDeserializer(ctxt.constructType(Object.class), property);
             return this;
@@ -92,7 +97,7 @@ public class UntypedDeserializationTest
         {
             Map<String,Object> map = new LinkedHashMap<String,Object>();
             while (p.nextValue() != JsonToken.END_OBJECT) {
-                map.put(p.getCurrentName(), "Y"+p.getText());
+                map.put(p.currentName(), "Y"+p.getText());
             }
             return map;
         }
@@ -100,7 +105,7 @@ public class UntypedDeserializationTest
 
     static class DelegatingUntyped {
         protected Object value;
-        
+
         @JsonCreator // delegating
         public DelegatingUntyped(Object v) {
             value = v;
@@ -110,6 +115,9 @@ public class UntypedDeserializationTest
     static class WrappedPolymorphicUntyped {
         @JsonTypeInfo(use=JsonTypeInfo.Id.CLASS)
         public Object value;
+
+        protected WrappedPolymorphicUntyped() { }
+        public WrappedPolymorphicUntyped(Object o) { value = o; }
     }
 
     static class WrappedUntyped1460 {
@@ -127,9 +135,10 @@ public class UntypedDeserializationTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
-    
+    private final ObjectMapper MAPPER = newJsonMapper();
+
     @SuppressWarnings("unchecked")
+    @Test
     public void testSampleDoc() throws Exception
     {
         final String JSON = SAMPLE_DOC_JSON_SPEC;
@@ -139,29 +148,29 @@ public class UntypedDeserializationTest
          */
         Object root = MAPPER.readValue(JSON, Object.class);
 
-        assertType(root, Map.class);
+        assertInstanceOf(Map.class, root);
         Map<?,?> rootMap = (Map<?,?>) root;
         assertEquals(1, rootMap.size());
         Map.Entry<?,?> rootEntry =  rootMap.entrySet().iterator().next();
         assertEquals("Image", rootEntry.getKey());
         Object image = rootEntry.getValue();
-        assertType(image, Map.class);
+        assertInstanceOf(Map.class, image);
         Map<?,?> imageMap = (Map<?,?>) image;
         assertEquals(5, imageMap.size());
 
         Object value = imageMap.get("Width");
-        assertType(value, Integer.class);
+        assertInstanceOf(Integer.class, value);
         assertEquals(Integer.valueOf(SAMPLE_SPEC_VALUE_WIDTH), value);
 
         value = imageMap.get("Height");
-        assertType(value, Integer.class);
+        assertInstanceOf(Integer.class, value);
         assertEquals(Integer.valueOf(SAMPLE_SPEC_VALUE_HEIGHT), value);
 
         assertEquals(SAMPLE_SPEC_VALUE_TITLE, imageMap.get("Title"));
 
         // Another Object, "thumbnail"
         value = imageMap.get("Thumbnail");
-        assertType(value, Map.class);
+        assertInstanceOf(Map.class, value);
         Map<?,?> tnMap = (Map<?,?>) value;
         assertEquals(3, tnMap.size());
 
@@ -172,7 +181,7 @@ public class UntypedDeserializationTest
 
         // And then number list, "IDs"
         value = imageMap.get("IDs");
-        assertType(value, List.class);
+        assertInstanceOf(List.class, value);
         List<Object> ids = (List<Object>) value;
         assertEquals(4, ids.size());
         assertEquals(Integer.valueOf(SAMPLE_SPEC_VALUE_TN_ID1), ids.get(0));
@@ -183,7 +192,8 @@ public class UntypedDeserializationTest
         // and that's all folks!
     }
 
-    @SuppressWarnings("unlikely-arg-type")
+    @SuppressWarnings({"unlikely-arg-type", "CollectionIncompatibleType"})
+    @Test
     public void testUntypedMap() throws Exception
     {
         // to get "untyped" default map-to-map, pass Object.class
@@ -206,10 +216,29 @@ public class UntypedDeserializationTest
         assertNull(result.get(3));
     }
 
+    @Test
+    public void testSimpleVanillaScalars() throws IOException
+    {
+        assertEquals("foo", MAPPER.readValue(q("foo"), Object.class));
+
+        assertEquals(Boolean.TRUE, MAPPER.readValue(" true ", Object.class));
+
+        assertEquals(Integer.valueOf(13), MAPPER.readValue("13 ", Object.class));
+        assertEquals(Double.valueOf(0.5), MAPPER.readValue("0.5 ", Object.class));
+    }
+
+    @Test
+    public void testSimpleVanillaStructured() throws IOException
+    {
+        List<?> list = (List<?>) MAPPER.readValue("[ 1, 2, 3]", Object.class);
+        assertEquals(Integer.valueOf(1), list.get(0));
+    }
+
+    @Test
     public void testNestedUntypes() throws IOException
     {
         // 05-Apr-2014, tatu: Odd failures if using shared mapper; so work around:
-        Object root = MAPPER.readValue(aposToQuotes("{'a':3,'b':[1,2]}"),
+        Object root = MAPPER.readValue(a2q("{'a':3,'b':[1,2]}"),
                 Object.class);
         assertTrue(root instanceof Map<?,?>);
         Map<?,?> map = (Map<?,?>) root;
@@ -222,6 +251,7 @@ public class UntypedDeserializationTest
         assertEquals(Integer.valueOf(2), l.get(1));
     }
 
+    @Test
     public void testUntypedWithCustomScalarDesers() throws IOException
     {
         SimpleModule m = new SimpleModule("test-module");
@@ -244,6 +274,7 @@ public class UntypedDeserializationTest
     }
 
     // Test that exercises non-vanilla variant, with just one simple custom deserializer
+    @Test
     public void testNonVanilla() throws IOException
     {
         SimpleModule m = new SimpleModule("test-module");
@@ -264,41 +295,9 @@ public class UntypedDeserializationTest
         assertEquals(2, l.size());
         assertTrue(l.get(0) instanceof Map<?,?>);
         assertTrue(l.get(1) instanceof List<?>);
-
-        ObjectReader rDefault = mapper.readerFor(WrappedPolymorphicUntyped.class);
-        ObjectReader rAlt = rDefault
-                .with(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS,
-                        DeserializationFeature.USE_BIG_INTEGER_FOR_INTS);
-        WrappedPolymorphicUntyped w;
-
-        w = rDefault.readValue(aposToQuotes("{'value':10}"));
-        assertEquals(Integer.valueOf(10), w.value);
-        w = rAlt.readValue(aposToQuotes("{'value':10}"));
-        assertEquals(BigInteger.TEN, w.value);
-
-        w = rDefault.readValue(aposToQuotes("{'value':5.0}"));
-        assertEquals(Double.valueOf(5.0), w.value);
-        w = rAlt.readValue(aposToQuotes("{'value':5.0}"));
-        assertEquals(new BigDecimal("5.0"), w.value);
-
-        StringBuilder sb = new StringBuilder(100).append("[0");
-        for (int i = 1; i < 100; ++i) {
-            sb.append(", ").append(i);
-        }
-        sb.append("]");
-        final String INT_ARRAY_JSON = sb.toString();
-
-        // First read as-is, no type wrapping
-        Object ob = mapper.readerFor(Object.class)
-                .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
-                .readValue(INT_ARRAY_JSON);
-        assertTrue(ob instanceof Object[]);
-        Object[] obs = (Object[]) ob;
-        for (int i = 0; i < 100; ++i) {
-            assertEquals(Integer.valueOf(i), obs[i]);
-        }
     }
 
+    @Test
     public void testUntypedWithListDeser() throws IOException
     {
         SimpleModule m = new SimpleModule("test-module");
@@ -316,6 +315,7 @@ public class UntypedDeserializationTest
         assertEquals("Xtrue", l.get(2));
     }
 
+    @Test
     public void testUntypedWithMapDeser() throws IOException
     {
         SimpleModule m = new SimpleModule("test-module");
@@ -331,6 +331,7 @@ public class UntypedDeserializationTest
         assertEquals("Ytrue", map.get("a"));
     }
 
+    @Test
     public void testNestedUntyped989() throws IOException
     {
         DelegatingUntyped pojo;
@@ -340,13 +341,14 @@ public class UntypedDeserializationTest
         assertTrue(pojo.value instanceof List);
         pojo = r.readValue("[{}]");
         assertTrue(pojo.value instanceof List);
-        
+
         pojo = r.readValue("{}");
         assertTrue(pojo.value instanceof Map);
         pojo = r.readValue("{\"a\":[]}");
         assertTrue(pojo.value instanceof Map);
     }
 
+    @Test
     public void testUntypedWithJsonArrays() throws Exception
     {
         // by default we get:
@@ -354,14 +356,16 @@ public class UntypedDeserializationTest
         assertTrue(ob instanceof List<?>);
 
         // but can change to produce Object[]:
-        MAPPER.configure(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY, true);
-        ob = MAPPER.readValue("[1]", Object.class);
+        ob = MAPPER.readerFor(Object.class)
+            .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
+            .readValue("[1]");
         assertEquals(Object[].class, ob.getClass());
     }
 
+    @Test
     public void testUntypedIntAsLong() throws Exception
     {
-        final String JSON = aposToQuotes("{'value':3}");
+        final String JSON = a2q("{'value':3}");
         WrappedUntyped1460 w = MAPPER.readerFor(WrappedUntyped1460.class)
                 .readValue(JSON);
         assertEquals(Integer.valueOf(3), w.value);
@@ -374,13 +378,14 @@ public class UntypedDeserializationTest
 
     // [databind#2115]: Consider `java.io.Serializable` as sort of alias of `java.lang.Object`
     // since all natural target types do implement `Serializable` so assignment works
+    @Test
     public void testSerializable() throws Exception
     {
-        final String JSON1 = aposToQuotes("{ 'value' : 123 }");
+        final String JSON1 = a2q("{ 'value' : 123 }");
         SerContainer cont = MAPPER.readValue(JSON1, SerContainer.class);
         assertEquals(Integer.valueOf(123), cont.value);
 
-        cont = MAPPER.readValue(aposToQuotes("{ 'value' : true }"), SerContainer.class);
+        cont = MAPPER.readValue(a2q("{ 'value' : true }"), SerContainer.class);
         assertEquals(Boolean.TRUE, cont.value);
 
         // But also via Map value, even key
@@ -393,6 +398,63 @@ public class UntypedDeserializationTest
         assertEquals("value", map.keySet().iterator().next());
     }
 
+    /*
+    /**********************************************************
+    /* Test methods, merging
+    /**********************************************************
+     */
+
+    @Test
+    public void testValueUpdateVanillaUntyped() throws Exception
+    {
+        Map<String, Object> map = new LinkedHashMap<>();
+        map.put("a", 42);
+
+        ObjectReader r = MAPPER.readerFor(Object.class).withValueToUpdate(map);
+        Object result = r.readValue(a2q("{'b' : 57}"));
+        assertSame(map, result);
+        assertEquals(2, map.size());
+        assertEquals(Integer.valueOf(57), map.get("b"));
+
+        // Try same with other types, too
+        List<Object> list = new ArrayList<>();
+        list.add(1);
+        r = MAPPER.readerFor(Object.class).withValueToUpdate(list);
+        result = r.readValue("[ 2, true ]");
+        assertSame(list, result);
+        assertEquals(3, list.size());
+        assertEquals(Boolean.TRUE, list.get(2));
+    }
+
+    @Test
+    public void testValueUpdateCustomUntyped() throws Exception
+    {
+        SimpleModule m = new SimpleModule("test-module")
+                .addDeserializer(String.class, new UCStringDeserializer());
+        final ObjectMapper customMapper = newJsonMapper()
+            .registerModule(m);
+
+        Map<String, Object> map = new LinkedHashMap<>();
+        map.put("a", 42);
+
+        ObjectReader r = customMapper.readerFor(Object.class).withValueToUpdate(map);
+        Object result = r.readValue(a2q("{'b' : 'value', 'c' : 111222333444, 'enabled':true}"));
+        assertSame(map, result);
+        assertEquals(4, map.size());
+        assertEquals("VALUE", map.get("b"));
+        assertEquals(Long.valueOf(111222333444L), map.get("c"));
+        assertEquals(Boolean.TRUE, map.get("enabled"));
+
+        // Try same with other types, too
+        List<Object> list = new ArrayList<>();
+        list.add(1);
+        r = customMapper.readerFor(Object.class).withValueToUpdate(list);
+        result = r.readValue(a2q("[ 2, 'foobar' ]"));
+        assertSame(list, result);
+        assertEquals(3, list.size());
+        assertEquals("FOOBAR", list.get(2));
+    }
+
     /*
     /**********************************************************
     /* Test methods, polymorphic
@@ -400,6 +462,7 @@ public class UntypedDeserializationTest
      */
 
     // Allow 'upgrade' of big integers into Long, BigInteger
+    @Test
     public void testObjectSerializeWithLong() throws IOException
     {
         final ObjectMapper mapper = newJsonMapper();
@@ -418,4 +481,74 @@ public class UntypedDeserializationTest
         assertSame(Long.class, n.getClass());
         assertEquals(Long.valueOf(VALUE), n);
     }
+
+    @Test
+    public void testPolymorphicUntypedVanilla() throws IOException
+    {
+        ObjectReader rDefault = MAPPER.readerFor(WrappedPolymorphicUntyped.class);
+        ObjectReader rAlt = rDefault
+                .with(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS,
+                        DeserializationFeature.USE_BIG_INTEGER_FOR_INTS);
+        WrappedPolymorphicUntyped w;
+
+        w = rDefault.readValue(a2q("{'value':10}"));
+        assertEquals(Integer.valueOf(10), w.value);
+        w = rAlt.readValue(a2q("{'value':10}"));
+        assertEquals(BigInteger.TEN, w.value);
+
+        w = rDefault.readValue(a2q("{'value':5.0}"));
+        assertEquals(Double.valueOf(5.0), w.value);
+        w = rAlt.readValue(a2q("{'value':5.0}"));
+        assertEquals(new BigDecimal("5.0"), w.value);
+
+        StringBuilder sb = new StringBuilder(100).append("[0");
+        for (int i = 1; i < 100; ++i) {
+            sb.append(", ").append(i);
+        }
+        sb.append("]");
+        final String INT_ARRAY_JSON = sb.toString();
+
+        // First read as-is, no type wrapping
+        Object ob = MAPPER.readerFor(Object.class)
+                .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
+                .readValue(INT_ARRAY_JSON);
+        assertTrue(ob instanceof Object[]);
+        Object[] obs = (Object[]) ob;
+        for (int i = 0; i < 100; ++i) {
+            assertEquals(Integer.valueOf(i), obs[i]);
+        }
+    }
+
+    @Test
+    public void testPolymorphicUntypedCustom() throws IOException
+    {
+        // register module just to override one deserializer, to prevent use of Vanilla deser
+        SimpleModule m = new SimpleModule("test-module")
+                .addDeserializer(String.class, new UCStringDeserializer());
+        final ObjectMapper customMapper = newJsonMapper()
+            .registerModule(m);
+        ObjectReader rDefault = customMapper.readerFor(WrappedPolymorphicUntyped.class);
+
+        WrappedPolymorphicUntyped w = rDefault.readValue(a2q("{'value':10}"));
+        assertEquals(Integer.valueOf(10), w.value);
+
+        w = rDefault.readValue(a2q("{'value':9988776655}"));
+        assertEquals(Long.valueOf(9988776655L), w.value);
+        w = rDefault.readValue(a2q("{'value':0.75}"));
+        assertEquals(Double.valueOf(0.75), w.value);
+
+        w = rDefault.readValue(a2q("{'value':'abc'}"));
+        assertEquals("ABC", w.value);
+        w = rDefault.readValue(a2q("{'value':false}"));
+        assertEquals(Boolean.FALSE, w.value);
+        w = rDefault.readValue(a2q("{'value':null}"));
+        assertNull(w.value);
+
+        // but... actually how about real type id?
+        final Object SHORT = Short.valueOf((short) 3);
+        String json = customMapper.writeValueAsString(new WrappedPolymorphicUntyped(SHORT));
+
+        WrappedPolymorphicUntyped result = rDefault.readValue(json);
+        assertEquals(SHORT, result.value);
+    }
 }
