@@ -3,10 +3,15 @@ package com.fasterxml.jackson.databind.format;
 import java.util.Calendar;
 import java.util.Date;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 
-public class DateFormatTest extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+public class DateFormatTest extends DatabindTestUtil
 {
     protected static class DateWrapper {
         public Date value;
@@ -16,6 +21,7 @@ public class DateFormatTest extends BaseMapTest
         public DateWrapper(Date v) { value = v; }
     }
 
+    @Test
     public void testTypeDefaults() throws Exception
     {
         ObjectMapper mapper = newJsonMapper();
@@ -23,10 +29,10 @@ public class DateFormatTest extends BaseMapTest
             .setFormat(JsonFormat.Value.forPattern("yyyy.dd.MM"));
         // First serialize, should result in this (in UTC):
         String json = mapper.writeValueAsString(new DateWrapper(0L));
-        assertEquals(aposToQuotes("{'value':'1970.01.01'}"), json);
+        assertEquals(a2q("{'value':'1970.01.01'}"), json);
 
         // and then read back
-        DateWrapper w = mapper.readValue(aposToQuotes("{'value':'1981.13.3'}"), DateWrapper.class);
+        DateWrapper w = mapper.readValue(a2q("{'value':'1981.13.3'}"), DateWrapper.class);
         assertNotNull(w);
         // arbitrary TimeZone, but good enough to ensure year is right
         Calendar c = Calendar.getInstance();
