@@ -3,11 +3,17 @@ package com.fasterxml.jackson.databind.format;
 import java.util.*;
 import java.util.concurrent.atomic.AtomicReference;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.JsonInclude;
+import com.fasterxml.jackson.annotation.JsonPropertyOrder;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 
-public class MapEntryFormatTest extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+public class MapEntryFormatTest extends DatabindTestUtil
 {
     static class BeanWithMapEntry {
         // would work with any other shape than OBJECT, or without annotation:
@@ -16,13 +22,14 @@ public class MapEntryFormatTest extends BaseMapTest
 
         protected BeanWithMapEntry() { }
         public BeanWithMapEntry(String key, String value) {
-            Map<String,String> map = new HashMap<>();
+            Map<String,String> map = new LinkedHashMap<>();
             map.put(key, value);
             entry = map.entrySet().iterator().next();
         }
     }
 
     @JsonFormat(shape=JsonFormat.Shape.OBJECT)
+    @JsonPropertyOrder({ "key", "value" })
     static class MapEntryAsObject implements Map.Entry<String,String> {
         protected String key, value;
 
@@ -31,7 +38,7 @@ public class MapEntryFormatTest extends BaseMapTest
             key = k;
             value = v;
         }
-        
+
         @Override
         public String getKey() {
             return key;
@@ -84,7 +91,7 @@ public class MapEntryFormatTest extends BaseMapTest
             entry = map.entrySet().iterator().next();
         }
     }
-    
+
     static class EmptyEntryWrapper {
         @JsonInclude(value=JsonInclude.Include.NON_EMPTY,
                 content=JsonInclude.Include.NON_EMPTY)
@@ -96,84 +103,90 @@ public class MapEntryFormatTest extends BaseMapTest
             entry = map.entrySet().iterator().next();
         }
     }
-    
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Test methods, basic
-    /**********************************************************
+    /**********************************************************************
      */
-    
+
     private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testInclusion() throws Exception
     {
-        assertEquals(aposToQuotes("{'entry':{'a':'b'}}"),
+        assertEquals(a2q("{'entry':{'a':'b'}}"),
                 MAPPER.writeValueAsString(new EmptyEntryWrapper("a", "b")));
-        assertEquals(aposToQuotes("{'entry':{'a':'b'}}"),
+        assertEquals(a2q("{'entry':{'a':'b'}}"),
                 MAPPER.writeValueAsString(new EntryWithDefaultWrapper("a", "b")));
-        assertEquals(aposToQuotes("{'entry':{'a':'b'}}"),
+        assertEquals(a2q("{'entry':{'a':'b'}}"),
                 MAPPER.writeValueAsString(new EntryWithNullWrapper("a", "b")));
 
-        assertEquals(aposToQuotes("{}"),
+        assertEquals(a2q("{}"),
                 MAPPER.writeValueAsString(new EmptyEntryWrapper("a", "")));
-        assertEquals(aposToQuotes("{}"),
+        assertEquals(a2q("{}"),
                 MAPPER.writeValueAsString(new EntryWithDefaultWrapper("a", "")));
-        assertEquals(aposToQuotes("{'entry':{'a':''}}"),
+        assertEquals(a2q("{'entry':{'a':''}}"),
                 MAPPER.writeValueAsString(new EntryWithNullWrapper("a", "")));
-        assertEquals(aposToQuotes("{}"),
+        assertEquals(a2q("{}"),
                 MAPPER.writeValueAsString(new EntryWithNullWrapper("a", null)));
     }
 
+    @Test
     public void testInclusionWithReference() throws Exception
     {
-        assertEquals(aposToQuotes("{'entry':{'a':'b'}}"),
+        assertEquals(a2q("{'entry':{'a':'b'}}"),
                 MAPPER.writeValueAsString(new EntryWithNonAbsentWrapper("a", "b")));
         // empty String not excluded since reference is not absent, just points to empty
         // (so would need 3rd level inclusion definition)
-        assertEquals(aposToQuotes("{'entry':{'a':''}}"),
+        assertEquals(a2q("{'entry':{'a':''}}"),
                 MAPPER.writeValueAsString(new EntryWithNonAbsentWrapper("a", "")));
-        assertEquals(aposToQuotes("{}"),
+        assertEquals(a2q("{}"),
                 MAPPER.writeValueAsString(new EntryWithNonAbsentWrapper("a", null)));
     }
 
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Test methods, as-Object (Shape)
-    /**********************************************************
+    /**********************************************************************
      */
 
+    @Test
     public void testAsNaturalRoundtrip() throws Exception
     {
         BeanWithMapEntry input = new BeanWithMapEntry("foo" ,"bar");
         String json = MAPPER.writeValueAsString(input);
-        assertEquals(aposToQuotes("{'entry':{'foo':'bar'}}"), json);
+        assertEquals(a2q("{'entry':{'foo':'bar'}}"), json);
         BeanWithMapEntry result = MAPPER.readValue(json, BeanWithMapEntry.class);
         assertEquals("foo", result.entry.getKey());
         assertEquals("bar", result.entry.getValue());
     }
+
     // should work via class annotation
+    @Test
     public void testAsObjectRoundtrip() throws Exception
     {
         MapEntryAsObject input = new MapEntryAsObject("foo" ,"bar");
         String json = MAPPER.writeValueAsString(input);
-        assertEquals(aposToQuotes("{'key':'foo','value':'bar'}"), json);
+        assertEquals(a2q("{'key':'foo','value':'bar'}"), json);
 
         // 16-Oct-2016, tatu: Happens to work by default because it's NOT basic
         //   `Map.Entry` but subtype.
-        
+
         MapEntryAsObject result = MAPPER.readValue(json, MapEntryAsObject.class);
         assertEquals("foo", result.getKey());
         assertEquals("bar", result.getValue());
     }
 
     // [databind#1895]
+    @Test
     public void testDefaultShapeOverride() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
-        mapper.configOverride(Map.Entry.class)
-            .setFormat(JsonFormat.Value.forShape(JsonFormat.Shape.OBJECT));
+        ObjectMapper mapper = jsonMapperBuilder()
+                .withConfigOverride(Map.Entry.class, cfg ->
+                    cfg.setFormat(JsonFormat.Value.forShape(JsonFormat.Shape.OBJECT)))
+                .build();
         Map.Entry<String,String> input = new BeanWithMapEntry("foo", "bar").entry;
-        assertEquals(aposToQuotes("{'key':'foo','value':'bar'}"),
-                mapper.writeValueAsString(input));
+        assertTrue(mapper.writeValueAsString(input).equals(a2q("{'key':'foo','value':'bar'}")) || mapper.writeValueAsString(input).equals(a2q("{'value':'bar','key':'foo'}")));
     }
 }
