@@ -3,23 +3,33 @@ package com.fasterxml.jackson.databind.introspect;
 import java.lang.annotation.Annotation;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
+import com.fasterxml.jackson.annotation.JacksonInject;
+import com.fasterxml.jackson.annotation.JsonIgnore;
 import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.annotation.JsonInclude;
 
+import com.fasterxml.jackson.annotation.JsonProperty;
+import com.fasterxml.jackson.annotation.OptBoolean;
 import com.fasterxml.jackson.core.Version;
 
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
+import com.fasterxml.jackson.databind.deser.std.NumberDeserializers;
 import com.fasterxml.jackson.databind.deser.std.StringDeserializer;
-import com.fasterxml.jackson.databind.deser.std.UntypedObjectDeserializer;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.jsontype.NamedType;
 import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
 import com.fasterxml.jackson.databind.ser.std.StringSerializer;
 import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
+
+import static org.junit.jupiter.api.Assertions.*;
 
 // started with [databind#1025] in mind
 @SuppressWarnings("serial")
-public class IntrospectorPairTest extends BaseMapTest
+public class IntrospectorPairTest extends DatabindTestUtil
 {
     static class Introspector1 extends AnnotationIntrospector {
         @Override
@@ -111,7 +121,7 @@ public class IntrospectorPairTest extends BaseMapTest
         /******************************************************
         /* General class annotations
         /******************************************************
-         */        
+         */
 
         @Override
         public PropertyName findRootName(AnnotatedClass ac) {
@@ -119,8 +129,8 @@ public class IntrospectorPairTest extends BaseMapTest
         }
 
         @Override
-        public JsonIgnoreProperties.Value findPropertyIgnorals(Annotated a) {
-            return (JsonIgnoreProperties.Value) values.get("findPropertyIgnorals");
+        public JsonIgnoreProperties.Value findPropertyIgnoralByName(MapperConfig<?> config, Annotated a) {
+            return (JsonIgnoreProperties.Value) values.get("findPropertyIgnoralByName");
         }
 
         @Override
@@ -132,7 +142,7 @@ public class IntrospectorPairTest extends BaseMapTest
         public Object findFilterId(Annotated ann) {
             return (Object) values.get("findFilterId");
         }
-        
+
         @Override
         public Object findNamingStrategy(AnnotatedClass ac) {
             return (Object) values.get("findNamingStrategy");
@@ -184,7 +194,7 @@ public class IntrospectorPairTest extends BaseMapTest
         {
             return (TypeResolverBuilder<?>) values.get("findPropertyContentTypeResolver");
         }
-        
+
         @SuppressWarnings("unchecked")
         @Override
         public List<NamedType> findSubtypes(Annotated a)
@@ -197,6 +207,38 @@ public class IntrospectorPairTest extends BaseMapTest
             return (String) values.get("findTypeName");
         }
 
+        /*
+        /******************************************************
+        /* General member (field, method/constructor) annotations
+        /******************************************************
+         */
+
+        @Override
+        public PropertyName findWrapperName(Annotated ann) {
+            return (PropertyName) values.get("findWrapperName");
+        }
+
+        /*
+        /******************************************************
+        /* Serialization introspection
+        /******************************************************
+         */
+
+        @Override
+        public Boolean hasAsKey(MapperConfig<?> config, Annotated a) {
+            return (Boolean) values.get("hasAsKey");
+        }
+
+        @Override
+        public Boolean hasAsValue(Annotated a) {
+            return (Boolean) values.get("hasAsValue");
+        }
+
+        @Override
+        public Boolean hasAnyGetter(Annotated ann) {
+            return (Boolean) values.get("hasAnyGetter");
+        }
+
         /*
         /******************************************************
         /* Deserialization introspection
@@ -212,7 +254,7 @@ public class IntrospectorPairTest extends BaseMapTest
         /******************************************************
         /* Helper methods
         /******************************************************
-         */        
+         */
 
         private boolean _boolean(String key) {
             Object ob = values.get(key);
@@ -228,6 +270,7 @@ public class IntrospectorPairTest extends BaseMapTest
 
     private final AnnotationIntrospector NO_ANNOTATIONS = AnnotationIntrospector.nopInstance();
 
+    @Test
     public void testVersion() throws Exception
     {
         Version v = new Version(1, 2, 3, null,
@@ -241,6 +284,7 @@ public class IntrospectorPairTest extends BaseMapTest
                 new AnnotationIntrospectorPair(noVersion, withVersion).version());
     }
 
+    @Test
     public void testAccess() throws Exception
     {
         IntrospectorWithMap intr1 = new IntrospectorWithMap();
@@ -253,6 +297,7 @@ public class IntrospectorPairTest extends BaseMapTest
         assertSame(NO_ANNOTATIONS, it.next());
     }
 
+    @Test
     public void testAnnotationBundle() throws Exception
     {
         IntrospectorWithMap isBundle = new IntrospectorWithMap()
@@ -271,6 +316,7 @@ public class IntrospectorPairTest extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testFindRootName() throws Exception
     {
         PropertyName name = new PropertyName("test");
@@ -281,20 +327,22 @@ public class IntrospectorPairTest extends BaseMapTest
         assertEquals(name, new AnnotationIntrospectorPair(intr, NO_ANNOTATIONS).findRootName(null));
     }
 
+    @Test
     public void testPropertyIgnorals() throws Exception
     {
         JsonIgnoreProperties.Value incl = JsonIgnoreProperties.Value.forIgnoredProperties("foo");
         IntrospectorWithMap intr = new IntrospectorWithMap()
-                .add("findPropertyIgnorals", incl);
+                .add("findPropertyIgnoralByName", incl);
         IntrospectorWithMap intrEmpty = new IntrospectorWithMap()
-                .add("findPropertyIgnorals", JsonIgnoreProperties.Value.empty());
+                .add("findPropertyIgnoralByName", JsonIgnoreProperties.Value.empty());
         assertEquals(JsonIgnoreProperties.Value.empty(),
-                new AnnotationIntrospectorPair(intrEmpty, intrEmpty).findPropertyIgnorals(null));
+                new AnnotationIntrospectorPair(intrEmpty, intrEmpty).findPropertyIgnoralByName(null, null));
         // should actually verify inclusion combining, but there are separate tests for that
-        assertEquals(incl, new AnnotationIntrospectorPair(intrEmpty, intr).findPropertyIgnorals(null));
-        assertEquals(incl, new AnnotationIntrospectorPair(intr, intrEmpty).findPropertyIgnorals(null));
+        assertEquals(incl, new AnnotationIntrospectorPair(intrEmpty, intr).findPropertyIgnoralByName(null, null));
+        assertEquals(incl, new AnnotationIntrospectorPair(intr, intrEmpty).findPropertyIgnoralByName(null, null));
     }
 
+    @Test
     public void testIsIgnorableType() throws Exception
     {
         IntrospectorWithMap intr1 = new IntrospectorWithMap()
@@ -306,6 +354,7 @@ public class IntrospectorPairTest extends BaseMapTest
         assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, intr1).isIgnorableType(null));
     }
 
+    @Test
     public void testFindFilterId() throws Exception
     {
         IntrospectorWithMap intr1 = new IntrospectorWithMap()
@@ -317,6 +366,7 @@ public class IntrospectorPairTest extends BaseMapTest
         assertEquals("b", new AnnotationIntrospectorPair(intr2, intr1).findFilterId(null));
     }
 
+    @Test
     public void testFindNamingStrategy() throws Exception
     {
         // shouldn't be bogus Classes for real use, but works here
@@ -331,6 +381,7 @@ public class IntrospectorPairTest extends BaseMapTest
                 new AnnotationIntrospectorPair(intr2, intr1).findNamingStrategy(null));
     }
 
+    @Test
     public void testFindClassDescription() throws Exception
     {
         IntrospectorWithMap intr1 = new IntrospectorWithMap()
@@ -346,12 +397,49 @@ public class IntrospectorPairTest extends BaseMapTest
 
     // // // 3 deprecated methods, skip
 
+    /*
+    /**********************************************************
+    /* Test methods, general member annotations
+    /**********************************************************
+     */
+
+    @Test
+    public void testFindWrapperName() throws Exception
+    {
+        final PropertyName NAME_WITH_NS = PropertyName.construct("simple", "ns");
+        final PropertyName NAME_NO_NS = PropertyName.construct("other", null);
+
+        assertNull(new AnnotationIntrospectorPair(NO_ANNOTATIONS, NO_ANNOTATIONS)
+                .findClassDescription(null));
+
+        // First: basic merging of namespace/localname info
+        IntrospectorWithMap intr1 = new IntrospectorWithMap()
+                .add("findWrapperName", NAME_WITH_NS);
+        IntrospectorWithMap intr2 = new IntrospectorWithMap()
+                .add("findWrapperName", NAME_NO_NS);
+        assertSame(NAME_WITH_NS,
+                new AnnotationIntrospectorPair(intr1, intr2).findWrapperName(null));
+        assertEquals(PropertyName.construct("other", "ns"),
+                new AnnotationIntrospectorPair(intr2, intr1).findWrapperName(null));
+
+        // [databind#4595]: NO_NAME should be retained, not merged
+        intr1 = new IntrospectorWithMap()
+                .add("findWrapperName", PropertyName.NO_NAME);
+        intr2 = new IntrospectorWithMap()
+                .add("findWrapperName", NAME_WITH_NS);
+        assertSame(PropertyName.NO_NAME,
+                new AnnotationIntrospectorPair(intr1, intr2).findWrapperName(null));
+        assertSame(NAME_WITH_NS,
+                new AnnotationIntrospectorPair(intr2, intr1).findWrapperName(null));
+    }
+
     /*
     /**********************************************************
     /* Test methods, ser/deser
     /**********************************************************
      */
 
+    @Test
     public void testFindSerializer() throws Exception
     {
         final JsonSerializer<?> serString = new StringSerializer();
@@ -378,19 +466,98 @@ public class IntrospectorPairTest extends BaseMapTest
         assertNull(new AnnotationIntrospectorPair(nop2, nop).findSerializer(null));
     }
 
+    @Test
+    public void testHasAsValue() throws Exception
+    {
+        IntrospectorWithMap intr1 = new IntrospectorWithMap()
+                .add("hasAsValue", Boolean.TRUE);
+        IntrospectorWithMap intr2 = new IntrospectorWithMap()
+                .add("hasAsValue", Boolean.FALSE);
+        assertNull(new AnnotationIntrospectorPair(NO_ANNOTATIONS, NO_ANNOTATIONS)
+                .hasAsValue(null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, NO_ANNOTATIONS)
+                .hasAsValue(null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr1)
+                .hasAsValue(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, NO_ANNOTATIONS)
+                .hasAsValue(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr2)
+                .hasAsValue(null));
+
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, intr2)
+                .hasAsValue(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, intr1)
+                .hasAsValue(null));
+    }
+
+    @Test
+    public void testHasAsKey() throws Exception
+    {
+        IntrospectorWithMap intr1 = new IntrospectorWithMap()
+                .add("hasAsKey", Boolean.TRUE);
+        IntrospectorWithMap intr2 = new IntrospectorWithMap()
+                .add("hasAsKey", Boolean.FALSE);
+        assertNull(new AnnotationIntrospectorPair(NO_ANNOTATIONS, NO_ANNOTATIONS)
+                .hasAsKey(null, null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, NO_ANNOTATIONS)
+                .hasAsKey(null, null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr1)
+                .hasAsKey(null, null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, NO_ANNOTATIONS)
+                .hasAsKey(null, null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr2)
+                .hasAsKey(null, null));
+
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, intr2)
+                .hasAsKey(null, null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, intr1)
+                .hasAsKey(null, null));
+    }
+
+    @Test
+    public void testHasAnyGetter() throws Exception
+    {
+        IntrospectorWithMap intr1 = new IntrospectorWithMap()
+                .add("hasAnyGetter", Boolean.TRUE);
+        IntrospectorWithMap intr2 = new IntrospectorWithMap()
+                .add("hasAnyGetter", Boolean.FALSE);
+        assertNull(new AnnotationIntrospectorPair(NO_ANNOTATIONS, NO_ANNOTATIONS)
+                .hasAnyGetter(null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, NO_ANNOTATIONS)
+                .hasAnyGetter(null));
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr1)
+                .hasAnyGetter(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, NO_ANNOTATIONS)
+                .hasAnyGetter(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(NO_ANNOTATIONS, intr2)
+                .hasAnyGetter(null));
+
+        assertEquals(Boolean.TRUE, new AnnotationIntrospectorPair(intr1, intr2)
+                .hasAnyGetter(null));
+        assertEquals(Boolean.FALSE, new AnnotationIntrospectorPair(intr2, intr1)
+                .hasAnyGetter(null));
+    }
+
+    /*
+    /**********************************************************
+    /* Test methods, deser
+    /**********************************************************
+     */
+
+    @Test
     public void testFindDeserializer() throws Exception
     {
         final JsonDeserializer<?> deserString = StringDeserializer.instance;
-        final JsonDeserializer<?> deserObject = UntypedObjectDeserializer.Vanilla.std;
+        final JsonDeserializer<?> deserBoolean = NumberDeserializers.find(Boolean.TYPE, "b");
 
         AnnotationIntrospector intr1 = new IntrospectorWithHandlers(deserString, null);
-        AnnotationIntrospector intr2 = new IntrospectorWithHandlers(deserObject, null);
+        AnnotationIntrospector intr2 = new IntrospectorWithHandlers(deserBoolean, null);
         AnnotationIntrospector nop = AnnotationIntrospector.nopInstance();
         AnnotationIntrospector nop2 = new IntrospectorWithHandlers(JsonDeserializer.None.class, null);
 
         assertSame(deserString,
                 new AnnotationIntrospectorPair(intr1, intr2).findDeserializer(null));
-        assertSame(deserObject,
+        assertSame(deserBoolean,
                 new AnnotationIntrospectorPair(intr2, intr1).findDeserializer(null));
         // also: no-op instance should not block real one, regardless
         assertSame(deserString,
@@ -409,6 +576,7 @@ public class IntrospectorPairTest extends BaseMapTest
     /******************************************************
      */
 
+    @Test
     public void testFindAutoDetectVisibility() throws Exception
     {
         VisibilityChecker<?> vc = VisibilityChecker.Std.defaultInstance();
@@ -428,6 +596,7 @@ public class IntrospectorPairTest extends BaseMapTest
     /******************************************************
      */
 
+    @Test
     public void testFindTypeResolver() throws Exception
     {
         /*
@@ -436,15 +605,19 @@ public class IntrospectorPairTest extends BaseMapTest
         return (TypeResolverBuilder<?>) values.get("findTypeResolver");
         */
     }
+    @Test
     public void testFindPropertyTypeResolver() {
     }
 
+    @Test
     public void testFindPropertyContentTypeResolver() {
     }
 
+    @Test
     public void testFindSubtypes() {
     }
 
+    @Test
     public void testFindTypeName() {
         IntrospectorWithMap intr1 = new IntrospectorWithMap()
                 .add("findTypeName", "type1");
@@ -464,6 +637,7 @@ public class IntrospectorPairTest extends BaseMapTest
      */
 
     // for [databind#1672]
+    @Test
     public void testHasAnySetter() {
         IntrospectorWithMap intr1 = new IntrospectorWithMap()
                 .add("hasAnySetter", Boolean.TRUE);
@@ -493,6 +667,7 @@ public class IntrospectorPairTest extends BaseMapTest
         = new AnnotationIntrospectorPair(new Introspector2(), new Introspector1());
 
     // for [databind#1025]
+    @Test
     public void testInclusionMerging() throws Exception
     {
         // argument is ignored by test introspectors, may be null
@@ -505,4 +680,95 @@ public class IntrospectorPairTest extends BaseMapTest
         assertEquals(JsonInclude.Include.NON_EMPTY, v21.getContentInclusion());
         assertEquals(JsonInclude.Include.NON_ABSENT, v21.getValueInclusion());
     }
+
+    /*
+    /**********************************************************
+    /* Introspectors and test for [jackson-modules-base#134]/[databind#962]
+    /**********************************************************
+     */
+    static class TestIntrospector extends NopAnnotationIntrospector {
+        @Override
+        public JacksonInject.Value findInjectableValue(AnnotatedMember m) {
+            if (m.getRawType() == UnreadableBean.class) {
+                return JacksonInject.Value.forId("jjj");
+            }
+            return null;
+        }
+    }
+
+    static class TestInjector extends InjectableValues {
+        @Override
+        public Object findInjectableValue(Object valueId, DeserializationContext ctxt, BeanProperty forProperty, Object beanInstance) {
+            if (valueId == "jjj") {
+                UnreadableBean bean = new UnreadableBean();
+                bean.setValue(1);
+                return bean;
+            }
+            return null;
+        }
+    }
+
+    enum SimpleEnum { ONE, TWO }
+
+    static class UnreadableBean {
+        public SimpleEnum value;
+
+        public void setValue(SimpleEnum value) {
+            this.value = value;
+        }
+
+        public void setValue(Integer intValue) {
+            this.value = SimpleEnum.values()[intValue];
+        }
+
+        public SimpleEnum getValue() {
+            return value;
+        }
+    }
+
+    static class ReadableInjectedBean {
+        public ReadableInjectedBean(@JacksonInject(useInput = OptBoolean.FALSE) UnreadableBean injectBean) {
+            this.injectBean = injectBean;
+        }
+        @JsonProperty
+        String foo;
+        @JsonIgnore
+        UnreadableBean injectBean;
+    }
+
+    static class UnreadableInjectedBean {
+        public UnreadableInjectedBean(@JacksonInject UnreadableBean injectBean) {
+            this.injectBean = injectBean;
+        }
+        @JsonProperty
+        private String foo;
+        @JsonIgnore
+        private UnreadableBean injectBean;
+    }
+
+    @Test
+    public void testMergingIntrospectorsForInjection() throws Exception {
+        AnnotationIntrospector testIntrospector = new TestIntrospector();
+        ObjectMapper mapper = new JsonMapper();
+        mapper.setInjectableValues(new TestInjector());
+        mapper.setAnnotationIntrospectors(
+                new AnnotationIntrospectorPair(testIntrospector,
+                        mapper.getSerializationConfig().getAnnotationIntrospector()),
+                new AnnotationIntrospectorPair(testIntrospector,
+                        mapper.getDeserializationConfig().getAnnotationIntrospector())
+        );
+        ReadableInjectedBean bean = mapper.readValue("{\"foo\": \"bob\"}", ReadableInjectedBean.class);
+        assertEquals("bob", bean.foo);
+        assertEquals(SimpleEnum.TWO, bean.injectBean.value);
+
+        boolean successReadingUnreadableInjectedBean;
+        try {
+            /*UnreadableInjectedBean noBean =*/ mapper.readValue("{\"foo\": \"bob\"}", UnreadableInjectedBean.class);
+            successReadingUnreadableInjectedBean = true;
+        } catch (JsonMappingException e) {
+            successReadingUnreadableInjectedBean = false;
+            assertTrue(e.getMessage().contains("Conflicting setter definitions"));
+        }
+        assertFalse(successReadingUnreadableInjectedBean);
+    }
 }
