@@ -1,22 +1,24 @@
 package com.fasterxml.jackson.databind.introspect;
 
-import java.lang.annotation.ElementType;
-import java.lang.annotation.Retention;
-import java.lang.annotation.RetentionPolicy;
-import java.lang.annotation.Target;
+import java.lang.annotation.*;
 import java.math.BigDecimal;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
+
+import static org.junit.jupiter.api.Assertions.*;
 
 public class POJOPropertiesCollectorTest
-    extends BaseMapTest
+    extends DatabindTestUtil
 {
     static class Simple {
         public int value;
-        
+
         @JsonProperty("value")
         public void valueSetter(int v) { value = v; }
 
@@ -28,18 +30,18 @@ public class POJOPropertiesCollectorTest
     {
         @JsonDeserialize String[] values;
     }
-    
+
     static class SimpleGetterVisibility {
         public int getA() { return 0; }
         protected int getB() { return 1; }
         @SuppressWarnings("unused")
         private int getC() { return 2; }
     }
-    
+
     // Class for testing 'shared ignore'
     static class Empty {
         public int value;
-        
+
         public void setValue(int v) { value = v; }
 
         @JsonIgnore
@@ -49,7 +51,7 @@ public class POJOPropertiesCollectorTest
     static class IgnoredSetter {
         @JsonProperty
         public int value;
-        
+
         @JsonIgnore
         public void setValue(int v) { value = v; }
 
@@ -61,18 +63,18 @@ public class POJOPropertiesCollectorTest
         @JsonIgnore public void setB(int b) { }
         public int c;
     }
-    
+
     // Should find just one setter for "y", due to partial ignore
     static class IgnoredRenamedSetter {
         @JsonIgnore public void setY(int value) { }
         @JsonProperty("y") void foobar(int value) { }
     }
-    
+
     // should produce a single property, "x"
     static class RenamedProperties {
         @JsonProperty("x")
         public int value;
-        
+
         public void setValue(int v) { value = v; }
 
         public int getX() { return value; }
@@ -84,11 +86,11 @@ public class POJOPropertiesCollectorTest
         public int getValue() { return 1; }
         public void setValue(int x) { }
     }
-    
+
     // Testing that we can "merge" properties with renaming
     static class MergedProperties {
         public int x;
-        
+
         @JsonProperty("x")
         public void setFoobar(int v) { x = v; }
     }
@@ -99,7 +101,7 @@ public class POJOPropertiesCollectorTest
     {
         public int b;
         public int c;
-        
+
         public void setD(int value) { }
         public void setA(int value) { }
     }
@@ -124,9 +126,9 @@ public class POJOPropertiesCollectorTest
             location.add(new FoodOrgLocation());
         }
 
-        public List<FoodOrgLocation> getLocation() { return location; } 
+        public List<FoodOrgLocation> getLocation() { return location; }
     }
-    
+
     static class FoodOrgLocation
     {
         protected Long id;
@@ -138,7 +140,7 @@ public class POJOPropertiesCollectorTest
         }
 
         public FoodOrgLocation(final Location foodOrg) { }
-                
+
         public FoodOrgLocation(final Long id, final String name, final Location location) { }
 
         public Location getLocation() { return location; }
@@ -165,13 +167,13 @@ public class POJOPropertiesCollectorTest
     static class Issue744Bean
     {
         protected Map<String,Object> additionalProperties;
-        
+
         @JsonAnySetter
         public void addAdditionalProperty(String key, Object value) {
             if (additionalProperties == null) additionalProperties = new HashMap<String, Object>();
             additionalProperties.put(key,value);
         }
-        
+
         public void setAdditionalProperties(Map<String, Object> additionalProperties) {
             this.additionalProperties = additionalProperties;
         }
@@ -194,7 +196,7 @@ public class POJOPropertiesCollectorTest
         public String a;
 
         protected int b;
-        
+
         public String getA() { return a; }
 
         public void setA(String a) { this.a = a; }
@@ -237,8 +239,9 @@ public class POJOPropertiesCollectorTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = objectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testSimple()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -252,6 +255,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.hasField());
     }
 
+    @Test
     public void testSimpleFieldVisibility()
     {
         // false -> deserialization
@@ -266,6 +270,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.hasField());
     }
 
+    @Test
     public void testSimpleGetterVisibility()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -278,9 +283,10 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.hasGetter());
         assertFalse(prop.hasField());
     }
-    
+
     // Unit test for verifying that a single @JsonIgnore can remove the
     // whole property, unless explicit property marker exists
+    @Test
     public void testEmpty()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -292,6 +298,7 @@ public class POJOPropertiesCollectorTest
     // Unit test for verifying handling of 'partial' @JsonIgnore; that is,
     // if there is at least one explicit annotation to indicate property,
     // only parts that are ignored are, well, ignored
+    @Test
     public void testPartialIgnore()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -305,6 +312,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.hasField());
     }
 
+    @Test
     public void testSimpleRenamed()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -318,6 +326,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.hasField());
     }
 
+    @Test
     public void testSimpleRenamed2()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -331,6 +340,7 @@ public class POJOPropertiesCollectorTest
         assertFalse(prop.hasField());
     }
 
+    @Test
     public void testMergeWithRename()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -343,7 +353,8 @@ public class POJOPropertiesCollectorTest
         assertFalse(prop.hasGetter());
         assertTrue(prop.hasField());
     }
-    
+
+    @Test
     public void testSimpleIgnoreAndRename()
     {
         POJOPropertiesCollector coll = collector(MAPPER,
@@ -357,6 +368,7 @@ public class POJOPropertiesCollectorTest
         assertFalse(prop.hasField());
     }
 
+    @Test
     public void testGlobalVisibilityForGetters()
     {
         ObjectMapper m = jsonMapperBuilder()
@@ -368,6 +380,7 @@ public class POJOPropertiesCollectorTest
         assertEquals(0, props.size());
     }
 
+    @Test
     public void testCollectionOfIgnored()
     {
         POJOPropertiesCollector coll = collector(MAPPER, ImplicitIgnores.class, false);
@@ -381,6 +394,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(ign.contains("b"));
     }
 
+    @Test
     public void testSimpleOrderingForDeserialization()
     {
         POJOPropertiesCollector coll = collector(MAPPER, SortedProperties.class, false);
@@ -392,6 +406,7 @@ public class POJOPropertiesCollectorTest
         assertEquals("d", props.get(3).getName());
     }
 
+    @Test
     public void testSimpleWithType()
     {
         // first for serialization; should base choice on getter
@@ -413,9 +428,10 @@ public class POJOPropertiesCollectorTest
         assertEquals(String.class, m.getRawType());
     }
 
+    @Test
     public void testInnerClassWithAnnotationsInCreator() throws Exception
     {
-        BasicBeanDescription beanDesc;
+        BeanDescription beanDesc;
         // first with serialization
         beanDesc = MAPPER.getSerializationConfig().introspect(MAPPER.constructType(Issue701Bean.class));
         assertNotNull(beanDesc);
@@ -424,13 +440,14 @@ public class POJOPropertiesCollectorTest
         assertNotNull(beanDesc);
     }
 
+    @Test
     public void testUseAnnotationsFalse() throws Exception
     {
         // note: need a separate mapper, need to reconfigure
         ObjectMapper mapper = jsonMapperBuilder()
                 .configure(MapperFeature.USE_ANNOTATIONS, false)
                 .build();
-        BasicBeanDescription beanDesc = mapper.getSerializationConfig().introspect(mapper.constructType(Jackson703.class));
+        BeanDescription beanDesc = mapper.getSerializationConfig().introspect(mapper.constructType(Jackson703.class));
         assertNotNull(beanDesc);
 
         Jackson703 bean = new Jackson703();
@@ -438,6 +455,7 @@ public class POJOPropertiesCollectorTest
         assertNotNull(json);
     }
 
+    @Test
     public void testJackson744() throws Exception
     {
         BeanDescription beanDesc = MAPPER.getDeserializationConfig().introspect
@@ -450,6 +468,7 @@ public class POJOPropertiesCollectorTest
     }
 
     // [databind#269]: Support new @JsonPropertyDescription
+    @Test
     public void testPropertyDesc() throws Exception
     {
         // start via deser
@@ -461,6 +480,7 @@ public class POJOPropertiesCollectorTest
     }
 
     // [databind#438]: Support @JsonProperty.index
+    @Test
     public void testPropertyIndex() throws Exception
     {
         BeanDescription beanDesc = MAPPER.getDeserializationConfig().introspect(MAPPER.constructType(PropDescBean.class));
@@ -469,6 +489,7 @@ public class POJOPropertiesCollectorTest
         _verifyProperty(beanDesc, false, true, "13");
     }
 
+    @Test
     public void testDuplicateGetters() throws Exception
     {
         POJOPropertiesCollector coll = collector(MAPPER, DuplicateGetterBean.class, true);
@@ -480,6 +501,7 @@ public class POJOPropertiesCollectorTest
         assertTrue(prop.getGetter().hasAnnotation(B.class));
     }
 
+    @Test
     public void testDuplicateGettersCreator() throws Exception
     {
         POJOPropertiesCollector coll = collector(MAPPER, DuplicateGetterCreatorBean.class, true);
@@ -528,10 +550,11 @@ public class POJOPropertiesCollectorTest
             }
         }
     }
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Helper methods
-    /**********************************************************
+    /**********************************************************************
      */
 
     protected POJOPropertiesCollector collector(ObjectMapper m0,
@@ -541,9 +564,9 @@ public class POJOPropertiesCollectorTest
         // no real difference between serialization, deserialization, at least here
         if (forSerialization) {
             return bci.collectProperties(m0.getSerializationConfig(),
-                    m0.constructType(cls), null, true, "set");
+                    m0.constructType(cls), null, true);
         }
         return bci.collectProperties(m0.getDeserializationConfig(),
-                m0.constructType(cls), null, false, "set");
+                m0.constructType(cls), null, false);
     }
 }
