@@ -6,22 +6,27 @@ import java.util.*;
 
 import javax.xml.namespace.QName;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
+
 import com.fasterxml.jackson.core.JsonGenerator;
 import com.fasterxml.jackson.core.JsonParser;
-import com.fasterxml.jackson.core.JsonProcessingException;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.*;
+import com.fasterxml.jackson.databind.cfg.MapperConfig;
 import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
-import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
-import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
 import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
 import com.fasterxml.jackson.databind.jsontype.impl.StdTypeResolverBuilder;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 @SuppressWarnings("serial")
 public class TestJacksonAnnotationIntrospector
-    extends BaseMapTest
+    extends DatabindTestUtil
 {
     public static enum EnumExample {
         VALUE1;
@@ -98,7 +103,7 @@ public class TestJacksonAnnotationIntrospector
 
         @Override
         public void serialize(QName value, JsonGenerator jgen, SerializerProvider provider)
-                throws IOException, JsonProcessingException
+                throws IOException
         {
             jgen.writeString(value.toString());
         }
@@ -110,7 +115,7 @@ public class TestJacksonAnnotationIntrospector
         public QNameDeserializer() { super(QName.class); }
         @Override
         public QName deserialize(JsonParser jp, DeserializationContext ctxt)
-                throws IOException, JsonProcessingException
+                throws IOException
         {
             return QName.valueOf(jp.readValueAs(String.class));
         }
@@ -144,17 +149,28 @@ public class TestJacksonAnnotationIntrospector
             }
             return names;
         }
+
+        @Override
+        public String[] findEnumValues(MapperConfig<?> config, AnnotatedClass annotatedClass,
+                Enum<?>[] enumValues, String[] names) {
+            // kinda sorta wrong, but for testing's sake...
+            for (int i = 0, len = enumValues.length; i < len; ++i) {
+                names[i] = enumValues[i].name().toLowerCase();
+            }
+            return names;
+        }
     }
-    
+
     /*
     /**********************************************************
     /* Unit tests
     /**********************************************************
      */
-    
+
     /**
      * tests getting serializer/deserializer instances.
      */
+    @Test
     public void testSerializeDeserializeWithJaxbAnnotations() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -181,6 +197,7 @@ public class TestJacksonAnnotationIntrospector
         assertEquals(ex.enumProperty, readEx.enumProperty);
     }
 
+    @Test
     public void testJsonTypeResolver() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -193,12 +210,13 @@ public class TestJacksonAnnotationIntrospector
         assertSame(DummyBuilder.class, rb.getClass());
     }
 
+    @Test
     public void testEnumHandling() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
         mapper.setAnnotationIntrospector(new LcEnumIntrospector());
         assertEquals("\"value1\"", mapper.writeValueAsString(EnumExample.VALUE1));
-        EnumExample result = mapper.readValue(quote("value1"), EnumExample.class);
+        EnumExample result = mapper.readValue(q("value1"), EnumExample.class);
         assertEquals(EnumExample.VALUE1, result);
     }
 }
