@@ -2,17 +2,21 @@ package com.fasterxml.jackson.databind.jsontype;
 
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.Id;
 
 import com.fasterxml.jackson.core.type.TypeReference;
-
-import com.fasterxml.jackson.databind.BaseMapTest;
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 
-public class ExistingPropertyTest extends BaseMapTest
+import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
+import static org.junit.jupiter.api.Assertions.*;
+
+public class ExistingPropertyTest extends DatabindTestUtil
 {
     /**
      * Polymorphic base class - existing property as simple property on subclasses
@@ -21,7 +25,7 @@ public class ExistingPropertyTest extends BaseMapTest
             visible=true)
     @JsonSubTypes({
         @Type(value = Apple.class, name = "apple") ,
-        @Type(value = Orange.class, name = "orange") 
+        @Type(value = Orange.class, name = "orange")
     })
     static abstract class Fruit {
         public String name;
@@ -29,6 +33,7 @@ public class ExistingPropertyTest extends BaseMapTest
     }
 
     @JsonTypeName("apple")
+    @JsonPropertyOrder({ "name", "seedCount", "type" })
     static class Apple extends Fruit
     {
         public int seedCount;
@@ -43,6 +48,7 @@ public class ExistingPropertyTest extends BaseMapTest
     }
 
     @JsonTypeName("orange")
+    @JsonPropertyOrder({ "name", "color", "type" })
     static class Orange extends Fruit
     {
         public String color;
@@ -68,13 +74,13 @@ public class ExistingPropertyTest extends BaseMapTest
 	@JsonTypeInfo(use = Id.NAME, include = As.EXISTING_PROPERTY, property = "type")
 	@JsonSubTypes({
 		@Type(value = Dog.class, name = "doggie") ,
-		@Type(value = Cat.class, name = "kitty") 
+		@Type(value = Cat.class, name = "kitty")
 		})
 	static abstract class Animal {
         public String name;
-        
+
         protected Animal(String n)  { name = n; }
-        
+
         public abstract String getType();
     }
 
@@ -82,34 +88,34 @@ public class ExistingPropertyTest extends BaseMapTest
     static class Dog extends Animal
     {
         public int boneCount;
-        
+
         private Dog() { super(null); }
         public Dog(String name, int b) {
             super(name);
             boneCount = b;
         }
-        
+
  		@Override
 		public String getType() {
         	return "doggie";
-        }        
+        }
     }
 
     @JsonTypeName("kitty")
     static class Cat extends Animal
     {
         public String furColor;
-        
+
         private Cat() { super(null); }
         public Cat(String name, String c) {
             super(name);
             furColor = c;
         }
-        
+
 		@Override
 		public String getType() {
         	return "kitty";
-        }        
+        }
     }
 
     static class AnimalWrapper {
@@ -124,10 +130,10 @@ public class ExistingPropertyTest extends BaseMapTest
 	@JsonTypeInfo(use = Id.NAME, include = As.EXISTING_PROPERTY, property = "type")
 	@JsonSubTypes({
 		@Type(value = Accord.class, name = "accord") ,
-		@Type(value = Camry.class, name = "camry") 
+		@Type(value = Camry.class, name = "camry")
 		})
 	static abstract class Car {
-        public String name;        
+        public String name;
         protected Car(String n)  { name = n; }
     }
 
@@ -135,32 +141,32 @@ public class ExistingPropertyTest extends BaseMapTest
     static class Accord extends Car
     {
         public int speakerCount;
-        
+
         private Accord() { super(null); }
         public Accord(String name, int b) {
             super(name);
             speakerCount = b;
         }
-        
+
 		public String getType() {
         	return "accord";
-        }        
+        }
     }
 
     @JsonTypeName("camry")
     static class Camry extends Car
     {
         public String exteriorColor;
-        
+
         private Camry() { super(null); }
         public Camry(String name, String c) {
             super(name);
             exteriorColor = c;
         }
-        
+
 		public String getType() {
         	return "camry";
-        }        
+        }
     }
 
     static class CarWrapper {
@@ -189,15 +195,79 @@ public class ExistingPropertyTest extends BaseMapTest
 
     static class Bean1635Default extends Bean1635 { }
 
+    // [databind#2785]
+    @JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.EXISTING_PROPERTY,
+            property = "packetType")
+    public interface Base2785  {
+    }
+    @JsonTypeName("myType")
+    static class Impl2785 implements Base2785 {
+    }
+
+    // [databind#3251]: Double vs BigDecimal
+    @JsonTypeInfo(
+        use = JsonTypeInfo.Id.NAME,
+        property = "type_alias"
+    )
+    static class GenericWrapperWithNew3251<T> {
+        private final T value;
+
+        @JsonCreator
+        public GenericWrapperWithNew3251(@JsonProperty("value") T value) {
+            this.value = value;
+        }
+
+        public T getValue() {
+            return value;
+        }
+    }
+
+    @JsonTypeInfo(
+        use = JsonTypeInfo.Id.NAME,
+        include = JsonTypeInfo.As.EXISTING_PROPERTY,
+        property = "fieldType",
+        visible = true,
+        defaultImpl = GenericWrapperWithExisting3251.class
+    )
+    static class GenericWrapperWithExisting3251<T> {
+        public String fieldType;
+        private final T value;
+
+        @JsonCreator
+        public GenericWrapperWithExisting3251(@JsonProperty("value") T value) {
+            this.value = value;
+        }
+
+        public T getValue() {
+            return value;
+        }
+    }
+
+    // [databind#3271]
+    @JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.EXISTING_PROPERTY,
+            visible = true, property = "type", defaultImpl = DefaultShape3271.class)
+    @JsonSubTypes({@JsonSubTypes.Type(value = Square3271.class, name = "square")})
+    static abstract class Shape3271 {
+        public String type;
+
+        public String getType() { return this.type; }
+
+        public void setType(String type) { this.type = type; }
+    }
+
+    static class Square3271 extends Shape3271 {}
+
+    static class DefaultShape3271 extends Shape3271 {}
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Mock data
-    /**********************************************************
+    /**********************************************************************
      */
 
     private static final Orange mandarin = new Orange("Mandarin Orange", "orange");
-    private static final String mandarinJson = "{\"name\":\"Mandarin Orange\",\"color\":\"orange\",\"type\":\"orange\"}";	
-    private static final Apple pinguo = new Apple("Apple-A-Day", 16);    
+    private static final String mandarinJson = "{\"name\":\"Mandarin Orange\",\"color\":\"orange\",\"type\":\"orange\"}";
+    private static final Apple pinguo = new Apple("Apple-A-Day", 16);
     private static final String pinguoJson = "{\"name\":\"Apple-A-Day\",\"seedCount\":16,\"type\":\"apple\"}";
     private static final FruitWrapper pinguoWrapper = new FruitWrapper(pinguo);
     private static final String pinguoWrapperJson = "{\"fruit\":" + pinguoJson + "}";
@@ -205,7 +275,7 @@ public class ExistingPropertyTest extends BaseMapTest
     private static final String fruitListJson = "[" + pinguoJson + "," + mandarinJson + "]";
 
     private static final Cat beelzebub = new Cat("Beelzebub", "tabby");
-    private static final String beelzebubJson = "{\"name\":\"Beelzebub\",\"furColor\":\"tabby\",\"type\":\"kitty\"}";	
+    private static final String beelzebubJson = "{\"name\":\"Beelzebub\",\"furColor\":\"tabby\",\"type\":\"kitty\"}";
     private static final Dog rover = new Dog("Rover", 42);
     private static final String roverJson = "{\"name\":\"Rover\",\"boneCount\":42,\"type\":\"doggie\"}";
     private static final AnimalWrapper beelzebubWrapper = new AnimalWrapper(beelzebub);
@@ -214,7 +284,7 @@ public class ExistingPropertyTest extends BaseMapTest
     private static final String animalListJson = "[" + beelzebubJson + "," + roverJson + "]";
 
     private static final Camry camry = new Camry("Sweet Ride", "candy-apple-red");
-    private static final String camryJson = "{\"name\":\"Sweet Ride\",\"exteriorColor\":\"candy-apple-red\",\"type\":\"camry\"}";	
+    private static final String camryJson = "{\"name\":\"Sweet Ride\",\"exteriorColor\":\"candy-apple-red\",\"type\":\"camry\"}";
     private static final Accord accord = new Accord("Road Rage", 6);
     private static final String accordJson = "{\"name\":\"Road Rage\",\"speakerCount\":6,\"type\":\"accord\"}";
     private static final CarWrapper camryWrapper = new CarWrapper(camry);
@@ -223,16 +293,17 @@ public class ExistingPropertyTest extends BaseMapTest
     private static final String carListJson = "[" + camryJson + "," + accordJson + "]";
 
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Test methods
-    /**********************************************************
+    /**********************************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     /**
      * Fruits - serialization tests for simple property on sub-classes
      */
+    @Test
     public void testExistingPropertySerializationFruits() throws Exception
     {
         Map<String,Object> result = writeAndMap(MAPPER, pinguo);
@@ -240,13 +311,13 @@ public class ExistingPropertyTest extends BaseMapTest
         assertEquals(pinguo.name, result.get("name"));
         assertEquals(pinguo.seedCount, result.get("seedCount"));
         assertEquals(pinguo.type, result.get("type"));
-        
+
         result = writeAndMap(MAPPER, mandarin);
         assertEquals(3, result.size());
         assertEquals(mandarin.name, result.get("name"));
         assertEquals(mandarin.color, result.get("color"));
         assertEquals(mandarin.type, result.get("type"));
-        
+
         String pinguoSerialized = MAPPER.writeValueAsString(pinguo);
         assertEquals(pinguoSerialized, pinguoJson);
 
@@ -263,6 +334,7 @@ public class ExistingPropertyTest extends BaseMapTest
     /**
      * Fruits - deserialization tests for simple property on sub-classes
      */
+    @Test
     public void testSimpleClassAsExistingPropertyDeserializationFruits() throws Exception
     {
         Fruit pinguoDeserialized = MAPPER.readValue(pinguoJson, Fruit.class);
@@ -286,7 +358,7 @@ public class ExistingPropertyTest extends BaseMapTest
         assertEquals("apple", ((Apple) fruits[0]).type);
         assertEquals(Orange.class, fruits[1].getClass());
         assertEquals("orange", ((Orange) fruits[1]).type);
-        
+
         List<Fruit> f2 = MAPPER.readValue(fruitListJson,
                 new TypeReference<List<Fruit>>() { });
         assertNotNull(f2);
@@ -298,6 +370,7 @@ public class ExistingPropertyTest extends BaseMapTest
     /**
      * Animals - serialization tests for abstract method in base class
      */
+    @Test
     public void testExistingPropertySerializationAnimals() throws Exception
     {
         Map<String,Object> result = writeAndMap(MAPPER, beelzebub);
@@ -311,13 +384,13 @@ public class ExistingPropertyTest extends BaseMapTest
         assertEquals(rover.name, result.get("name"));
         assertEquals(rover.boneCount, result.get("boneCount"));
         assertEquals(rover.getType(), result.get("type"));
-        
+
         String beelzebubSerialized = MAPPER.writeValueAsString(beelzebub);
         assertEquals(beelzebubSerialized, beelzebubJson);
-        
+
         String roverSerialized = MAPPER.writeValueAsString(rover);
         assertEquals(roverSerialized, roverJson);
-        
+
         String animalWrapperSerialized = MAPPER.writeValueAsString(beelzebubWrapper);
         assertEquals(animalWrapperSerialized, beelzebubWrapperJson);
 
@@ -328,6 +401,7 @@ public class ExistingPropertyTest extends BaseMapTest
     /**
      * Animals - deserialization tests for abstract method in base class
      */
+    @Test
     public void testSimpleClassAsExistingPropertyDeserializationAnimals() throws Exception
     {
         Animal beelzebubDeserialized = MAPPER.readValue(beelzebubJson, Animal.class);
@@ -344,7 +418,7 @@ public class ExistingPropertyTest extends BaseMapTest
         assertEquals(beelzebub.name, beelzebubExtracted.name);
         assertEquals(beelzebub.furColor, ((Cat) beelzebubExtracted).furColor);
         assertEquals(beelzebub.getType(), beelzebubExtracted.getType());
-    	
+
         @SuppressWarnings("unchecked")
         List<Animal> animalListDeserialized = MAPPER.readValue(animalListJson, List.class);
         assertNotNull(animalListDeserialized);
@@ -360,6 +434,7 @@ public class ExistingPropertyTest extends BaseMapTest
     /**
      * Cars - serialization tests for no abstract method or type variable in base class
      */
+    @Test
     public void testExistingPropertySerializationCars() throws Exception
     {
         Map<String,Object> result = writeAndMap(MAPPER, camry);
@@ -379,7 +454,7 @@ public class ExistingPropertyTest extends BaseMapTest
 
         String accordSerialized = MAPPER.writeValueAsString(accord);
         assertEquals(accordSerialized, accordJson);
-        
+
         String carWrapperSerialized = MAPPER.writeValueAsString(camryWrapper);
         assertEquals(carWrapperSerialized, camryWrapperJson);
 
@@ -390,6 +465,7 @@ public class ExistingPropertyTest extends BaseMapTest
     /**
      * Cars - deserialization tests for no abstract method or type variable in base class
      */
+    @Test
     public void testSimpleClassAsExistingPropertyDeserializationCars() throws Exception
     {
         Car camryDeserialized = MAPPER.readValue(camryJson, Camry.class);
@@ -421,9 +497,10 @@ public class ExistingPropertyTest extends BaseMapTest
     }
 
     // for [databind#1635]: simple usage
+    @Test
     public void testExistingEnumTypeId() throws Exception
     {
-        Bean1635 result = MAPPER.readValue(aposToQuotes("{'value':3, 'type':'A'}"),
+        Bean1635 result = MAPPER.readValue(a2q("{'value':3, 'type':'A'}"),
                 Bean1635.class);
         assertEquals(Bean1635A.class, result.getClass());
         Bean1635A bean = (Bean1635A) result;
@@ -433,11 +510,68 @@ public class ExistingPropertyTest extends BaseMapTest
 
     // for [databind#1635]: verify that `defaultImpl` does not block assignment of
     // type id
+    @Test
     public void testExistingEnumTypeIdViaDefault() throws Exception
     {
-        Bean1635 result = MAPPER.readValue(aposToQuotes("{'type':'C'}"),
+        Bean1635 result = MAPPER.readValue(a2q("{'type':'C'}"),
                 Bean1635.class);
         assertEquals(Bean1635Default.class, result.getClass());
         assertEquals(ABC.C, result.type);
     }
+
+    // [databind#2785]
+    @Test
+    public void testCopyOfSubtypeResolver2785() throws Exception {
+        ObjectMapper objectMapper = new ObjectMapper().copy();
+        objectMapper.registerSubtypes(Impl2785.class);
+        Object result = objectMapper.readValue("{ \"packetType\": \"myType\" }", Base2785.class);
+        assertNotNull(result);
+    }
+
+    // [databind#3271]: verify that `null` token does not become "null" String
+    @Test
+    public void testDeserializationWithValidType() throws Exception {
+        Shape3271 deserShape = MAPPER.readValue("{\"type\":\"square\"}", Shape3271.class);
+        assertEquals("square", deserShape.getType());
+    }
+
+    @Test
+    public void testDeserializationWithInvalidType() throws Exception {
+        Shape3271 deserShape = MAPPER.readValue("{\"type\":\"invalid\"}", Shape3271.class);
+        assertEquals("invalid", deserShape.getType());
+    }
+
+    @Test
+    public void testDeserializationNull() throws Exception {
+        Shape3271 deserShape = MAPPER.readValue("{\"type\":null}", Shape3271.class);
+        assertNull(deserShape.getType()); // error: "expected null, but was:<null>"
+    }
+
+    // [databind#3251]: Double vs BigDecimal
+    @Test
+    public void test3251WithNewProperty() throws Exception
+    {
+        GenericWrapperWithNew3251<?> wrapper = new GenericWrapperWithNew3251<>(123.5);
+
+        String json = MAPPER.writeValueAsString(wrapper);
+        GenericWrapperWithNew3251<?> actualWrapper = MAPPER.readValue(json, GenericWrapperWithNew3251.class);
+
+        assertThat(actualWrapper).satisfies(it -> assertThat(it.getValue()).isEqualTo(123.5));
+        assertThat(actualWrapper.getValue()).isInstanceOf(Double.class);
+        assertThat(json).contains("\"value\":123.5");
+    }
+
+    @Test
+    public void test3251WithExistingProperty() throws Exception
+    {
+        GenericWrapperWithExisting3251<?> wrapper = new GenericWrapperWithExisting3251<>(123.5);
+
+        String json = MAPPER.writeValueAsString(wrapper);
+        GenericWrapperWithExisting3251<?> actualWrapper = MAPPER.readValue(json, GenericWrapperWithExisting3251.class);
+
+        assertThat(actualWrapper).satisfies(it -> assertThat(it.getValue()).isEqualTo(123.5));
+        assertThat(actualWrapper.getValue()).isInstanceOf(Double.class);
+        assertThat(json).contains("\"value\":123.5");
+    }
+
 }
