@@ -3,19 +3,25 @@ package com.fasterxml.jackson.databind.jsontype;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.Id;
 
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.ObjectMapper.DefaultTyping;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+
 /**
  * Unit tests for checking how combination of interfaces, implementation
  * classes are handled, with respect to type names.
  */
-public class TestAbstractTypeNames  extends BaseMapTest
+public class TestAbstractTypeNames  extends DatabindTestUtil
 {
     @JsonTypeName("Employee")
     public interface Employee extends User {
@@ -96,6 +102,7 @@ public class TestAbstractTypeNames  extends BaseMapTest
     /**********************************************************
      */
 
+    @Test
     public void testEmptyCollection() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -113,7 +120,7 @@ public class TestAbstractTypeNames  extends BaseMapTest
         mapper = new ObjectMapper();
         mapper.registerSubtypes(DefaultEmployee.class);
         mapper.registerSubtypes(DefaultUser.class);
-        
+
         User result = mapper.readValue(json, User.class);
         assertNotNull(result);
         assertEquals(DefaultEmployee.class, result.getClass());
@@ -123,8 +130,9 @@ public class TestAbstractTypeNames  extends BaseMapTest
         assertEquals(DefaultUser.class, friends.get(0).getClass());
         assertEquals(DefaultEmployee.class, friends.get(1).getClass());
     }
-    
+
     // [JACKSON-584]: change anonymous non-static inner type into static type:
+    @Test
     public void testInnerClassWithType() throws Exception
     {
         ObjectMapper mapper = jsonMapperBuilder()
