@@ -3,15 +3,20 @@ package com.fasterxml.jackson.databind.jsontype;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 
 import com.fasterxml.jackson.core.type.TypeReference;
 
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 public class TestGenericListSerialization
-    extends BaseMapTest
+    extends DatabindTestUtil
 {
     // [JACKSON-356]
     public static class JSONResponse<T> {
@@ -25,7 +30,7 @@ public class TestGenericListSerialization
         public void setResult(T result) {
             this.result = result;
         }
-    } 
+    }
 
     @JsonTypeInfo(use=JsonTypeInfo.Id.CLASS, include=JsonTypeInfo.As.PROPERTY, property="@class")
     public static class Parent {
@@ -46,21 +51,22 @@ public class TestGenericListSerialization
     /**********************************************************
      */
 
+    @Test
     public void testSubTypesFor356() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
-        
         JSONResponse<List<Parent>> input = new JSONResponse<List<Parent>>();
 
         List<Parent> embedded = new ArrayList<Parent>();
         embedded.add(new Child1());
         embedded.add(new Child2());
         input.setResult(embedded);
-        mapper.configure(MapperFeature.USE_STATIC_TYPING, true);
+        ObjectMapper mapper = jsonMapperBuilder()
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
 
         JavaType rootType = TypeFactory.defaultInstance().constructType(new TypeReference<JSONResponse<List<Parent>>>() { });
         byte[] json = mapper.writerFor(rootType).writeValueAsBytes(input);
-        
+
         JSONResponse<List<Parent>> out = mapper.readValue(json, 0, json.length, rootType);
 
         List<Parent> deserializedContent = out.getResult();
@@ -77,5 +83,5 @@ public class TestGenericListSerialization
         assertEquals("CHILD1", ((Child1) deserializedContent.get(0)).childContent1);
         assertEquals("CHILD2", ((Child2) deserializedContent.get(1)).childContent2);
     }
-    
+
 }
