@@ -1,22 +1,24 @@
 package com.fasterxml.jackson.databind.jsontype;
 
-import com.fasterxml.jackson.core.Version;
-
 import java.util.*;
 
-import com.fasterxml.jackson.annotation.JsonSubTypes;
-import com.fasterxml.jackson.annotation.JsonTypeInfo;
-import com.fasterxml.jackson.annotation.JsonTypeName;
+import org.junit.jupiter.api.Test;
+
+import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
-import com.fasterxml.jackson.databind.JsonMappingException;
+
+import com.fasterxml.jackson.core.Version;
+
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.SerializationFeature;
 import com.fasterxml.jackson.databind.exc.InvalidTypeIdException;
-import com.fasterxml.jackson.databind.jsontype.NamedType;
 import com.fasterxml.jackson.databind.module.SimpleModule;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 
-public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+public class TestSubtypes extends DatabindTestUtil
 {
     @JsonTypeInfo(use=JsonTypeInfo.Id.NAME)
     static abstract class SuperType {
@@ -38,7 +40,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
     // "Empty" bean
     @JsonTypeInfo(use=JsonTypeInfo.Id.NAME)
     static abstract class BaseBean { }
-    
+
     static class EmptyBean extends BaseBean { }
 
     static class EmptyNonFinal { }
@@ -49,7 +51,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
     {
         @JsonTypeInfo(use=JsonTypeInfo.Id.NAME)
         public SuperType value;
-        
+
         public PropertyBean() { this(null); }
         public PropertyBean(SuperType v) { value = v; }
     }
@@ -70,9 +72,33 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         public int a;
     }
 
+    static class Sub extends SuperTypeWithoutDefault {
+        public int a;
+
+        public Sub(){}
+        public Sub(int a) {
+            this.a = a;
+        }
+    }
+
+    static class POJOWrapper {
+        @JsonProperty
+        Sub sub1;
+        @JsonProperty
+        Sub sub2;
+
+        public POJOWrapper(){}
+        public POJOWrapper(Sub sub1, Sub sub2) {
+            this.sub1 = sub1;
+            this.sub2 = sub2;
+        }
+    }
+
     @JsonTypeInfo(use=JsonTypeInfo.Id.NAME, include=As.PROPERTY, property="type")
     @JsonSubTypes({ @JsonSubTypes.Type(ImplX.class),
-          @JsonSubTypes.Type(ImplY.class) })
+        @JsonSubTypes.Type(ImplY.class),
+        @JsonSubTypes.Type(ImplAbs.class)
+    })
     static abstract class BaseX { }
 
     @JsonTypeName("x")
@@ -88,6 +114,11 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         public int y;
     }
 
+    // for [databind#919] testing
+    @JsonTypeName("abs")
+    abstract static class ImplAbs extends BaseX {
+    }
+
     // [databind#663]
     static class AtomicWrapper {
         public BaseX value;
@@ -118,7 +149,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         public Issue1125Wrapper() { }
         public Issue1125Wrapper(Base1125 v) { value = v; }
     }
-    
+
     @JsonTypeInfo(use=JsonTypeInfo.Id.NAME, defaultImpl=Default1125.class)
     @JsonSubTypes({ @JsonSubTypes.Type(Interm1125.class) })
     static class Base1125 {
@@ -170,6 +201,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
 
     private final ObjectMapper MAPPER = new ObjectMapper();
 
+    @Test
     public void testPropertyWithSubtypes() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -181,6 +213,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
     }
 
     // also works via modules
+    @Test
     public void testSubtypesViaModule() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -204,7 +237,8 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         result = mapper.readValue(json, PropertyBean.class);
         assertSame(SubC.class, result.value.getClass());
     }
-    
+
+    @Test
     public void testSerialization() throws Exception
     {
         // serialization can detect type name ok without anything extra:
@@ -217,9 +251,21 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         assertEquals("{\"@type\":\"typeB\",\"b\":1}", mapper.writeValueAsString(bean));
 
         // and default name ought to be simple class name; with context
-        assertEquals("{\"@type\":\"TestSubtypes$SubD\",\"d\":0}", mapper.writeValueAsString(new SubD()));  
+        assertEquals("{\"@type\":\"TestSubtypes$SubD\",\"d\":0}", mapper.writeValueAsString(new SubD()));
     }
 
+    @Test
+    public void testSerializationWithDuplicateRegisteredSubtypes() throws Exception {
+        ObjectMapper mapper = new ObjectMapper();
+        mapper.registerSubtypes(new NamedType(Sub.class, "sub1"));
+        mapper.registerSubtypes(new NamedType(Sub.class, "sub2"));
+
+        // the first registered type name is used for serialization
+        Sub sub = new Sub(15);
+        assertEquals("{\"#type\":\"sub1\",\"a\":15}", mapper.writeValueAsString(sub));
+    }
+
+    @Test
     public void testDeserializationNonNamed() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -231,6 +277,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         assertEquals(1, ((SubC) bean).c);
     }
 
+    @Test
     public void testDeserializatioNamed() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -247,7 +294,31 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         assertEquals(-4, ((SubD) bean).d);
     }
 
+    @Test
+    public void testDeserializationWithDuplicateRegisteredSubtypes()
+        throws Exception {
+        ObjectMapper mapper = new ObjectMapper();
+
+        // We can register the same class with different names
+        mapper.registerSubtypes(new NamedType(Sub.class, "sub1"));
+        mapper.registerSubtypes(new NamedType(Sub.class, "sub2"));
+
+        // fields of a POJO will be deserialized correctly according to their field name
+        POJOWrapper pojoWrapper = mapper.readValue("{\"sub1\":{\"#type\":\"sub1\",\"a\":10},\"sub2\":{\"#type\":\"sub2\",\"a\":50}}", POJOWrapper.class);
+        assertEquals(10, pojoWrapper.sub1.a);
+        assertEquals(50, pojoWrapper.sub2.a);
+
+        // Instances of the same object can be deserialized with multiple names
+        SuperTypeWithoutDefault sub1 = mapper.readValue("{\"#type\":\"sub1\", \"a\":20}", SuperTypeWithoutDefault.class);
+        assertSame(Sub.class, sub1.getClass());
+        assertEquals(20, ((Sub) sub1).a);
+        SuperTypeWithoutDefault sub2 = mapper.readValue("{\"#type\":\"sub2\", \"a\":30}", SuperTypeWithoutDefault.class);
+        assertSame(Sub.class, sub2.getClass());
+        assertEquals(30, ((Sub) sub2).a);
+    }
+
     // Trying to reproduce [JACKSON-366]
+    @Test
     public void testEmptyBean() throws Exception
     {
         // First, with annotations
@@ -270,6 +341,7 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         assertEquals("[\"com.fasterxml.jackson.databind.jsontype.TestSubtypes$EmptyNonFinal\",{}]", json);
     }
 
+    @Test
     public void testDefaultImpl() throws Exception
     {
         // first, test with no type information
@@ -292,10 +364,11 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
     }
 
     // [JACKSON-505]: ok to also default to mapping there might be for base type
+    @Test
     public void testDefaultImplViaModule() throws Exception
     {
         final String JSON = "{\"a\":123}";
-        
+
         // first: without registration etc, epic fail:
         try {
             MAPPER.readValue(JSON, SuperTypeWithoutDefault.class);
@@ -317,19 +390,21 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         bean = mapper.readValue("{\"#type\":\"foobar\"}", SuperTypeWithoutDefault.class);
         assertEquals(DefaultImpl505.class, bean.getClass());
         assertEquals(0, ((DefaultImpl505) bean).a);
-    
     }
 
+    @Test
     public void testErrorMessage() throws Exception {
         ObjectMapper mapper = new ObjectMapper();
         try {
             mapper.readValue("{ \"type\": \"z\"}", BaseX.class);
             fail("Should have failed");
-        } catch (JsonMappingException e) {
-            verifyException(e, "known type ids =");
+        } catch (InvalidTypeIdException e) {
+            verifyException(e, "Could not resolve type id 'z' as a subtype of");
+            verifyException(e, "known type ids = [x, y]");
         }
     }
 
+    @Test
     public void testViaAtomic() throws Exception {
         AtomicWrapper input = new AtomicWrapper(3);
         String json = MAPPER.writeValueAsString(input);
@@ -342,10 +417,11 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
 
     // Test to verify that base/impl restriction is applied to polymorphic handling
     // even if class name is used as the id
+    @Test
     public void testSubclassLimits() throws Exception
     {
         try {
-            MAPPER.readValue(aposToQuotes("{'value':['"
+            MAPPER.readValue(a2q("{'value':['"
                     +TheBomb.class.getName()+"',{'a':13}] }"), DateWrapper.class);
             fail("Should not pass");
         } catch (InvalidTypeIdException e) {
@@ -358,10 +434,11 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
 
     // [databind#1125]: properties from base class too
 
+    @Test
     public void testIssue1125NonDefault() throws Exception
     {
         String json = MAPPER.writeValueAsString(new Issue1125Wrapper(new Impl1125(1, 2, 3)));
-        
+
         Issue1125Wrapper result = MAPPER.readValue(json, Issue1125Wrapper.class);
         assertNotNull(result.value);
         assertEquals(Impl1125.class, result.value.getClass());
@@ -371,9 +448,10 @@ public class TestSubtypes extends com.fasterxml.jackson.databind.BaseMapTest
         assertEquals(3, impl.c);
     }
 
+    @Test
     public void testIssue1125WithDefault() throws Exception
     {
-        Issue1125Wrapper result = MAPPER.readValue(aposToQuotes("{'value':{'a':3,'def':9,'b':5}}"),
+        Issue1125Wrapper result = MAPPER.readValue(a2q("{'value':{'a':3,'def':9,'b':5}}"),
         		Issue1125Wrapper.class);
         assertNotNull(result.value);
         assertEquals(Default1125.class, result.value.getClass());
