@@ -3,6 +3,8 @@ package com.fasterxml.jackson.databind.jsontype;
 import java.io.IOException;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
@@ -11,15 +13,18 @@ import com.fasterxml.jackson.annotation.JsonTypeInfo.Id;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.ser.BeanSerializerFactory;
 import com.fasterxml.jackson.databind.ser.ResolvableSerializer;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 
-public class TestWithGenerics extends BaseMapTest
+import static org.junit.jupiter.api.Assertions.*;
+
+public class TestWithGenerics extends DatabindTestUtil
 {
     @JsonTypeInfo(use = Id.NAME, include = As.PROPERTY, property = "object-type")
     @JsonSubTypes( { @Type(value = Dog.class, name = "doggy") })
     static abstract class Animal {
         public String name;
-    }    
+    }
 
     static class Dog extends Animal {
         public int boneCount;
@@ -44,13 +49,13 @@ public class TestWithGenerics extends BaseMapTest
 
         public ContainerWithField(T a) { animal = a; }
     }
-    
+
     static class WrappedContainerWithField {
         public ContainerWithField<?> animalContainer;
     }
 
 	// Beans for [JACKSON-387], [JACKSON-430]
-    
+
     @JsonTypeInfo(use=JsonTypeInfo.Id.CLASS, include=JsonTypeInfo.As.PROPERTY, property="@classAttr1")
     static class MyClass {
         public List<MyParam<?>> params = new ArrayList<MyParam<?>>();
@@ -67,29 +72,29 @@ public class TestWithGenerics extends BaseMapTest
     static class SomeObject {
         public String someValue = UUID.randomUUID().toString();
     }
-    
+
     // Beans for [JACKSON-430]
-    
+
     static class CustomJsonSerializer extends JsonSerializer<Object>
         implements ResolvableSerializer
     {
         private final JsonSerializer<Object> beanSerializer;
-    
+
         public CustomJsonSerializer( JsonSerializer<Object> beanSerializer ) { this.beanSerializer = beanSerializer; }
-    
+
         @Override
         public void serialize( Object value, JsonGenerator jgen, SerializerProvider provider )
-            throws IOException, JsonProcessingException
+            throws IOException
         {
             beanSerializer.serialize( value, jgen, provider );
         }
-    
+
         @Override
         public Class<Object> handledType() { return beanSerializer.handledType(); }
-    
+
         @Override
         public void serializeWithType( Object value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer )
-            throws IOException, JsonProcessingException
+            throws IOException
         {
             beanSerializer.serializeWithType( value, jgen, provider, typeSer );
         }
@@ -102,7 +107,7 @@ public class TestWithGenerics extends BaseMapTest
             }
         }
     }
-    
+
     @SuppressWarnings("serial")
     protected static class CustomJsonSerializerFactory extends BeanSerializerFactory
     {
@@ -112,7 +117,7 @@ public class TestWithGenerics extends BaseMapTest
         protected JsonSerializer<Object> constructBeanOrAddOnSerializer(SerializerProvider prov,
                 JavaType type, BeanDescription beanDesc, boolean staticTyping)
             throws JsonMappingException
-        {                
+        {
             return new CustomJsonSerializer(super.constructBeanOrAddOnSerializer(prov, type, beanDesc, staticTyping) );
         }
     }
@@ -120,21 +125,22 @@ public class TestWithGenerics extends BaseMapTest
     // [databind#543]
     static class ContainerWithTwoAnimals<U extends Animal,V extends Animal> extends ContainerWithField<U> {
          public V otherAnimal;
-        
+
          public ContainerWithTwoAnimals(U a1, V a2) {
               super(a1);
               otherAnimal = a2;
          }
     }
-    
+
     /*
     /**********************************************************
     /* Unit tests
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = objectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
+    @Test
     public void testWrapperWithGetter() throws Exception
     {
         Dog dog = new Dog("Fluffy", 3);
@@ -144,6 +150,7 @@ public class TestWithGenerics extends BaseMapTest
         }
     }
 
+    @Test
     public void testWrapperWithField() throws Exception
     {
         Dog dog = new Dog("Fluffy", 3);
@@ -152,7 +159,8 @@ public class TestWithGenerics extends BaseMapTest
             fail("polymorphic type not kept, result == "+json+"; should contain 'object-type':'...'");
         }
     }
-    
+
+    @Test
     public void testWrapperWithExplicitType() throws Exception
     {
         Dog dog = new Dog("Fluffy", 3);
@@ -163,7 +171,8 @@ public class TestWithGenerics extends BaseMapTest
             fail("polymorphic type not kept, result == "+json+"; should contain 'object-type':'...'");
         }
     }
-    
+
+    @Test
     public void testJackson387() throws Exception
     {
         ObjectMapper om = new ObjectMapper();
@@ -178,7 +187,7 @@ public class TestWithGenerics extends BaseMapTest
         MyParam<String> moc2 = new MyParam<String>("valueX");
 
         SomeObject so = new SomeObject();
-        so.someValue = "xxxxxx"; 
+        so.someValue = "xxxxxx";
         MyParam<SomeObject> moc3 = new MyParam<SomeObject>(so);
 
         List<SomeObject> colist = new ArrayList<SomeObject>();
@@ -193,13 +202,14 @@ public class TestWithGenerics extends BaseMapTest
         mc.params.add( moc4 );
 
         String json = om.writeValueAsString( mc );
-        
+
         MyClass mc2 = om.readValue(json, MyClass.class );
         assertNotNull(mc2);
         assertNotNull(mc2.params);
         assertEquals(4, mc2.params.size());
     }
 
+    @Test
     public void testJackson430() throws Exception
     {
         ObjectMapper om = new ObjectMapper();
@@ -210,7 +220,7 @@ public class TestWithGenerics extends BaseMapTest
 
         String str = om.writeValueAsString( mc );
 //        System.out.println( str );
-        
+
         MyClass mc2 = om.readValue( str, MyClass.class );
         assertNotNull(mc2);
         assertNotNull(mc2.params);
@@ -218,6 +228,7 @@ public class TestWithGenerics extends BaseMapTest
     }
 
     // [Issue#543]
+    @Test
     public void testValueWithMoreGenericParameters() throws Exception
     {
         WrappedContainerWithField wrappedContainerWithField = new WrappedContainerWithField();
