@@ -2,19 +2,28 @@ package com.fasterxml.jackson.databind.jsontype.deftyping;
 
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
 
 import com.fasterxml.jackson.core.*;
+
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
+
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.MapperConfig;
+import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
 import com.fasterxml.jackson.databind.json.JsonMapper;
+import com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 import com.fasterxml.jackson.databind.util.TokenBuffer;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 public class TestDefaultForObject
-    extends BaseMapTest
+    extends DatabindTestUtil
 {
     static abstract class AbstractBean { }
-    
+
     static class StringBean extends AbstractBean { // ha, punny!
         public String name;
 
@@ -36,9 +45,9 @@ public class TestDefaultForObject
         MAYBE(true), PROBABLY_NOT(false);
 
         private boolean state;
-    	
+
         private ComplexChoice(boolean b) { state = b; }
-    	
+
         @Override
         public String toString() { return String.valueOf(state); }
     }
@@ -46,7 +55,7 @@ public class TestDefaultForObject
     static class PolymorphicType {
         public String foo;
         public Object bar;
-        
+
         public PolymorphicType() { }
         public PolymorphicType(String foo, int bar) {
             this.foo = foo;
@@ -57,7 +66,7 @@ public class TestDefaultForObject
     final static class BeanHolder
     {
         public AbstractBean bean;
-        
+
         public BeanHolder() { }
         public BeanHolder(AbstractBean b) { bean = b; }
     }
@@ -78,22 +87,34 @@ public class TestDefaultForObject
         public String subject;
     }
 
-    static public class DomainBeanWrapper {
+    static class DomainBeanWrapper {
         public String name;
         public Object myBean;
     }
 
+    @SuppressWarnings("serial")
+    static class BlockAllPTV extends PolymorphicTypeValidator.Base
+    {
+        @Override
+        public Validity validateBaseType(MapperConfig<?> config, JavaType baseType) {
+            return Validity.DENIED;
+        }
+    }
+
     /*
     /**********************************************************
     /* Unit tests
     /**********************************************************
      */
 
+    private final ObjectMapper MAPPER = newJsonMapper();
+
     /**
      * Unit test that verifies that a bean is stored with type information,
      * when declared type is <code>Object.class</code> (since it is within
      * Object[]), and default type information is enabled.
      */
+    @Test
     public void testBeanAsObject() throws Exception
     {
         ObjectMapper m = JsonMapper.builder()
@@ -103,7 +124,7 @@ public class TestDefaultForObject
         String str = m.writeValueAsString(new Object[] { new StringBean("abc") });
 
         _verifySerializationAsMap(str);
-        
+
         // Ok: serialization seems to work as expected. Now deserialize:
         Object ob = m.readValue(str, Object[].class);
         assertNotNull(ob);
@@ -114,6 +135,7 @@ public class TestDefaultForObject
     }
 
     // with 2.5, another test to check that "as-property" is valid option
+    @Test
     public void testBeanAsObjectUsingAsProperty() throws Exception
     {
         ObjectMapper m = JsonMapper.builder()
@@ -123,18 +145,37 @@ public class TestDefaultForObject
                 .build();
         // note: need to wrap, to get declared as Object
         String json = m.writeValueAsString(new StringBean("abc"));
-        
+
         // Ok: serialization seems to work as expected. Now deserialize:
         Object result = m.readValue(json, Object.class);
         assertNotNull(result);
         assertEquals(StringBean.class, result.getClass());
         assertEquals("abc", ((StringBean) result).name);
     }
-    
+
+    // [databind#2840]: ensure "as-property" uses PTV passed
+    @Test
+    public void testAsPropertyWithPTV() throws Exception {
+        ObjectMapper m = JsonMapper.builder()
+                .activateDefaultTypingAsProperty(new BlockAllPTV(),
+                        ObjectMapper.DefaultTyping.NON_FINAL,
+                        "@classy")
+                .build();
+        String json = m.writeValueAsString(new StringBean("abc"));
+        try {
+            /*Object result =*/ m.readValue(json, Object.class);
+            fail("Should not pass");
+        } catch (InvalidDefinitionException e) {
+            verifyException(e, "Configured `PolymorphicTypeValidator`");
+            verifyException(e, "denied resolution of all subtypes of ");
+        }
+    }
+
     /**
      * Unit test that verifies that an abstract bean is stored with type information
      * if default type information is enabled for non-concrete types.
      */
+    @Test
     public void testAbstractBean() throws Exception
     {
         // First, let's verify that we'd fail without enabling default type info
@@ -144,9 +185,8 @@ public class TestDefaultForObject
         try {
             m.readValue(serial, AbstractBean[].class);
             fail("Should have failed");
-        } catch (JsonMappingException e) {
-            // let's use whatever is currently thrown exception... may change tho
-            verifyException(e, "cannot construct");
+        } catch (InvalidDefinitionException e) {
+            verifyException(e, "cannot construct instance of");
         }
         // and then that we will succeed with default type info
         m = JsonMapper.builder()
@@ -164,6 +204,7 @@ public class TestDefaultForObject
      * Unit test to verify that type information is included for
      * all non-final types, if default typing suitably configured
      */
+    @Test
     public void testNonFinalBean() throws Exception
     {
         ObjectMapper m = JsonMapper.builder()
@@ -182,6 +223,7 @@ public class TestDefaultForObject
             m.writeValueAsString(bean));
     }
 
+    @Test
     public void testNullValue() throws Exception
     {
         ObjectMapper m = JsonMapper.builder()
@@ -195,15 +237,16 @@ public class TestDefaultForObject
         assertNotNull(result);
         assertNull(result.bean);
     }
-    
+
+    @Test
     public void testEnumAsObject() throws Exception
     {
         // wrapping to be declared as object
         Object[] input = new Object[] { Choice.YES };
         Object[] input2 = new Object[] { ComplexChoice.MAYBE};
         // first, without type info:
-        assertEquals("[\"YES\"]", serializeAsString(input));
-        assertEquals("[\"MAYBE\"]", serializeAsString(input2));
+        assertEquals("[\"YES\"]", MAPPER.writeValueAsString(input));
+        assertEquals("[\"MAYBE\"]", MAPPER.writeValueAsString(input2));
 
         // and then with it
         ObjectMapper m = JsonMapper.builder()
@@ -227,6 +270,7 @@ public class TestDefaultForObject
     }
 
     @SuppressWarnings("unchecked")
+    @Test
     public void testEnumSet() throws Exception
     {
         EnumSet<Choice> set = EnumSet.of(Choice.NO);
@@ -246,6 +290,7 @@ public class TestDefaultForObject
     }
 
     @SuppressWarnings("unchecked")
+    @Test
     public void testEnumMap() throws Exception
     {
         EnumMap<Choice,String> map = new EnumMap<Choice,String>(Choice.class);
@@ -265,6 +310,7 @@ public class TestDefaultForObject
         assertNull(map2.get(Choice.YES));
     }
 
+    @Test
     public void testJackson311() throws Exception
     {
         ObjectMapper mapper = JsonMapper.builder()
@@ -278,6 +324,7 @@ public class TestDefaultForObject
     }
 
     // Also, let's ensure TokenBuffer gets properly handled
+    @Test
     public void testTokenBuffer() throws Exception
     {
         ObjectMapper mapper = JsonMapper.builder()
@@ -335,6 +382,7 @@ public class TestDefaultForObject
         buf.close();
     }
 
+    @Test
     public void testIssue352() throws Exception
     {
         ObjectMapper mapper = JsonMapper.builder()
@@ -352,9 +400,10 @@ public class TestDefaultForObject
         assertNotNull(result);
         assertNotNull(wrapper.myBean);
         assertSame(DiscussBean.class, wrapper.myBean.getClass());
-    }    
+    }
 
     // Test to ensure we can also use "As.PROPERTY" inclusion and custom property name
+    @Test
     public void testFeature432() throws Exception
     {
         ObjectMapper mapper = JsonMapper.builder()
@@ -365,6 +414,7 @@ public class TestDefaultForObject
         assertEquals("{\"bean\":{\"*CLASS*\":\"com.fasterxml.jackson.databind.jsontype.deftyping.TestDefaultForObject$StringBean\",\"name\":\"punny\"}}", json);
     }
 
+    @Test
     public void testNoGoWithExternalProperty() throws Exception
     {
         try {
@@ -380,21 +430,28 @@ public class TestDefaultForObject
     }
 
     // [databind#2349]
-    public void testWithFinalClass() throws Exception
+    @Test
+    public void testWithFinalClass_NonFinal() throws Exception
     {
         // First: type info NOT included
         ObjectMapper mapper = JsonMapper.builder()
                 .activateDefaultTyping(NoCheckSubTypeValidator.instance,
                         ObjectMapper.DefaultTyping.NON_FINAL)
                 .build();
-        assertEquals(aposToQuotes("{'name':'abc'}"),
+        assertEquals(a2q("{'name':'abc'}"),
                 mapper.writeValueAsString(new FinalStringBean("abc")));
+    }
 
-        mapper = JsonMapper.builder()
+    // [databind#2349]
+    @SuppressWarnings("deprecation")
+    @Test
+    public void testWithFinalClass_Everything() throws Exception
+    {
+        ObjectMapper mapper = JsonMapper.builder()
                 .activateDefaultTyping(NoCheckSubTypeValidator.instance,
                         ObjectMapper.DefaultTyping.EVERYTHING)
                 .build();
-        assertEquals(aposToQuotes("['"+FinalStringBean.class.getName()+"',{'name':'abc'}]"),
+        assertEquals(a2q("['"+FinalStringBean.class.getName()+"',{'name':'abc'}]"),
                 mapper.writeValueAsString(new FinalStringBean("abc")));
     }
 
@@ -403,7 +460,7 @@ public class TestDefaultForObject
     /* Helper methods
     /**********************************************************
      */
-    
+
     @SuppressWarnings("unchecked")
     private void _verifySerializationAsMap(String str) throws Exception
     {
