@@ -3,12 +3,18 @@ package com.fasterxml.jackson.databind.mixins;
 import java.io.*;
 import java.util.*;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.json.JsonMapper;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
+
+import static org.junit.jupiter.api.Assertions.*;
 
 public class TestMixinSerForClass
-    extends BaseMapTest
+    extends DatabindTestUtil
 {
     @JsonInclude(JsonInclude.Include.ALWAYS)
     static class BaseClass
@@ -61,24 +67,37 @@ public class TestMixinSerForClass
         static final String FILTER_NAME = "PortletRenderExecutionEventFilter";
     }
 
+    // [databind#3035]
+    static class Bean3035 {
+        public int getA() { return 42; }
+    }
+
+    static class Rename3035Mixin extends Bean3035 {
+        @JsonProperty("b")
+        @Override
+        public int getA() { return super.getA(); }
+    }
+
     /*
-    /**********************************************************
-    /( Unit tests
-    /**********************************************************
+    /**********************************************************************
+    /* Test methods
+    /**********************************************************************
      */
 
+    private final ObjectMapper MAPPER = newJsonMapper();
+
+    @Test
     public void testClassMixInsTopLevel() throws IOException
     {
-        ObjectMapper mapper = new ObjectMapper();
         Map<String,Object> result;
 
         // first: with no mix-ins:
-        result = writeAndMap(mapper, new LeafClass("abc"));
+        result = writeAndMap(MAPPER, new LeafClass("abc"));
         assertEquals(1, result.size());
         assertEquals("abc", result.get("a"));
 
         // then with top-level override
-        mapper = new ObjectMapper();
+        ObjectMapper mapper = new ObjectMapper();
         mapper.addMixIn(LeafClass.class, MixIn.class);
         result = writeAndMap(mapper, new LeafClass("abc"));
         assertEquals(2, result.size());
@@ -93,21 +112,21 @@ public class TestMixinSerForClass
         assertEquals("abc", result.get("a"));
     }
 
+    @Test
     public void testClassMixInsMidLevel() throws IOException
     {
-        ObjectMapper mapper = new ObjectMapper();
         Map<String,Object> result;
         LeafClass bean = new LeafClass("xyz");
         bean._c = "c2";
 
         // with no mix-ins first...
-        result = writeAndMap(mapper, bean);
+        result = writeAndMap(MAPPER, bean);
         assertEquals(2, result.size());
         assertEquals("xyz", result.get("a"));
         assertEquals("c2", result.get("c"));
 
         // then with working mid-level override, which effectively suppresses 'a'
-        mapper = new ObjectMapper();
+        ObjectMapper mapper = new ObjectMapper();
         mapper.addMixIn(BaseClass.class, MixInAutoDetect.class);
         result = writeAndMap(mapper, bean);
         assertEquals(1, result.size());
@@ -123,4 +142,27 @@ public class TestMixinSerForClass
         assertEquals(1, result.size());
         assertEquals("c2", result.get("c"));
     }
+
+    // [databind#3035]: ability to remove mix-ins:
+    @Test
+    public void testClassMixInRemoval() throws Exception
+    {
+        // First, no mix-in
+        assertEquals(a2q("{'a':42}"), MAPPER.writeValueAsString(new Bean3035()));
+
+        // then with mix-in
+        assertEquals(a2q("{'b':42}"),
+                JsonMapper.builder()
+                    .addMixIn(Bean3035.class, Rename3035Mixin.class)
+                    .build()
+                    .writeValueAsString(new Bean3035()));
+
+        // and then with a convoluted example to verify removal can work:
+        assertEquals(a2q("{'a':42}"),
+                JsonMapper.builder()
+                    .addMixIn(Bean3035.class, Rename3035Mixin.class)
+                    .removeMixIn(Bean3035.class)
+                    .build()
+                    .writeValueAsString(new Bean3035()));
+    }
 }
