@@ -3,18 +3,23 @@ package com.fasterxml.jackson.databind.module;
 import java.io.IOException;
 import java.lang.reflect.Type;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.core.*;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
-import com.fasterxml.jackson.databind.module.SimpleDeserializers;
-import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.databind.ser.Serializers;
+import com.fasterxml.jackson.databind.testutil.DatabindTestUtil;
 import com.fasterxml.jackson.databind.type.*;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 @SuppressWarnings("serial")
-public class TestTypeModifiers extends BaseMapTest
+public class TestTypeModifiers extends DatabindTestUtil
 {
     /*
     /**********************************************************
@@ -59,7 +64,6 @@ public class TestTypeModifiers extends BaseMapTest
                 @Override
                 public JsonDeserializer<?> findCollectionLikeDeserializer(CollectionLikeType type, DeserializationConfig config,
                         BeanDescription beanDesc, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
-                    throws JsonMappingException
                 {
                     if (CollectionMarker.class.isAssignableFrom(type.getRawClass())) {
                         return new MyCollectionDeserializer();
@@ -70,7 +74,6 @@ public class TestTypeModifiers extends BaseMapTest
                 public JsonDeserializer<?> findMapLikeDeserializer(MapLikeType type, DeserializationConfig config,
                         BeanDescription beanDesc, KeyDeserializer keyDeserializer,
                         TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
-                    throws JsonMappingException
                 {
                     if (MapMarker.class.isAssignableFrom(type.getRawClass())) {
                         return new MyMapDeserializer();
@@ -88,7 +91,7 @@ public class TestTypeModifiers extends BaseMapTest
             jgen.writeString("xxx:"+value);
         }
     }
-    
+
     interface MapMarker<K,V> {
         public K getKey();
         public V getValue();
@@ -131,12 +134,12 @@ public class TestTypeModifiers extends BaseMapTest
     {
         protected final JsonSerializer<Object> _keySerializer;
         protected final JsonSerializer<Object> _valueSerializer;
-        
+
         public MyMapSerializer(JsonSerializer<Object> keySer, JsonSerializer<Object> valueSer) {
             _keySerializer = keySer;
             _valueSerializer = valueSer;
         }
-        
+
         @Override
         public void serialize(MapMarker<?,?> value, JsonGenerator jgen, SerializerProvider provider) throws IOException {
             jgen.writeStartObject();
@@ -157,14 +160,14 @@ public class TestTypeModifiers extends BaseMapTest
     {
         @Override
         public MapMarker<?,?> deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {
-            if (jp.getCurrentToken() != JsonToken.START_OBJECT) throw new IOException("Wrong token: "+jp.getCurrentToken());
-            if (jp.nextToken() != JsonToken.FIELD_NAME) throw new IOException("Wrong token: "+jp.getCurrentToken());
-            String key = jp.getCurrentName();
-            if (jp.nextToken() != JsonToken.VALUE_NUMBER_INT) throw new IOException("Wrong token: "+jp.getCurrentToken());
+            if (jp.currentToken() != JsonToken.START_OBJECT) throw new IOException("Wrong token: "+jp.currentToken());
+            if (jp.nextToken() != JsonToken.FIELD_NAME) throw new IOException("Wrong token: "+jp.currentToken());
+            String key = jp.currentName();
+            if (jp.nextToken() != JsonToken.VALUE_NUMBER_INT) throw new IOException("Wrong token: "+jp.currentToken());
             int value = jp.getIntValue();
-            if (jp.nextToken() != JsonToken.END_OBJECT) throw new IOException("Wrong token: "+jp.getCurrentToken());
+            if (jp.nextToken() != JsonToken.END_OBJECT) throw new IOException("Wrong token: "+jp.currentToken());
             return new MyMapLikeType(key, value);
-        }        
+        }
     }
 
     static class MyCollectionSerializer extends JsonSerializer<MyCollectionLikeType>
@@ -180,12 +183,12 @@ public class TestTypeModifiers extends BaseMapTest
     {
         @Override
         public MyCollectionLikeType deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {
-            if (jp.getCurrentToken() != JsonToken.START_ARRAY) throw new IOException("Wrong token: "+jp.getCurrentToken());
-            if (jp.nextToken() != JsonToken.VALUE_NUMBER_INT) throw new IOException("Wrong token: "+jp.getCurrentToken());
+            if (jp.currentToken() != JsonToken.START_ARRAY) throw new IOException("Wrong token: "+jp.currentToken());
+            if (jp.nextToken() != JsonToken.VALUE_NUMBER_INT) throw new IOException("Wrong token: "+jp.currentToken());
             int value = jp.getIntValue();
-            if (jp.nextToken() != JsonToken.END_ARRAY) throw new IOException("Wrong token: "+jp.getCurrentToken());
+            if (jp.nextToken() != JsonToken.END_ARRAY) throw new IOException("Wrong token: "+jp.currentToken());
             return new MyCollectionLikeType(value);
-        }        
+        }
     }
 
     static class MyTypeModifier extends TypeModifier
@@ -205,22 +208,24 @@ public class TestTypeModifiers extends BaseMapTest
             return type;
         }
     }
-    
+
     /*
     /**********************************************************
     /* Unit tests
     /**********************************************************
      */
 
+    private final ObjectMapper MAPPER_WITH_MODIFIER = JsonMapper.builder()
+            .typeFactory(TypeFactory.defaultInstance().withModifier(new MyTypeModifier()))
+            .build();
+
     /**
      * Basic test for ensuring that we can get "xxx-like" types recognized.
      */
+    @Test
     public void testMapLikeTypeConstruction() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
-        mapper.setTypeFactory(mapper.getTypeFactory().withModifier(new MyTypeModifier()));
-
-        JavaType type = mapper.constructType(MyMapLikeType.class);
+        JavaType type = MAPPER_WITH_MODIFIER.constructType(MyMapLikeType.class);
         assertTrue(type.isMapLikeType());
         // also, must have resolved type info
         JavaType param = ((MapLikeType) type).getKeyType();
@@ -231,8 +236,25 @@ public class TestTypeModifiers extends BaseMapTest
         assertSame(Integer.class, param.getRawClass());
     }
 
+    @Test
+    public void testMapLikeTypeViaParametric() throws Exception
+    {
+        // [databind#2796]: should refine with another call too
+        JavaType type = MAPPER_WITH_MODIFIER.getTypeFactory().constructParametricType(MapMarker.class,
+                new Class<?>[] { String.class, Double.class });
+        assertTrue(type.isMapLikeType());
+        JavaType param = ((MapLikeType) type).getKeyType();
+        assertNotNull(param);
+        assertSame(String.class, param.getRawClass());
+
+        param = ((MapLikeType) type).getContentType();
+        assertNotNull(param);
+        assertSame(Double.class, param.getRawClass());
+    }
+
     // [databind#2395] Can trigger problem this way too
-    // NOTE: oddly enough, seems to ONLY fail 
+    // NOTE: oddly enough, seems to ONLY fail
+    @Test
     public void testTypeResolutionForRecursive() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -245,18 +267,17 @@ public class TestTypeModifiers extends BaseMapTest
         assertNotNull(mapper.readTree("{}"));
     }
 
+    @Test
     public void testCollectionLikeTypeConstruction() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
-        mapper.setTypeFactory(mapper.getTypeFactory().withModifier(new MyTypeModifier()));
-
-        JavaType type = mapper.constructType(MyCollectionLikeType.class);
+        JavaType type = MAPPER_WITH_MODIFIER.constructType(MyCollectionLikeType.class);
         assertTrue(type.isCollectionLikeType());
         JavaType param = ((CollectionLikeType) type).getContentType();
         assertNotNull(param);
         assertSame(Integer.class, param.getRawClass());
     }
 
+    @Test
     public void testCollectionLikeSerialization() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -265,6 +286,7 @@ public class TestTypeModifiers extends BaseMapTest
         assertEquals("[19]", mapper.writeValueAsString(new MyCollectionLikeType(19)));
     }
 
+    @Test
     public void testMapLikeSerialization() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -275,6 +297,7 @@ public class TestTypeModifiers extends BaseMapTest
     }
 
 
+    @Test
     public void testCollectionLikeDeserialization() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
@@ -286,6 +309,7 @@ public class TestTypeModifiers extends BaseMapTest
         assertEquals(Integer.valueOf(13), result.getValue());
     }
 
+    @Test
     public void testMapLikeDeserialization() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
