@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.node;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.*;
+import java.util.stream.Collectors;
 
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonInclude;
@@ -29,16 +30,9 @@ public class ObjectNodeTest
         public DataImpl(JsonNode n) {
             root = n;
         }
-        
+
         @JsonValue
         public JsonNode value() { return root; }
-        
-        /*
-        public Wrapper(ObjectNode n) { root = n; }
-        
-        @JsonValue
-        public ObjectNode value() { return root; }
-        */
     }
 
     static class ObNodeWrapper {
@@ -69,7 +63,7 @@ public class ObjectNodeTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = sharedMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     public void testSimpleObject() throws Exception
     {
@@ -87,10 +81,10 @@ public class ObjectNodeTest
         assertFalse(root.isBoolean());
         assertFalse(root.isTextual());
         assertFalse(root.isNumber());
-        assertFalse(root.canConvertToInt()); 
-        assertFalse(root.canConvertToLong()); 
-        assertFalse(root.canConvertToExactIntegral()); 
-        
+        assertFalse(root.canConvertToInt());
+        assertFalse(root.canConvertToLong());
+        assertFalse(root.canConvertToExactIntegral());
+
         Iterator<JsonNode> it = root.iterator();
         assertNotNull(it);
         assertTrue(it.hasNext());
@@ -132,13 +126,13 @@ public class ObjectNodeTest
         Data w = MAPPER.readValue("{}", Data.class);
         assertNotNull(w);
     }
-    
+
     public void testBasics()
     {
         ObjectNode n = new ObjectNode(JsonNodeFactory.instance);
         assertStandardEquals(n);
         assertTrue(n.isEmpty());
-        
+
         assertFalse(n.elements().hasNext());
         assertFalse(n.fields().hasNext());
         assertFalse(n.fieldNames().hasNext());
@@ -147,7 +141,7 @@ public class ObjectNodeTest
 
         TextNode text = TextNode.valueOf("x");
         assertSame(n, n.set("a", text));
-        
+
         assertEquals(1, n.size());
         assertTrue(n.elements().hasNext());
         assertTrue(n.fields().hasNext());
@@ -168,7 +162,7 @@ public class ObjectNodeTest
         n2.put("b", 13);
         assertFalse(n.equals(n2));
         n.setAll(n2);
-        
+
         assertEquals(2, n.size());
         n.set("null", (JsonNode)null);
         assertEquals(3, n.size());
@@ -299,11 +293,11 @@ public class ObjectNodeTest
         assertEquals("c", ob.get("c").textValue());
     }
 
-    public void testValidWith() throws Exception
+    public void testValidWithObject() throws Exception
     {
         ObjectNode root = MAPPER.createObjectNode();
         assertEquals("{}", MAPPER.writeValueAsString(root));
-        JsonNode child = root.with("prop");
+        JsonNode child = root.withObject("/prop");
         assertTrue(child instanceof ObjectNode);
         assertEquals("{\"prop\":{}}", MAPPER.writeValueAsString(root));
     }
@@ -317,34 +311,59 @@ public class ObjectNodeTest
         assertEquals("{\"arr\":[]}", MAPPER.writeValueAsString(root));
     }
 
-    public void testInvalidWith() throws Exception
+    public void testInvalidWithObject() throws Exception
     {
         JsonNode root = MAPPER.createArrayNode();
         try { // should not work for non-ObjectNode nodes:
-            root.with("prop");
+            root.withObject("/prop");
             fail("Expected exception");
         } catch (UnsupportedOperationException e) {
-            verifyException(e, "not of type ObjectNode");
+            verifyException(e, "`withObject(String)` not implemented");
+            verifyException(e, "ArrayNode");
         }
         // also: should fail of we already have non-object property
         ObjectNode root2 = MAPPER.createObjectNode();
         root2.put("prop", 13);
         try { // should not work for non-ObjectNode nodes:
-            root2.with("prop");
+            root2.withObject("/prop");
             fail("Expected exception");
         } catch (UnsupportedOperationException e) {
-            verifyException(e, "has value that is not");
+            verifyException(e, "Cannot replace `JsonNode` of type ");
+            verifyException(e, "IntNode");
         }
     }
 
     public void testInvalidWithArray() throws Exception
+    {
+        JsonNode root = MAPPER.createArrayNode();
+        try { // should not work for non-ObjectNode nodes:
+            root.withArray("/prop");
+            fail("Expected exception");
+        } catch (UnsupportedOperationException e) {
+            verifyException(e, "Cannot replace context node (of type");
+            verifyException(e, "ArrayNode");
+        }
+        // also: should fail of we already have non-Array property
+        ObjectNode root2 = MAPPER.createObjectNode();
+        root2.put("prop", 13);
+        try { // should not work for non-ObjectNode nodes:
+            root2.withArray("/prop");
+            fail("Expected exception");
+        } catch (UnsupportedOperationException e) {
+            verifyException(e, "Cannot replace `JsonNode` of type ");
+            verifyException(e, "IntNode");
+        }
+    }
+
+    // Test for pre-2.14 behavior for "simple property"
+    public void testInvalidWithArrayLegacy() throws Exception
     {
         JsonNode root = MAPPER.createArrayNode();
         try { // should not work for non-ObjectNode nodes:
             root.withArray("prop");
             fail("Expected exception");
         } catch (UnsupportedOperationException e) {
-            verifyException(e, "not of type ObjectNode");
+            verifyException(e, "not of type `ObjectNode`");
         }
         // also: should fail of we already have non-Array property
         ObjectNode root2 = MAPPER.createObjectNode();
@@ -398,12 +417,12 @@ public class ObjectNodeTest
     public void testFailOnDupKeys() throws Exception
     {
         final String DUP_JSON = "{ \"a\":1, \"a\":2 }";
-        
+
         // first: verify defaults:
         assertFalse(MAPPER.isEnabled(DeserializationFeature.FAIL_ON_READING_DUP_TREE_KEY));
         ObjectNode root = (ObjectNode) MAPPER.readTree(DUP_JSON);
         assertEquals(2, root.path("a").asInt());
-        
+
         // and then enable checks:
         try {
             MAPPER.reader(DeserializationFeature.FAIL_ON_READING_DUP_TREE_KEY).readTree(DUP_JSON);
@@ -434,7 +453,7 @@ public class ObjectNodeTest
         ObjectNode ob2 = MAPPER.createObjectNode();
 
         // same contents, different insertion order; should not matter
-        
+
         ob1.put("a", 1);
         ob1.put("b", 2);
         ob1.put("c", 3);
@@ -484,12 +503,30 @@ public class ObjectNodeTest
 
     public void testSimpleMismatch() throws Exception
     {
-        ObjectMapper mapper = objectMapper();
         try {
-            mapper.readValue("[ 1, 2, 3 ]", ObjectNode.class);
+            MAPPER.readValue("[ 1, 2, 3 ]", ObjectNode.class);
             fail("Should not pass");
         } catch (MismatchedInputException e) {
             verifyException(e, "from Array value (token `JsonToken.START_ARRAY`)");
         }
     }
+
+    // [databind#3809]
+    public void testPropertiesTraversal() throws Exception
+    {
+        // Nothing to traverse for other types
+        assertEquals("", _toString(MAPPER.createArrayNode()));
+        assertEquals("", _toString(MAPPER.getNodeFactory().textNode("foo")));
+
+        // But yes for ObjectNode:
+        JsonNode n = MAPPER.readTree(a2q(
+                "{ 'a':1, 'b':true,'c':'stuff'}"));
+        assertEquals("a/1,b/true,c/\"stuff\"", _toString(n));
+    }
+
+    private String _toString(JsonNode n) {
+        return n.properties().stream()
+                .map(e -> e.getKey() + "/" + e.getValue())
+                .collect(Collectors.joining(","));
+    }
 }
