@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.struct;
 import com.fasterxml.jackson.annotation.*;
 
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 
 /**
  * Unit tests for verifying that basic {@link JsonUnwrapped} annotation
@@ -33,7 +34,7 @@ public class TestUnwrapped extends BaseMapTest
             this.y = y;
         }
     }
-    
+
     static class DeepUnwrapping
     {
         @JsonUnwrapped
@@ -44,7 +45,7 @@ public class TestUnwrapped extends BaseMapTest
             unwrapped = new Unwrapping(str, x, y);
         }
     }
-    
+
     static class UnwrappingWithCreator {
         public String name;
 
@@ -92,9 +93,8 @@ public class TestUnwrapped extends BaseMapTest
     }
 
     static class Outer {
-        // @JsonProperty
         @JsonUnwrapped
-        private Inner inner;
+        Inner inner;
     }
 
     // [databind#1493]: case-insensitive handling
@@ -107,7 +107,7 @@ public class TestUnwrapped extends BaseMapTest
         public String street;
         public String addon;
         public String zip;
-        public String town;    
+        public String town;
         public String country;
     }
 
@@ -148,13 +148,15 @@ public class TestUnwrapped extends BaseMapTest
     private final ObjectMapper MAPPER = new ObjectMapper();
 
     public void testSimpleUnwrappingSerialize() throws Exception {
-        assertEquals("{\"name\":\"Tatu\",\"x\":1,\"y\":2}",
-                MAPPER.writeValueAsString(new Unwrapping("Tatu", 1, 2)));
+        JsonMapper mapper = JsonMapper.builder().enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY).build();
+        assertEquals("{\"x\":1,\"y\":2,\"name\":\"Tatu\"}",
+                mapper.writeValueAsString(new Unwrapping("Tatu", 1, 2)));
     }
 
     public void testDeepUnwrappingSerialize() throws Exception {
-        assertEquals("{\"name\":\"Tatu\",\"x\":1,\"y\":2}",
-                MAPPER.writeValueAsString(new DeepUnwrapping("Tatu", 1, 2)));
+        JsonMapper mapper = JsonMapper.builder().enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY).build();
+        assertEquals("{\"x\":1,\"y\":2,\"name\":\"Tatu\"}",
+                mapper.writeValueAsString(new DeepUnwrapping("Tatu", 1, 2)));
     }
 
     /*
