@@ -1,8 +1,11 @@
 package com.fasterxml.jackson.databind.struct;
 
 import com.fasterxml.jackson.annotation.JsonUnwrapped;
+import com.fasterxml.jackson.annotation.JsonPropertyOrder;
 import com.fasterxml.jackson.databind.BaseMapTest;
+import com.fasterxml.jackson.databind.MapperFeature;
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 
 public class TestUnwrappedWithPrefix extends BaseMapTest
 {
@@ -53,7 +56,7 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
             location = new Location(x, y);
         }
     }
-    
+
     static class DeepPrefixUnwrap
     {
         @JsonUnwrapped(prefix="u.")
@@ -66,12 +69,12 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
     }
 
     // Let's actually test hierarchic names with unwrapping bit more:
-    
+    @JsonPropertyOrder({ "general", "misc" })
     static class ConfigRoot
     {
         @JsonUnwrapped(prefix="general.")
         public ConfigGeneral general = new ConfigGeneral();
-        
+
         @JsonUnwrapped(prefix="misc.")
         public ConfigMisc misc = new ConfigMisc();
 
@@ -87,12 +90,12 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
     {
         @JsonUnwrapped
         public ConfigGeneral general = new ConfigGeneral();
-        
+
         @JsonUnwrapped(prefix="misc.")
         public ConfigMisc misc = new ConfigMisc();
 
         public int id;
-        
+
         public ConfigAlternate() { }
         public ConfigAlternate(int id, String name, int value)
         {
@@ -106,7 +109,7 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
     {
         @JsonUnwrapped(prefix="names.")
         public ConfigNames names = new ConfigNames();
-        
+
         public ConfigGeneral() { }
         public ConfigGeneral(String name) {
             names.name = name;
@@ -148,14 +151,16 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
 
     public void testPrefixedUnwrappingSerialize() throws Exception
     {
-        assertEquals("{\"name\":\"Tatu\",\"_x\":1,\"_y\":2}",
-                MAPPER.writeValueAsString(new PrefixUnwrap("Tatu", 1, 2)));
+        JsonMapper mapper = JsonMapper.builder().enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY).build();
+        assertEquals("{\"_x\":1,\"_y\":2,\"name\":\"Tatu\"}",
+                mapper.writeValueAsString(new PrefixUnwrap("Tatu", 1, 2)));
     }
 
     public void testDeepPrefixedUnwrappingSerialize() throws Exception
     {
-        String json = MAPPER.writeValueAsString(new DeepPrefixUnwrap("Bubba", 1, 1));
-        assertEquals("{\"u.name\":\"Bubba\",\"u._x\":1,\"u._y\":1}", json);
+        JsonMapper mapper = JsonMapper.builder().enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY).build();
+        String json = mapper.writeValueAsString(new DeepPrefixUnwrap("Bubba", 1, 1));
+        assertEquals("{\"u._x\":1,\"u._y\":1,\"u.name\":\"Bubba\"}", json);
     }
 
     public void testHierarchicConfigSerialize() throws Exception
@@ -179,7 +184,7 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
         assertEquals(4, bean.location.x);
         assertEquals(7, bean.location.y);
     }
-    
+
     public void testDeepPrefixedUnwrappingDeserialize() throws Exception
     {
         DeepPrefixUnwrap bean = MAPPER.readValue("{\"u.name\":\"Bubba\",\"u._x\":2,\"u._y\":3}",
@@ -190,7 +195,7 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
         assertEquals(3, bean.unwrapped.location.y);
         assertEquals("Bubba", bean.unwrapped.name);
     }
-    
+
     public void testHierarchicConfigDeserialize() throws Exception
     {
         ConfigRoot root = MAPPER.readValue("{\"general.names.name\":\"Bob\",\"misc.value\":3}",
@@ -240,7 +245,7 @@ public class TestUnwrappedWithPrefix extends BaseMapTest
 
         assertNotNull(output.c1.sc1);
         assertNotNull(output.c2.sc1);
-        
+
         assertEquals("a", output.c1.sc1.value);
         assertEquals("b", output.c2.sc1.value);
     }
