@@ -20,13 +20,13 @@ public class DelegatingCreatorWithAbstractProp2252Test extends BaseMapTest
     }
 
     // NOTE! Abstract POJO is fine, only Map/Collection causes issues for some reason
-    
+
 //    static abstract class MyAbstractMap extends AbstractMap<String, Object> { }
 
     @SuppressWarnings("serial")
     static abstract class MyAbstractList extends ArrayList<String> { }
 
-    private final ObjectMapper MAPPER = newObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     // loosely based on [databind#2251], in which delegating creator is used, but
     // theoretically necessary type for setter can cause issues -- shouldn't, as no
